@@ -56,27 +56,18 @@ DO_COMMAND(do_action)
 	{
 		if (show_node_with_wild(ses, left, LIST_ACTION) == FALSE) 
 		{
-			if (show_message(ses, LIST_ACTION))
-			{
-				tintin_puts2("#That action is not defined.", ses);
-			}
+			show_message(ses, LIST_ACTION, "#ACTION: NO MATCH(ES) FOUND FOR {%s}.", left);
 		}
 	}
 	else
 	{
 		updatenode_list(ses, left, right, pr, LIST_ACTION);
 
-		if (show_message(ses, LIST_ACTION)) 
-		{
-			tintin_printf2(ses, "#Ok. {%s} now triggers {%s} @ {%s}", left, right, pr);
-		}
+		show_message(ses, LIST_ACTION, "#OK. {%s} NOW TRIGGERS {%s} @ {%s}.", left, right, pr);
 	}
 	return ses;
 }
 
-/*
-	the #unaction command
-*/
 
 DO_COMMAND(do_unaction)
 {
@@ -87,21 +78,20 @@ DO_COMMAND(do_unaction)
 
 	root = ses->list[LIST_ACTION];
 
-	arg = get_arg_in_braces(arg, left, 1);
+	arg = get_arg_in_braces(arg, left, TRUE);
 
 	while ((node = search_node_with_wild(root, left))) 
 	{
-		if (show_message(ses, LIST_ACTION))
-		{
-			tintin_printf2(ses, "#Ok. {%s} is no longer a trigger.", node->left);
-		}
+		show_message(ses, LIST_ACTION, "#OK. {%s} IS NO LONGER A TRIGGER.", node->left);
+
 		deletenode_list(ses, node, LIST_ACTION);
+
 		flag = TRUE;
 	}
 
-	if (!flag && show_message(ses, LIST_ACTION))
+	if (!flag)
 	{
-		tintin_printf(ses, "#No match(es) found for {%s}", left);
+		show_message(ses, LIST_ACTION, "#ACTION: NO MATCH(ES) FOUND FOR {%s}.", left);
 	}
 	return ses;
 }
@@ -126,7 +116,7 @@ void check_all_actions(const char *original, char *line, struct session *ses)
 
 			if (HAS_BIT(ses->list[LIST_ACTION]->flags, LIST_FLAG_DEBUG))
 			{
-				tintin_printf2(ses, "[ACTION: %s]", buffer);
+				show_message(ses, LIST_ACTION, "#ACTION DEBUG: %s", buffer);
 			}
 			parse_input(buffer, ses);
 
@@ -135,32 +125,3 @@ void check_all_actions(const char *original, char *line, struct session *ses)
 	}
 	return;
 }
-
-
-int check_postsub_actions(const char *original, char *line, struct session *ses)
-{
-	struct listnode *node;
-	struct listroot *root;
-
-	root = ses->list[LIST_ACTION];
-
-	for (node = root->update ; node ; node = root->update)
-	{
-		root->update = node->next;
-
-		if (check_one_action(line, original, node->left, ses))
-		{
-			char buffer[BUFFER_SIZE];
-
-			substitute(ses, node->right, buffer, SUB_ARG|SUB_SEC);
-
-			if (HAS_BIT(ses->list[LIST_ACTION]->flags, LIST_FLAG_DEBUG))
-			{
-				tintin_printf2(ses, "[ACTION: %s]", buffer);
-			}
-			parse_input(buffer, ses);
-			return TRUE;
-		}
-	}
-	return FALSE;
-}
