@@ -33,25 +33,24 @@ void init_buffer(struct session *ses, int size)
 {
 	int cnt;
 
+	push_call("init_buffer(%p,%p)",ses,size);
+
 	if (ses->scroll_max == size)
 	{
+		pop_call();
 		return;
 	}
 
 	if (ses->buffer)
 	{
 		cnt = ses->scroll_row;
-
+		
 		do
 		{
-			if (cnt == ses->scroll_max)
+			if (++cnt == ses->scroll_max)
 			{
 				cnt = 0;
 			}
-			else
-			{
-				cnt++;
-			}
 
 			if (ses->buffer[cnt] == NULL)
 			{
@@ -63,11 +62,19 @@ void init_buffer(struct session *ses, int size)
 		while (cnt != ses->scroll_row);
 	}
 
-	ses->buffer = calloc(size, sizeof(char *));
+	if (ses->buffer)
+	{
+		free(ses->buffer);
+	}
 
-	ses->scroll_max = size;
-	ses->scroll_row = size - 1;
+	if (size)
+	{
+		ses->buffer = calloc(size, sizeof(char *));
 
+		ses->scroll_max = size;
+		ses->scroll_row = size - 1;
+	}
+	pop_call();
 	return;
 }
 
@@ -77,6 +84,14 @@ void add_line_buffer(struct session *ses, const char *line, int more_output)
 	char linebuf[BUFFER_SIZE], linelog[BUFFER_SIZE];
 	char *pti, *pto;
 	int lines;
+	int sav_row, sav_col, cur_row, cur_col, top_row, bot_row;
+
+	sav_row = ses->sav_row;
+	sav_col = ses->sav_col;
+	cur_row = ses->cur_row;
+	cur_col = ses->cur_col;
+	top_row = ses->top_row;
+	bot_row = ses->bot_row;
 
 	if (HAS_BIT(ses->flags, SES_FLAG_SCROLLSTOP))
 	{
@@ -104,7 +119,7 @@ void add_line_buffer(struct session *ses, const char *line, int more_output)
 	{
 		while (skip_vt102_codes_non_graph(pti))
 		{
-			interpret_vt102_codes(ses, pti);
+			interpret_vt102_codes(ses, pti, FALSE);
 
 			pti += skip_vt102_codes_non_graph(pti);
 		}
@@ -134,9 +149,17 @@ void add_line_buffer(struct session *ses, const char *line, int more_output)
 		str_hash_grep(ses->buffer[ses->scroll_row], TRUE);
 	}
 
-	if (ses->logfile)
+	if (!HAS_BIT(ses->flags, SES_FLAG_LOGLEVEL))
 	{
-		logit(ses, linelog, ses->logfile);
+		if (ses->logfile)
+		{
+			logit(ses, linelog, ses->logfile);
+		}
+	}
+
+	if (gtd->chat)
+	{
+		chat_forward_session(ses, linelog);
 	}
 
 	if (ses->logline)
@@ -156,6 +179,14 @@ void add_line_buffer(struct session *ses, const char *line, int more_output)
 	{
 		ses->buffer[ses->scroll_row] = str_unhash(ses->buffer[ses->scroll_row]);
 	}
+
+	ses->sav_row = sav_row;
+	ses->sav_col = sav_col;
+	ses->cur_row = cur_row;
+	ses->cur_col = cur_col;
+	ses->top_row = top_row;
+	ses->bot_row = bot_row;
+
 	return;
 }
 
@@ -302,10 +333,14 @@ int show_buffer(struct session *ses)
 	}
 
 	scroll_size = get_scroll_size(ses);
-	scroll_add  = 0;
+	scroll_add  = 0 - ses->scroll_base;
 	scroll_tmp  = 0;
 	scroll_cnt  = ses->scroll_line;
 
+	/*
+		Find the upper limit of the buffer shown
+	*/
+
 	while (TRUE)
 	{
 		if (ses->buffer[scroll_cnt] == NULL)
@@ -315,15 +350,15 @@ int show_buffer(struct session *ses)
 
 		scroll_tmp = str_hash_lines(ses->buffer[scroll_cnt]);
 
-		if (scroll_add + scroll_tmp > ses->scroll_base + scroll_size)
+		if (scroll_add + scroll_tmp > scroll_size)
 		{
-			if (scroll_add == ses->scroll_base + scroll_size)
+			if (scroll_add == scroll_size)
 			{
 				scroll_tmp = 0;
 			}
 			else
 			{
-				scroll_tmp -= ses->scroll_base + scroll_size - scroll_add;
+				scroll_tmp -= scroll_size - scroll_add;
 			}
 			break;
 		}
@@ -340,10 +375,6 @@ int show_buffer(struct session *ses)
 		}
 	}
 
-	if (scroll_cnt == ses->scroll_line)
-	{
-		return FALSE;
-	}
 
 	if (ses->buffer[scroll_cnt] == NULL)
 	{
@@ -357,7 +388,9 @@ int show_buffer(struct session *ses)
 		SET_BIT(ses->flags, SES_FLAG_READMUD);
 	}
 
-/*	tintin_header(ses, " LINE %d OF %d ", ses->scroll_line, ses->scroll_max); */
+	/*
+		If the top line exists of multiple lines split in the middle.
+	*/
 
 	if (ses->buffer[scroll_cnt] && scroll_tmp)
 	{
@@ -396,9 +429,29 @@ int show_buffer(struct session *ses)
 		}
 		*wrap_ptr = 0;
 
-		printf("%s%s\n\r", wrap, temp_ptr);
+		if (scroll_add >= 0)
+		{
+			printf("%s%s\n\r", wrap, temp_ptr);
+		}
+		else
+		{
+			wrap_ptr = temp_ptr;
+
+			for (wrap_ptr = temp_ptr ; scroll_tmp++ < scroll_size ; temp_ptr++)
+			{
+				temp_ptr = strchr(temp_ptr, '\r');
+			}
+			*temp_ptr = 0;
+			printf("%s", wrap_ptr);
+
+			goto eof;
+		}
 	}
 
+	/*
+		Print away
+	*/
+
 	while (TRUE)
 	{
 		if (scroll_cnt == 0)
@@ -417,9 +470,9 @@ int show_buffer(struct session *ses)
 
 		scroll_tmp = word_wrap(ses, ses->buffer[scroll_cnt], temp, FALSE);
 
-		if (scroll_add - scroll_tmp < ses->scroll_base)
+		if (scroll_add - scroll_tmp < 0)
 		{
-			scroll_tmp = scroll_add - ses->scroll_base;
+			scroll_tmp = scroll_add;
 			break;
 		}
 
@@ -428,22 +481,25 @@ int show_buffer(struct session *ses)
 		printf("%s\n\r", temp);
 	}
 
+	/*
+		If the bottom line exits of multiple lines split in the middle
+	*/
+
 	if (scroll_tmp && ses->buffer[scroll_cnt])
 	{
 		temp_ptr = temp;
 
-		while (scroll_tmp)
+		while (scroll_tmp--)
 		{
 			temp_ptr = strchr(temp_ptr, '\r');
 			temp_ptr++;
-			scroll_tmp--;
 		}
 		*temp_ptr = 0;
 
 		printf("%s", temp);
 	}
 
-/*	tintin_header(ses, ""); */
+	eof:
 
 	if (IS_SPLIT(ses))
 	{
@@ -481,6 +537,10 @@ DO_COMMAND(do_buffer)
 			buffer_e();
 			break;
 
+		case 'f':
+			buffer_f(right);
+			break;
+
 		case 'w':
 			do_writebuffer(ses, right);
 			break;
@@ -489,6 +549,10 @@ DO_COMMAND(do_buffer)
 			do_hash(ses, right);
 			break;
 
+		case 'l':
+			buffer_l();
+			break;
+
 		default:
 			buffer_e();
 			break;
@@ -558,7 +622,6 @@ void buffer_d(void)
 
 	if (gtd->ses->scroll_line == -1)
 	{
-		buffer_e();
 		return;
 	}
 
@@ -566,6 +629,14 @@ void buffer_d(void)
 	scroll_cnt  = gtd->ses->scroll_line;
 	buffer_add  = gtd->ses->scroll_base;
 
+	if (scroll_size <= buffer_add)
+	{
+		gtd->ses->scroll_base = buffer_add - scroll_size;
+
+		show_buffer(gtd->ses);
+		return;
+	}
+
 	if (scroll_cnt == 0)
 	{
 		scroll_cnt = gtd->ses->scroll_max - 1;
@@ -623,7 +694,7 @@ void buffer_h(void)
 	{
 		gtd->ses->scroll_line = gtd->ses->scroll_max - 1;
 	}
-	gtd->ses->scroll_base = 1;
+	gtd->ses->scroll_base = str_hash_lines(gtd->ses->buffer[gtd->ses->scroll_line]);
 
 	buffer_d();
 }
@@ -651,3 +722,107 @@ void buffer_e(void)
 	gtd->ses->scroll_line = -1;
 	gtd->ses->scroll_base =  0;
 }
+
+void buffer_l(void)
+{
+	if (gtd->ses->buffer == NULL)
+	{
+		return;
+	}
+
+	if (gtd->ses->scroll_line == -1)
+	{
+		gtd->ses->scroll_line = gtd->ses->scroll_row + 1;
+	}
+	else
+	{
+		buffer_e();
+	}
+}
+
+void buffer_f(const char *arg)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	int scroll_cnt, grep_cnt, grep_max;
+
+	grep_cnt = grep_max = scroll_cnt = 0;
+
+	get_arg_in_braces(arg, left,  FALSE);
+
+	if (gtd->ses->buffer == NULL)
+	{
+		tintin_puts2("#BUFFER, NO SCROLL BUFFER AVAILABLE.", NULL);
+	}
+	else if (*left == 0)
+	{
+		tintin_puts2("#BUFFER, FIND WHAT?", NULL);
+	}
+	else
+	{
+		if (is_number(left))
+		{
+			arg = get_arg_in_braces(arg, left,  FALSE);
+			arg = get_arg_in_braces(arg, right, TRUE);
+
+			if (*right == 0)
+			{
+				tintin_printf2(NULL, "#BUFFER, FIND WHAT OF OCCURANCE {%s} ?", left);
+
+				return;
+			}
+			grep_max = atoi(left);
+		}
+		else
+		{
+			arg = get_arg_in_braces(arg, right, TRUE);
+		}
+
+		sprintf(left, "*%s*", right);
+
+		scroll_cnt = gtd->ses->scroll_row;
+
+		do
+		{
+			if (scroll_cnt == gtd->ses->scroll_max -1)
+			{
+				scroll_cnt = 0;
+			}
+			else
+			{
+				scroll_cnt++;
+			}
+
+			if (gtd->ses->buffer[scroll_cnt] == NULL)
+			{
+				break;
+			}
+
+			if (str_hash_grep(gtd->ses->buffer[scroll_cnt], FALSE))
+			{
+				continue;
+			}
+
+			if (regexp(left, gtd->ses->buffer[scroll_cnt]))
+			{
+				if (grep_cnt == grep_max)
+				{
+					break;
+				}
+				grep_cnt++;
+			}
+		}
+		while (scroll_cnt != gtd->ses->scroll_row);
+
+		if (gtd->ses->buffer[scroll_cnt] == NULL || scroll_cnt == gtd->ses->scroll_row)
+		{
+			tintin_puts2("#BUFFER, NO MATCHES FOUND.", NULL);
+		}
+		else
+		{
+			gtd->ses->scroll_line = scroll_cnt;
+
+			show_buffer(gtd->ses);
+		}
+	}
+	return;
+}
