@@ -48,10 +48,10 @@ DO_COMMAND(do_class)
 
 		for (node = root->f_node ; node ; node = node->next)
 		{
-			tintin_printf2(ses, "%-20s %3lld %s",
+			tintin_printf2(ses, "%-20s %3d %s",
 			node->left,
-			node->data / NODE_FLAG_MAX,
-			HAS_BIT(node->data, NODE_FLAG_CLASS) ? "OPEN" : "CLOSED");
+			count_class(ses, node),
+			HAS_BIT(node->flags, NODE_FLAG_CLASS) ? "OPEN" : "CLOSED");
 		}
 	}
 	else if (*right == 0)
@@ -95,53 +95,93 @@ DO_COMMAND(do_class)
 		}
 		else
 		{
-			updatenode_list(ses, left, right, pr, LIST_CLASS);
-
-			class_table[cnt].class(ses, left);
+			if (!searchnode_list(root, left))
+			{
+				updatenode_list(ses, left, right, pr, LIST_CLASS);
+			}
+			class_table[cnt].class(ses, left, pr);
 		}
 	}
 	return ses;
 }
 
+/*
+	Not needed with #class <name> kill
 
 DO_COMMAND(do_unclass)
 {
 	char left[BUFFER_SIZE];
 	struct listroot *root;
 	struct listnode *node;
-	int flag = FALSE;
+	int found = FALSE;
 
-	root = ses->list[LIST_TAB];
+	root = ses->list[LIST_CLASS];
 
-	arg = get_arg_in_braces(arg, left, 1);
+	arg = get_arg_in_braces(arg, left, TRUE);
 
 	while ((node = search_node_with_wild(root, left)) != NULL)
 	{
-		if (show_message(ses, LIST_CLASS))
-		{
-			tintin_printf(ses, "#Ok. {%s} removed from class list.", node->left);
-		}
+		show_message(ses, LIST_CLASS, "#OK. {%s} IS NO LONGER A CLASS.", left);
+
 		deletenode_list(ses, node, LIST_CLASS);
-		flag = TRUE;
+
+		found = TRUE;
 	}
 
-	if (!flag && show_message(ses, LIST_CLASS))
+	if (found == FALSE)
 	{
-		tintin_puts2("#THAT CLASS IS NOT DEFINED.", ses);
+		show_message(ses, LIST_CLASS, "#UNCLASS: NO MATCH(ES) FOUND FOR {%s}.", left);
 	}
 	return ses;
 }
 
+*/
+
+int count_class(struct session *ses, struct listnode *class)
+{
+	struct listnode *node;
+	int list, cnt;
+
+	for (cnt = list = 0 ; list < LIST_MAX ; list++)
+	{
+		for (node = ses->list[list]->f_node ; node ; node = node->next)
+		{
+			if (node->class == class)
+			{
+				cnt++;
+			}
+		}
+	}
+	return cnt;
+}
+
+void kill_classes(struct session *ses)
+{
+	int list;
+	struct listnode *node, *node_next;
+
+	for (list = 0 ; list < LIST_ALL ; list++)
+	{
+		for (node = ses->list[list]->f_node ; node ; node = node_next)
+		{
+			node_next = node->next;
+
+			if (node->class || list == LIST_CLASS)
+			{
+				deletenode_list(ses, node, list);
+			}
+		}
+	}
+}
+
 DO_CLASS(class_open)
 {
-	ses->class = search_node_with_wild(ses->list[LIST_CLASS], arg);
+	ses->class = search_node_with_wild(ses->list[LIST_CLASS], left);
 
-	SET_BIT(ses->class->data, NODE_FLAG_CLASS);
+	SET_BIT(ses->class->flags, NODE_FLAG_CLASS);
+
+	show_message(ses, LIST_CLASS, "#CLASS {%s} HAS BEEN OPENED.", left);
 
-	if (show_message(ses, LIST_CLASS))
-	{
-		tintin_printf2(ses, "#CLASS {%s} HAS BEEN OPENED.", arg);
-	}
 	return ses;
 }
 
@@ -149,13 +189,11 @@ DO_CLASS(class_close)
 {
 	struct listnode *node;
 
-	node = search_node_with_wild(ses->list[LIST_CLASS], arg);
+	node = search_node_with_wild(ses->list[LIST_CLASS], left);
 
-	if (show_message(ses, LIST_CLASS))
-	{
-		tintin_printf2(ses, "#CLASS {%s} HAS BEEN CLOSED.", node->left);
-	}
-	DEL_BIT(node->data, NODE_FLAG_CLASS);
+	show_message(ses, LIST_CLASS, "#CLASS {%s} HAS BEEN CLOSED.", node->left);
+
+	DEL_BIT(node->flags, NODE_FLAG_CLASS);
 
 	if (node == ses->class)
 	{
@@ -163,25 +201,24 @@ DO_CLASS(class_close)
 
 		for (node = ses->list[LIST_CLASS]->l_node ; node ; node = node->prev)
 		{
-			if (HAS_BIT(node->data, NODE_FLAG_CLASS))
+			if (HAS_BIT(node->flags, NODE_FLAG_CLASS))
 			{
-				class_open(ses, node->left);
+				class_open(ses, node->left, "");
 
 				break;
 			}
 		}
 	}
-
 	return ses;
 }
 
 DO_CLASS(class_read)
 {
-	struct listnode *node = search_node_with_wild(ses->list[LIST_CLASS], arg);
+	class_open(ses, left, "");
 
-	DEL_BIT(node->data, NODE_FLAG_CLASS);
+	do_read(ses, right);
 
-	readfile(ses, node->pr, node);
+	class_close(ses, left, "");
 
 	return ses;
 }
@@ -189,20 +226,21 @@ DO_CLASS(class_read)
 DO_CLASS(class_write)
 {
 	FILE *file;
-	char temp[BUFFER_SIZE], filename[BUFFER_SIZE];
+	char temp[BUFFER_SIZE];
 	struct listnode *node, *class;
 	int cnt;
 
-	class = search_node_with_wild(ses->list[LIST_CLASS], arg);
+	class = search_node_with_wild(ses->list[LIST_CLASS], left);
 
-	get_arg_in_braces(class->pr, filename, TRUE);
-
-	if (*filename == 0 || (file = fopen(filename, "w")) == NULL)
+	if (*right == 0 || (file = fopen(right, "w")) == NULL)
 	{
-		tintin_printf2(ses, "#ERROR - COULDN'T OPEN '%s' TO WRITE.", filename);
+		tintin_printf(ses, "#ERROR: #CLASS WRITE {%s} - COULDN'T OPEN FILE TO WRITE.", right);
+
 		return ses;
 	}
 
+	fprintf(file, "%cCLASS {%s} OPEN\n\n", gtd->tintin_char, left);
+
 	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
 	{
 		for (node = ses->list[cnt]->f_node ; node ; node = node->next)
@@ -216,27 +254,31 @@ DO_CLASS(class_write)
 		}
 	}
 
+	fprintf(file, "\n%cCLASS {%s} CLOSE\n", gtd->tintin_char, left);
+
 	fclose(file);
 
-	if (show_message(ses, LIST_CLASS))
-	{
-		tintin_printf2(ses, "#CLASS {%s} HAS BEEN WRITTEN TO FILE.", class->left);
-	}
+	show_message(ses, LIST_CLASS, "#CLASS {%s} HAS BEEN WRITTEN TO FILE.", left);
+
 	return ses;
 }
 
 DO_CLASS(class_kill)
 {
-	struct listnode *node, *class;
+	struct listnode *node, *node_next, *class;
 	int cnt;
 
-	class = search_node_with_wild(ses->list[LIST_CLASS], arg);
+	class = search_node_with_wild(ses->list[LIST_CLASS], left);
 
 	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
 	{
-		for (node = ses->list[cnt]->f_node ; node ; node = ses->list[cnt]->update)
+		if (cnt == LIST_CLASS)
 		{
-			ses->list[cnt]->update = node->next;
+			continue;
+		}
+		for (node = ses->list[cnt]->f_node ; node ; node = node_next)
+		{
+			node_next = node->next;
 
 			if (node->class == class)
 			{
@@ -245,78 +287,9 @@ DO_CLASS(class_kill)
 		}
 	}
 
-	if (show_message(ses, LIST_CLASS))
-	{
-		tintin_printf2(ses, "#CLASS {%s} HAS BEEN CLEARED.", class->left);
-	}
+	show_message(ses, LIST_CLASS, "#CLASS {%s} HAS BEEN CLEARED.", left);
 
 	deletenode_list(ses, class, LIST_CLASS);
 
 	return ses;
 }
-
-void parse_class(struct session *ses, char *input, struct listnode *class)
-{
-	char temp[BUFFER_SIZE], command[BUFFER_SIZE], arg[BUFFER_SIZE];
-	char *pti, *pta;
-	struct listnode *ln;
-	int i;
-
-	push_call("parse_class(%p,%p,%p)",ses,input,class);
-
-	pti = input;
-
-	while (*pti)
-	{
-		if (*pti == ';')
-		{
-			if (*++pti == 0)
-			{
-				break;
-			}
-		}
-		pti = (char *)get_arg_stop_spaces(pti, temp);
-		pti = (char *)get_arg_all(pti, arg);
-
-		substitute(ses, temp, command, SUB_VAR|SUB_FUN);
-
-		if (*command == gtd->tintin_char)
-		{
-			for (i = 0 ; *command_table[i].name != 0 ; i++)
-			{
-				if (is_abbrev(&command[1], command_table[i].name))
-				{
-					break;
-				}
-			}
-			if (strcmp(command_table[i].name, "class"))
-			{
-				continue;
-			}
-			if (!is_abbrev(class->left, arg))
-			{
-				continue;
-			}
-			(command_table[i].command) (ses, arg);
-		}
-		else if ((ln = searchnode_list_begin(ses->list[LIST_ALIAS], command, ALPHA)))
-		{
-			strcpy(gtd->vars[0], arg);
-
-			pta = arg;
-
-			for (i = 1 ; i < 10 ; i++)
-			{
-				pta = (char *) get_arg_in_braces(pta, gtd->vars[i], FALSE);
-			}
-			substitute(ses, ln->right, command, SUB_ARG);
-
-			if (!strcmp(ln->right, command) && *arg)
-			{
-				sprintf(command, "%s %s", ln->right, arg);
-			}
-			parse_class(ses, command, class);
-		}
-	}
-	return;
-}
