@@ -57,27 +57,20 @@ DO_COMMAND(do_configure)
 
 		tintin_header(ses, "");
 	}
-	else if (*right == 0)
-	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG <OPTION> <ARGUMENT>");
-	}
 	else
 	{
-		for (cnt = 0 ; cnt < CONFIG_MAX ; cnt++)
+		for (cnt = 0 ; *config_table[cnt].name != 0 ; cnt++)
 		{
 			if (is_abbrev(left, config_table[cnt].name))
 			{
 				if (config_table[cnt].config(ses, right, cnt) != NULL)
 				{
-					if (show_message(ses, LIST_CONFIG))
-					{
-						tintin_printf2(ses, "#CONFIG {%s} HAS BEEN SET TO {%s}.", config_table[cnt].name, capitalize(right));
-					}
+					show_message(ses, LIST_CONFIG, "#CONFIG {%s} HAS BEEN SET TO {%s}.", config_table[cnt].name, capitalize(right));
 				}
 				return ses;
 			}
 		}
-		tintin_printf2(ses, "#CONFIG {%s} IS NOT A VALID OPTION.", capitalize(left));
+		tintin_printf(ses, "#ERROR: #CONFIG {%s} IS NOT A VALID OPTION.", capitalize(left));
 	}
 	return ses;
 }
@@ -97,7 +90,7 @@ DO_CONFIG(config_speedwalk)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
@@ -123,7 +116,7 @@ DO_CONFIG(config_verbatim)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
@@ -148,7 +141,7 @@ DO_CONFIG(config_repeatenter)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
@@ -173,7 +166,7 @@ DO_CONFIG(config_echocommand)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
@@ -198,7 +191,7 @@ DO_CONFIG(config_verbose)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
@@ -223,7 +216,7 @@ DO_CONFIG(config_wordwrap)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
@@ -255,7 +248,7 @@ DO_CONFIG(config_log)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG LOG <HTML|PLAIN|RAW>");
+		tintin_printf(ses, "#SYNTAX: #CONFIG LOG <HTML|PLAIN|RAW>");
 
 		return NULL;
 	}
@@ -272,14 +265,14 @@ DO_CONFIG(config_buffersize)
 
 	if (!is_number(arg))
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {BUFFER SIZE} <NUMBER>");
+		tintin_printf(ses, "#SYNTAX: #CONFIG {BUFFER SIZE} <NUMBER>");
 
 		return NULL;
 	}
 
 	if (atoi(arg) < 100 || atoi(arg) > 999999)
 	{
-		tintin_printf2(ses, "#ERROR: #CONFIG BUFFER: PROVIDE A NUMBER BETWEEN 100 and 999999");
+		tintin_printf(ses, "#ERROR: #CONFIG BUFFER: PROVIDE A NUMBER BETWEEN 100 and 999999");
 
 		return NULL;
 	}
@@ -307,7 +300,7 @@ DO_CONFIG(config_scrolllock)
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
@@ -324,7 +317,7 @@ DO_CONFIG(config_connectretry)
 
 	if (!is_number(arg))
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {CONNECT RETRY} <NUMBER>");
+		tintin_printf(ses, "#SYNTAX: #CONFIG {CONNECT RETRY} <NUMBER>");
 
 		return NULL;
 	}
@@ -345,12 +338,12 @@ DO_CONFIG(config_packetpatch)
 
 	if (!is_number(arg))
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {PACKET PATCH} <NUMBER>");
+		tintin_printf(ses, "#SYNTAX: #CONFIG {PACKET PATCH} <NUMBER>");
 	}
 
-	if (atoi(arg) < 0 || atoi(arg) > 1000)
+	if (atoi(arg) < 0 || atoi(arg) > 10000)
 	{
-		tintin_printf2(ses, "#CONFIG PACKET PATCH: PROVIDE A NUMBER BETWEEN 0 and 1000");
+		tintin_printf(ses, "#ERROR: #CONFIG PACKET PATCH: PROVIDE A NUMBER BETWEEN 0 and 10000");
 
 		return NULL;
 	}
@@ -371,12 +364,12 @@ DO_CONFIG(config_historysize)
 
 	if (!is_number(arg))
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {HISTORY SIZE} <NUMBER>");
+		tintin_printf(ses, "#SYNTAX: #CONFIG {HISTORY SIZE} <NUMBER>");
 	}
 
 	if (atoi(arg) < 0 || atoi(arg) > 9999)
 	{
-		tintin_printf2(ses, "#CONFIG HISTORY: PROVIDE A NUMBER BETWEEN 0 and 9999");
+		tintin_printf(ses, "#ERROR: #CONFIG HISTORY: PROVIDE A NUMBER BETWEEN 0 and 9999");
 
 		return NULL;
 	}
@@ -437,15 +430,15 @@ DO_CONFIG(config_debugtelnet)
 
 	if (!strcasecmp(arg, "ON"))
 	{
-		SET_BIT(ses->flags, SES_FLAG_DEBUGTELNET);
+		SET_BIT(ses->telopts, TELOPT_FLAG_DEBUG);
 	}
 	else if (!strcasecmp(arg, "OFF"))
 	{
-		DEL_BIT(ses->flags, SES_FLAG_DEBUGTELNET);
+		DEL_BIT(ses->telopts, TELOPT_FLAG_DEBUG);
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
 
 		return NULL;
 	}
@@ -462,21 +455,42 @@ DO_CONFIG(config_convertmeta)
 
 	if (!strcasecmp(arg, "ON"))
 	{
-		rl_initialize();
-		rl_prep_terminal(1);
-
 		SET_BIT(ses->flags, SES_FLAG_CONVERTMETA);
 	}
 	else if (!strcasecmp(arg, "OFF"))
 	{
-		rl_initialize();
-		rl_deprep_terminal();
-
 		DEL_BIT(ses->flags, SES_FLAG_CONVERTMETA);
 	}
 	else
 	{
-		tintin_printf2(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+
+		return NULL;
+	}
+	sprintf(buf, "%d", index);
+
+	updatenode_list(ses, config_table[index].name, capitalize(arg), buf, LIST_CONFIG);
+
+	check_character_mode(ses);
+
+	return ses;
+}
+
+DO_CONFIG(config_loglevel)
+{
+	char buf[NUMBER_SIZE];
+
+	if (!strcasecmp(arg, "LOW"))
+	{
+		SET_BIT(ses->flags, SES_FLAG_LOGLEVEL);
+	}
+	else if (!strcasecmp(arg, "HIGH"))
+	{
+		DEL_BIT(ses->flags, SES_FLAG_LOGLEVEL);
+	}
+	else
+	{
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <LOW|HIGH>", config_table[index].name);
 
 		return NULL;
 	}
@@ -486,4 +500,3 @@ DO_CONFIG(config_convertmeta)
 
 	return ses;
 }
-
