@@ -29,24 +29,17 @@
 /*********************************************************************/
 
 #include "tintin.h"
+#include <sys/stat.h>
 
 /*
-	read and execute a command file
+	read and execute a command file, supports multi lines - Scandum
 */
 
 DO_COMMAND(do_read)
-{
-	return readfile(ses, arg, NULL);
-}
-
-/*
-	Rewritten from scratch since old version wasn't working - Scandum
-*/
-
-struct session *readfile(struct session *ses, const char *arg, struct listnode *class)
 {
 	FILE *fp;
-	char bufi[FILE_SIZE], bufo[FILE_SIZE], filename[BUFFER_SIZE], temp[BUFFER_SIZE], *pti, *pto;
+	struct stat filedata;
+	char *bufi, *bufo, filename[BUFFER_SIZE], temp[BUFFER_SIZE], *pti, *pto;
 	int lvl, cnt, com, lnc, fix;
 	int counter[LIST_MAX];
 
@@ -54,7 +47,7 @@ struct session *readfile(struct session *ses, const char *arg, struct listnode *
 
 	if ((fp = fopen(filename, "r")) == NULL)
 	{
-		tintin_puts("#ERROR - COULDN'T OPEN THAT FILE.", ses);
+		tintin_printf(ses, "#ERROR: #READ {%s} - COULDN'T OPEN THAT FILE.", filename);
 		return ses;
 	}
 
@@ -62,7 +55,8 @@ struct session *readfile(struct session *ses, const char *arg, struct listnode *
 
 	if (!isgraph(temp[0]))
 	{
-		tintin_puts("#ERROR - INVALID START OF FILE.", ses);
+		tintin_printf(ses, "#ERROR: #READ {%s} - INVALID START OF FILE.", filename);
+
 		fclose(fp);
 
 		return ses;
@@ -75,26 +69,39 @@ struct session *readfile(struct session *ses, const char *arg, struct listnode *
 		counter[cnt] = ses->list[cnt]->count;
 	}
 
-	for (bufi[0] = 0 ; fgets(bufo, BUFFER_SIZE, fp) ; strcat(bufi, bufo))
+	stat(filename, &filedata);
+
+	if ((bufi = calloc(1, filedata.st_size + 2)) == NULL || (bufo = calloc(1, filedata.st_size + 2)) == NULL)
 	{
-		if (strlen(bufo) + strlen(bufi) >= FILE_SIZE)
-		{
-			tintin_printf2(ses, "#READ, FILESIZE MUST BE SMALLER THAN %d.", FILE_SIZE);
+		tintin_printf(ses, "#ERROR: #READ {%s} - FAILED TO ALLOCATE MEMORY.", filename);
 
-			fclose(fp);
+		fclose(fp);
 
-			return ses;
-		}
+		return ses;
 	}
 
+	fread(bufi, 1, filedata.st_size, fp);
+
 	pti = bufi;
 	pto = bufo;
+
 	lvl = com = lnc = fix = 0;
 
 	while (*pti)
 	{
 		if (com == 0)
 		{
+#ifdef BIG5
+			if (*pti & 0x80)
+			{
+				*pto++ = *pti++;
+				if (*pti)
+				{
+					*pto++ = *pti++;
+				}
+				continue;
+			}
+#endif
 			switch (*pti)
 			{
 				case DEFAULT_OPEN:
@@ -216,19 +223,25 @@ struct session *readfile(struct session *ses, const char *arg, struct listnode *
 
 	if (lvl)
 	{
-		tintin_printf2(ses, "#READ '%s' MISSING %d '%c' ON OR BEFORE LINE %d.", filename, abs(lvl), lvl < 0 ? DEFAULT_OPEN : DEFAULT_CLOSE, fix == 0 ? lnc + 1 : fix);
+		tintin_printf(ses, "#ERROR: #READ {%s} - MISSING %d '%c' ON OR BEFORE LINE %d.", filename, abs(lvl), lvl < 0 ? DEFAULT_OPEN : DEFAULT_CLOSE, fix == 0 ? lnc + 1 : fix);
 
 		fclose(fp);
 
+		free(bufi);
+		free(bufo);
+
 		return ses;
 	}
 
 	if (com)
 	{
-		tintin_printf2(ses, "#READ '%s' MISSING %d '%s'", filename, abs(com), com < 0 ? "/*" : "*/");
+		tintin_printf(ses, "#ERROR: #READ {%s} - MISSING %d '%s'", filename, abs(com), com < 0 ? "/*" : "*/");
 
 		fclose(fp);
 
+		free(bufi);
+		free(bufo);
+
 		return ses;
 	}
 
@@ -257,23 +270,19 @@ struct session *readfile(struct session *ses, const char *arg, struct listnode *
 
 			bufi[20] = 0;
 
-			tintin_printf2(ses, "#READ '%s' BUFFER OVERFLOW AT COMMAND: %s.", filename, bufi);
+			tintin_printf(ses, "#ERROR: #READ {%s} - BUFFER OVERFLOW AT COMMAND: %s.", filename, bufi);
 
 			fclose(fp);
 
+			free(bufi);
+			free(bufo);
+
 			return ses;
 		}
 
 		if (bufi[0])
 		{
-			if (class == NULL || HAS_BIT(class->data, NODE_FLAG_CLASS))
-			{
-				ses = parse_input(bufi, ses);
-			}
-			else
-			{
-				parse_class(ses, bufi, class);
-			}
+			ses = parse_input(bufi, ses);
 		}
 		pto = bufi;
 		pti++;
@@ -291,20 +300,24 @@ struct session *readfile(struct session *ses, const char *arg, struct listnode *
 					break;
 
 				case 1:
-					tintin_printf2(ses, "#OK %2d %s LOADED.", ses->list[cnt]->count - counter[cnt], list_table[cnt].name);
+					tintin_printf(ses, "#OK: %3d %s LOADED.", ses->list[cnt]->count - counter[cnt], list_table[cnt].name);
 					break;
 
 				default:
-					tintin_printf2(ses, "#OK %2d %s LOADED.", ses->list[cnt]->count - counter[cnt], list_table[cnt].name_multi);
+					tintin_printf(ses, "#OK: %3d %s LOADED.", ses->list[cnt]->count - counter[cnt], list_table[cnt].name_multi);
 					break;
 			}
 		}
 	}
 	fclose(fp);
 
+	free(bufi);
+	free(bufo);
+
 	return ses;
 }
 
+
 DO_COMMAND(do_write)
 {
 	FILE *file;
@@ -316,7 +329,7 @@ DO_COMMAND(do_write)
 
 	if (*filename == 0 || (file = fopen(filename, "w")) == NULL)
 	{
-		tintin_printf2(ses, "#ERROR - COULDN'T OPEN '%s' TO WRITE.", filename);
+		tintin_printf(ses, "#ERROR: #write {%s} - COULDN'T OPEN FILE TO WRITE.", filename);
 		return ses;
 	}
 
@@ -332,47 +345,7 @@ DO_COMMAND(do_write)
 
 	fclose(file);
 
-	if (show_message(ses, -1))
-	{
-		tintin_printf2(ses, "#COMMANDO-FILE WRITTEN.");
-	}
-	return ses;
-}
-
-
-DO_COMMAND(do_writesession)
-{
-	FILE *file;
-	char buffer[BUFFER_SIZE];
-	struct listnode *node;
-	int cnt;
-
-	get_arg_in_braces(arg, buffer, TRUE);
-
-	if (*buffer == 0 || (file = fopen(buffer, "w")) == NULL)
-	{
-		tintin_printf2(ses, "#ERROR - COULDN'T OPEN '%s' TO WRITE.", buffer);
-		return ses;
-	}
-
-	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
-	{
-		for (node = ses->list[cnt]->f_node ; node ; node = node->next)
-		{
-			if (gts != ses && searchnode_list(gts->list[cnt], node->left))
-			{
-				continue;
-			}
-
-			prepare_for_write(cnt, node, buffer);
-
-			fputs(buffer, file);
-		}
-	}
-
-	fclose(file);
-
-	tintin_printf2(ses, "#COMMANDO-FILE WRITTEN.");
+	show_message(ses, -1, "#OK. COMMAND FILE WRITTEN.", filename);
 
 	return ses;
 }
@@ -401,17 +374,16 @@ void prepare_for_write(int list, struct listnode *node, char *result)
 }
 
 
-
 DO_COMMAND(do_textin)
 {
 	FILE *fp;
-	char buffer[BUFFER_SIZE], *cptr;
+	char filename[BUFFER_SIZE], buffer[BUFFER_SIZE], *cptr;
 
-	get_arg_in_braces(arg, buffer, 1);
+	get_arg_in_braces(arg, filename, TRUE);
 
-	if ((fp = fopen(buffer, "r")) == NULL)
+	if ((fp = fopen(filename, "r")) == NULL)
 	{
-		tintin_printf2(ses, "#TEXTIN, FILE '%s' NOT FOUND.", buffer);
+		tintin_printf(ses, "#ERROR: #TEXTIN {%s} - FILE NOT FOUND.", filename);
 		return ses;
 	}
 
@@ -421,19 +393,15 @@ DO_COMMAND(do_textin)
 		{
 			;
 		}
-		*cptr = '\0';
+		*cptr++ = '\r';
+		*cptr++ = '\n';
+		*cptr++ = '\0';
 
-		if (*buffer)
-		{
-			write_line_mud(buffer, ses);
-		}
-		else
-		{
-			write_line_mud(" ", ses);
-		}
+		write_line_mud(buffer, ses);
 	}
 	fclose(fp);
-	tintin_puts2("#OK, TEXTIN COMPLETED.", ses);
+
+	show_message(ses, -1, "#OK. FILE READ.");
 
 	return ses;
 }
@@ -442,13 +410,13 @@ DO_COMMAND(do_textin)
 DO_COMMAND(do_scan)
 {
 	FILE *fp;
-	char buffer[BUFFER_SIZE];
+	char filename[BUFFER_SIZE];
 
-	get_arg_in_braces(arg, buffer, 1);
+	get_arg_in_braces(arg, filename, TRUE);
 
-	if ((fp = fopen(buffer, "r")) == NULL)
+	if ((fp = fopen(filename, "r")) == NULL)
 	{
-		tintin_printf2(ses, "#SCAN, FILE '%s' NOT FOUND.", buffer);
+		tintin_printf(ses, "#ERROR: #SCAN {%s} - FILE NOT FOUND.", filename);
 		return ses;
 	}
 
@@ -467,96 +435,9 @@ DO_COMMAND(do_scan)
 
 	DEL_BIT(ses->flags, SES_FLAG_SCAN);
 
-	tintin_printf2(ses, "OK, SCANNED FILE '%s'.", buffer);
+	show_message(ses, -1, "#OK. FILE READ.", filename);
 
 	fclose(fp);
 
 	return ses;
 }
-
-DO_COMMAND(do_readmap)
-{
-	FILE *myfile;
-	char buffer[BUFFER_SIZE], *cptr;
-	const char *filename = arg;
-
-	get_arg_in_braces(filename, buffer, 1);
-
-	if ((myfile = fopen(buffer, "r")) == NULL)
-	{
-		tintin_puts("#ERROR - COULDN'T OPEN THAT FILE.", ses);
-		return(ses);
-	}
-
-	SET_BIT(gts->flags, SES_FLAG_QUIET);
-
-	while (fgets(buffer, BUFFER_SIZE - 1, myfile))
-	{
-		for (cptr = buffer ; *cptr && *cptr != '\n' ; cptr++);
-
-		*cptr = 0;
-
-		switch (buffer[0])
-		{
-			case 'R':
-				ses->in_room = create_room(ses, &buffer[2]);
-				break;
-			case 'E':
-				create_exit(ses, &buffer[2]);
-				break;
-		}
-	}
-
-	DEL_BIT(gts->flags, SES_FLAG_QUIET);
-
-	fclose(myfile);
-
-	return(ses);
-}
-
-
-DO_COMMAND(do_writemap)
-{
-	FILE *file;
-	char buffer[BUFFER_SIZE];
-	struct exit_data *exit;
-	int vnum;
-
-	get_arg_in_braces(arg, buffer, 1);
-
-	if (*buffer == 0 || (file = fopen(buffer, "w")) == NULL)
-	{
-		tintin_printf2(ses, "#ERROR - COULDN'T OPEN '%s' TO WRITE.", buffer);
-		return ses;
-	}
-
-	for (vnum = 0 ; vnum < MAX_ROOM ; vnum++)
-	{
-		if (ses->room_list[vnum])
-		{
-			sprintf(buffer, "R {%5d} {%d} {%s} {%s}\n",
-				ses->room_list[vnum]->vnum,
-				ses->room_list[vnum]->flags,
-				ses->room_list[vnum]->color,
-				ses->room_list[vnum]->name);
-
-			fputs(buffer, file);
-
-			for (exit = ses->room_list[vnum]->f_exit ; exit ; exit = exit->next)
-			{
-				sprintf(buffer, "E {%5d} {%s} {%s}\n",
-					exit->vnum,
-					exit->name,
-					exit->cmd);
-
-				fputs(buffer, file);
-			}
-		}
-	}
-
-	fclose(file);
-
-	tintin_printf2(ses, "#MAP-FILE WRITTEN.");
-
-	return ses;
-}
