@@ -28,6 +28,7 @@
 
 #include "tintin.h"
 
+
 struct help_type
 {
 	char                  * name;
@@ -38,7 +39,7 @@ struct help_type
 	This help table is a mess, but I got better things to do - Scandum
 */
 
-const struct help_type help_table[81] =
+const struct help_type help_table[] =
 {
 	{
 		"ALIAS",
@@ -50,7 +51,7 @@ const struct help_type help_table[81] =
 		"\n"
 		"         If the commands include variables %0-9, these are substituted as\n"
 		"         part of the parsing.  %0 is set to all text after the word was input,\n"
-		"         %1 is set to the first word following the aliased word, and %2 is the\n"
+    		"         %1 is set to the first word following the aliased word, and %2 is the\n"
 		"         second, and so on.\n"
 		"\n"
 		"Example: #alias gb get bread bag\n"
@@ -67,6 +68,7 @@ const struct help_type help_table[81] =
 		"         Have tintin search for a certain string of text from the mud, if the\n"
 		"         string is found it will execute the commands.  Variables %0 to %9\n"
 		"         are substituted from the input string, and can be used in the command.\n"
+		"         Brackets can be used for selective keywords\n"
 		"         The third parameter of the command will tell how important tintin\n"
 		"         should reguard this action, 0 most important, 9 least important.  If\n"
 		"         no priority is given, a default value of 5 is assigned.\n"
@@ -80,6 +82,8 @@ const struct help_type help_table[81] =
 		"         fooled by someone saying 'You are hungry'.\n"
 		"Example: #act {%0 tells you 'hello'} {tell %0 Hello %0. How are you today?}\n"
 		"         This allows you to seem like a nice, attentive, fast typing gentleman.\n"
+		"Example: #act {A puppy barks at you, wagging [his|her|its] tail.} {slap puppy}\n"
+		"         This allows you to deal with annoy puppies regardless of their sex.\n"
 	},
 	{
 		"ALL",
@@ -98,23 +102,49 @@ const struct help_type help_table[81] =
 		"Example: #action {tells you} {#bell}\n"
 		"         Will chime every time someone gives you a tell.\n"
 	},
-	{
-		"BOSS",
-		"Command: #boss\n"
-		"\n"
-		"         Puts some fake text on your screen so nobody knows you're mudding.\n"
-		"         No new text will appear untill you press a button.\n"
-		"\n"
-		"Example: #macro {\\e[11~} {#boss}\n"
-		"         Will make you go into boss mode whenever you hit F1\n"
-	},
 	{
 		"BUFFER",
-		"Command: #buffer {home|up|down|end|write filename|info}\n"
+		"Command: #buffer {home|up|down|end|find|write filename|info}\n"
 		"         The buffer command allows you to add macros to scroll in case the\n"
 		"         default bindings do not work. The write option allows you to save\n"
 		"         the entire scrollback buffer to file.\n"
 	},
+	{
+		"CHAT",
+		"Command: #chat {option} {argument}\n"
+		"\n"
+		"         #chat {init}       {port}             Initilizes a chat port.\n"
+		"         #chat {name}       {name}             Sets your chat name.\n"
+		"         #chat {message}    {buddy|all} {text} Sends a chat message\n"
+		"\n"
+		"         #chat {accept}     {buddy} {boosted}  Accept a file transfer\n"
+		"         #chat {call}       {address} {port}   Connect to a buddy\n"
+		"         #chat {cancel}     {buddy}            Cancel a file transfer\n"
+		"         #chat {color}      {color names}      Set the default color\n"
+		"         #chat {decline}    {buddy}            Decline a file transfer\n"
+		"         #chat {dnd}                           Decline new connections\n"
+		"         #chat {download}   {buddy}            Set your download dir\n"
+		"         #chat {emote}      {buddy|all} {text} Send an emote message\n"
+		"         #chat {forward}    {buddy}            Forward all chat messages\n"
+		"         #chat {forwardall} {buddy}            Forward all session output\n"
+		"         #chat {filestat}   {buddy}            Show file transfer data\n"
+		"         #chat {group}      {buddy} {name}     Assign a chat group\n"
+		"         #chat {ignore}     {buddy}            Ignores someone\n"
+		"         #chat {info}                          Displays your info\n"
+		"         #chat {ip}         {address}          Changes your IP address\n"
+		"         #chat {paste}      {buddy|all} {text} Pastes a block of text\n"
+		"         #chat {peek}       {buddy}            Show one's public connections\n"
+		"         #chat {ping}       {buddy}            Display response time\n"
+		"         #chat {private}    {buddy|all}        Make a connection private\n"
+		"         #chat {public}     {buddy|all}        Make a connection public\n"
+		"         #chat {reply}      {text}             Reply to last private message\n"
+		"         #chat {request}    {buddy}            Request one's public connections\n"
+		"         #chat {send}       {buddy|all} {text} Sends a raw data string\n"
+		"         #chat {sendfile}   {buddy} {filename} Start a file transfer\n"
+		"         #chat {serve}      {buddy}            Forward all public chat messages\n"
+		"         #chat {who}                           Show all connections\n"
+		"         #chat {zap}        {buddy}            Close a connection\n"
+	},
 	{
 		"CLASS",
 		"Command: #class {name} {open|close|read filename|write filename|kill}\n"
@@ -123,8 +153,7 @@ const struct help_type help_table[81] =
 		"         will be assigned to that class.\n"
 		"         The {close} option will close the given class and make the last\n"
 		"         accessed open class active.\n"
-		"         The {read} option will only read triggers of the given class from file.\n"
-		"         the class when finished.\n"
+		"         The {read} option will open the class, read, and close afterwards.\n"
 		"         The {write} option will write all triggers of the given class to file.\n"
 		"         The {kill} option will delete all triggers of the given class.\n"
 	},
@@ -155,8 +184,8 @@ const struct help_type help_table[81] =
 		"         This will show: <115>t<828>e<838>s<848>t<088> <468>!<568>!<088>\n"
 	},
 	{
-		"CONFIGURE",
-		"Commands: #configure {option} {argument}\n"
+		"CONFIG",
+		"Commands: #config {option} {argument}\n"
 		"\n"
 		"          This allows you to configure various settings, the settings can be\n"
 		"          written to file with the #write or #writesession command.\n"
@@ -166,6 +195,12 @@ const struct help_type help_table[81] =
 		"\n"
 		"          It's advised to make a configuration file to read on startup if you\n"
 		"          do not like the default settings.\n"
+		"\n"
+		"          Config options which aren't listed by default:\n"
+		"\n"
+		"          #CONFIG {CONVERT META} {ON|OFF} Shows color codes and key bindings.\n"
+		"          #CONFIG {DEBUG TELNET} {ON|OFF} Shows telnet negotiations y/n.\n"
+		"          #CONFIG {LOG LEVEL}  {LOW|HIGH} LOW logs mud output before triggers\n"
 	},
 	{
 		"CR",
@@ -178,7 +213,8 @@ const struct help_type help_table[81] =
 	{
 		"CREDITS",
 		"\n"
-		"<068>    ######################################################################\n"
+		"<068>    #<068>####################################################################<068>#\n"
+		"<068>    #<078>                                                                    <068>#\n"
 		"<068>    #<078>                           T I N T I N + +                          <068>#\n"
 		"<068>    #<078>                                                                    <068>#\n"
 		"<068>    #<078>            (<068>T<078>)he k(<068>I<078>)cki(<068>N<078>) (<068>T<078>)ickin d(<068>I<078>)kumud clie(<068>N<078>)t            <068>#\n"
@@ -186,10 +222,11 @@ const struct help_type help_table[81] =
 		"<068>    #<078>                 Original TINTIN code by Peter Unold                <068>#\n"
 		"<068>    #<078>       new code by Bill Reiss, David A. Wagner, Rob Ellsworth,      <068>#\n"
 		"<068>    #<078>                 Jeremy C. Jack, Igor van den Hoven                 <068>#\n"
-		"<068>    #<078>                              1994,2005                             <068>#\n"
+		"<068>    #<078>                             1992, 2006                             <068>#\n"
 		"<068>    #<078>                                                                    <068>#\n"
-		"<068>    #<078>                           Version 1.95.1                           <068>#\n"
-		"<068>    ######################################################################<088>\n"
+		"<068>    #<078>                           Version 1.96.3                           <068>#\n"
+		"<068>    #<078>                                                                    <068>#\n"
+		"<068>    #<068>####################################################################<068>#<088>\n\n"
 	},
 	{
 		"DELAY",
@@ -215,7 +252,6 @@ const struct help_type help_table[81] =
 		"         Stores the current epoch value in the $time variable, and echoes\n"
 		"         the military date and time based on this value. As I write this it\n"
 		"         printed: The current time and date is: 10-03-2004 23:34\n"
-		"         You might want to escape the %'s when using echo in an action or alias\n"
 	},
 	{
 		"END",
@@ -250,10 +286,10 @@ const struct help_type help_table[81] =
 	{
 		"HIGHLIGHT",
 		"\n"
-		"Command: #highlight {string} {color codes}\n"
+		"Command: #highlight {string} {color names}\n"
 		"\n"
 		"         The highlight command is used to allow you to highlight strings of text\n"
-		"         from the mud.  Available color codes are:\n"
+		"         from the mud.  Available color names are:\n"
 		"\n"
 		"         default, light, faint, underscore, blink, reverse, dim, black, red,\n"
 		"         green, yellow, blue, magenta, cyan, white, b black, b red, b green,\n"
@@ -275,6 +311,15 @@ const struct help_type help_table[81] =
 		"\n"
 		"Notice:  See #help substitute, for more advanced color substitution.\n"
 	},
+	{
+		"HISTORY",
+		"\n"
+		"Command: #history {add} {line}\n"
+		"         #history {repeat} {number}\n"
+		"\n"
+		"         Either adds a command to the command history or repeats a command in\n"
+		"         the history list, 0 being the last command.\n"
+	},
 	{
 		"IF",
 		"\n"
@@ -306,7 +351,7 @@ const struct help_type help_table[81] =
 	},
 	{
 		"DEBUG",
-		"Command: #debug {listname} {on|off}\n"
+		"Command: #debug {listname} {on|off|log}\n"
 		"\n"
 		"         Toggles a list on or off. With no argument it shows your current\n"
 		"         settings, as well as the list names that you can debug.\n"
@@ -314,13 +359,32 @@ const struct help_type help_table[81] =
 		"         If you for example set ACTIONS to ON you will get debug information\n"
 		"         whenever an action is triggered.\n"
 		"\n"
+		"         #debug {listname} {log} will silently write debugging information to\n"
+		"         the log file, you must be logging in order for this to work.\n"
+		"\n"
 		"         Not every list has debug support yet.\n"
 	},
+	{
+		"ESCAPE",
+		"You may use the escape character \\ for various special characters.\n"
+		"\n"
+		"\\a   will beep the terminal.\n"
+		"\\e   will start an escape sequence.\n"
+		"\\n   will send a new line.\n"
+		"\\r   will send a carriage return.\n"
+		"\\t   will send a tab.\n"
+		"\\x   will print a hexadecimal value, \xFF for example.\n"
+		"\\x7B will send the '{' character.\n"
+		"\\x7D will send the '}' character.\n"
+		"\n"
+		"Ending a line with \\ will stop tintin from appending \\r\\n. To escape\n"
+		"arguments in an alias use %%90 %%91 %%92 etc.\n"
+	},
 	{
 		"FORALL",
 		"Command: #forall {list} {command}\n"
 		"         How this works is best shown with an example\n"
-		"Example: #forall {a b c d} {say $forall}\n"
+		"Example: #forall {a b c d} {say &0}\n"
 		"         This equals: say a;say b;say c;say d;\n"
 		"         Useful for whenever it's useful.\n"
 	},
@@ -341,15 +405,19 @@ const struct help_type help_table[81] =
 		"         #format {test} {%l}   {string}  lowercase text\n"
 		"         #format {test} {%m}   {string}  perform mathematical calculation\n"
 		"         #format {test} {%n}     {name}  capitalize the first letter\n"
+		"         #format {test} {%p}   {string}  strip leading and trailing spaces\n"
 		"         #format {test} {%r}   {string}  reverse text, hiya = ayih\n"
 		"         #format {test} {%t}     {time}  print the current military time\n"
 		"         #format {test} {%u}   {string}  uppercase text\n"
-		"         #format {time} {%T}             store the epoch time in {time}\n"
-		"         #format {time} {%U}             store the micro epoch time in {time}\n"
 		"         #format {test} {%C}   {colums}  store the screen width in {colums}\n"
+		"         #format {test} {%D}     {time}  print the current day\n"
 		"         #format {test} {%G}   {number}  perform thousand grouping on {number}\n"
 		"         #format {test} {%L}   {string}  store the string length in {test}\n"
+		"         #format {test} {%M}     {time}  print the current month\n"
 		"         #format {test} {%R}     {rows}  store the screen height in {rows}\n"
+		"         #format {time} {%T}             store the epoch time in {time}\n"
+		"         #format {time} {%U}             store the micro epoch time in {time}\n"
+		"         #format {time} {%Y}     {time}  print the current year\n"
 	},
 	{
 		"FUNCTION",
@@ -374,36 +442,14 @@ const struct help_type help_table[81] =
 		"#showme The time is @gettime{}\n"
 		"This will print the current military time.\n"
 	},
-	{
-		"GETITEMNR",
-		"\n"
-		"Command: getitemnr - retrieves specified element from a list (NEW)\n"
-		"\n"
-		"Syntax:  #getitemnr {destination variable} {item number} {list}\n"
-		"\n"
-		"Example: #geti {dothis} {2} {smile {say Hi!} flip bounce}  <--- sets dothis to 'say Hi!'\n"
-		"\n"
-		"Example: #getl Length {$rndsocial};#math itemnr {1 d $Length};#geti temp {$itemnr}\n"
-		"{$rndsocial};{$temp}\n"
-		"         (will perform a random social from the 'rndsocial' list)\n"
-	},
-	{
-		"GETLISTLENGTH",
-		"\n"
-		"Command: getlistlength - returns the length of a list\n"
-		"\n"
-		"Syntax:  #getlistlength {dest var} {list}\n"
-		"\n"
-		"Example: #getl Length {$alignlist}   <--- returns 3 in the Length variable\n"
-	},
 	{
 		"GREP",
 		"Command: #grep [page] {search string}\n"
 		"         This command allows you to search for matching lines in your scroll\n"
-		"         back buffer. The amount of matches shown equals your screen size. If you\n"
-		"         want to search back further use a page number. You can use wildcards\n"
-		"         for better search results. Default * *'s are placed around the search\n"
-		"         string to make life easier.\n"
+		"         back buffer. The amount of matches shown equals your screen size. If\n"
+		"         you want to search back further use a page number. You can use\n"
+		"         wildcards for better search results. Default * *'s are placed around\n"
+		"         the search string to make things a little easier.\n"
 	},
 	{
 		"INFO",
@@ -413,18 +459,40 @@ const struct help_type help_table[81] =
 		"         Displays all the settings of every tintin list available.\n"
 	},
 	{
-		"KILLALL",
+		"KILL",
 		"\n"
-		"Command: #killall\n"
+		"Command: #kill {list|all}\n"
 		"\n"
-		"         Killall deletes all lists.  Useful so you don't have to exit tintin++\n"
-		"         to load up a new command file.\n"
+		"         Without an argument kill deletes all lists.  Useful so you don't\n"
+		"         have to exit tintin++ to load up a new command file.\n"
+		"\n"
+		"         With an argument a specific list can be killed.\n"
+	},
+	{
+		"LIST",
+		"Command: #list {variable} {del|ins|get|set|len} {argument}\n"
+		"\n"
+		"#list {list} {del} {index}             Delete an item from the list\n"
+		"#list {list} {ins} {index} {string}    Insert {string} at given index\n"
+		"#list {list} {fnd} {string} {variable} Return index if {string} is found\n"
+		"#list {list} {get} {index} {variable}  Copy an item to {variable}\n"
+		"#list {list} {set} {index} {string}    Change an item at the given index\n"
+		"#list {list} {len} {variable}          Copy list length to {variable}\n"
+		"\n"
+		"The index should be between 1 and the list's length. You can also give\n"
+		"a negative value, in which case -1 equals the last item in the list, -2\n"
+		"the second last, etc.\n"
+		"\n"
+		"When inserting an item a positive index will prepend the item at the given\n"
+		"index, while a negative index will append the item.\n"
+		"\n"
+		"A length of 0 is returned for an empty or non existant list."
 	},
 	{
 		"LOADPATH",
 		"Command: #loadpath {alias name}\n"
 		"\n"
-		"Load a saved path back into the path, useful for the #walk command.\n"
+		"Load a saved path alias back into the path, useful for the #walk command.\n"
 	},
 	{
 		"LOG",
@@ -449,10 +517,10 @@ const struct help_type help_table[81] =
 		"\n"
 		"Like a for statement, loop will loop from start to finish incrementing or\n"
 		"decrementing by 1 each time through.  The value of the loop variable is\n"
-		"placed in $loop, which you can use in the commands.\n"
+		"placed in &0, which you can use in the commands.\n"
 		"\n"
 		"examples:\n"
-		"#loop {1 3} {get all $loop.corpse}\n"
+		"#loop {1 3} {get all &0.corpse}\n"
 		"equivalent to the following:\n"
 		"  get all 1.corpse;get all 2.corpse;get all 3.corpse\n"
 		"#loop {3,1} {drop &0.key}\n"
@@ -528,8 +596,6 @@ const struct help_type help_table[81] =
 		"level.  The above will modify that variable after every kill, showing\n"
 		"the amount still needed.\n"
 		"#act {%0 tells %1} {#if {\"%0\" == \"Scandum\" && $afk} {reply $afk Sir} {reply $afk dude}}\n"
-		"Compare the string and reply appropriately if $afk has been set to\n"
-		"'Watching britney spears on MTV' or something of that nature =]\n"
 	},
 	{
 		"MESSAGE",
@@ -553,6 +619,15 @@ const struct help_type help_table[81] =
 		"example:\n"
 		"#nop This is the start of my autoactions\n"
 	},
+	{
+		"PARSE",
+		"\n"
+		"Command: #parse {string} {commands}\n"
+		"\n"
+		"Like a for statement, parse will loop from start to finish through the\n"
+		"given string. The value of the character parse is at will be placed in\n"
+		"the &0 variable.\n"
+	},
 	{
 		"PATH",
 		"\n"
@@ -591,8 +666,14 @@ const struct help_type help_table[81] =
 		"READ",
 		"Command: #read {filename}\n"
 		"\n"
-		"Reads a coms file into memory.  The coms file is merged in with the\n"
-		"currently loaded commands.  Duplicate commands are overwritten.\n"
+		"         Reads a coms file into memory.  The coms file is merged in with the\n"
+		"         currently loaded commands.  Duplicate commands are overwritten.\n"
+		"\n"
+		"         If you uses braces, { and } you can use several lines for 1 commands.\n"
+		"         This however means you must always match every { with a } for the read\n"
+		"         command to work.\n"
+		"\n"
+		"         You can comment text using /* test */\n"
 	},
 	{
 		"REDIT",
@@ -606,6 +687,8 @@ const struct help_type help_table[81] =
 		"#redit find firstroom (searches the shortest path, see #path)\n"
 		"#redit color <code> will set the room's color, see #help colors for info.\n"
 		"#redit link <direction> <room name> will link to an already mapped room\n"
+		"#redit load <filename> loads a map from file\n"
+		"#redit save <filename> saves a map to file\n"
 	},
 	{
 		"REPLACESTRING",
@@ -637,13 +720,21 @@ const struct help_type help_table[81] =
 		"\n"
 		"Command: #scan {filename}\n"
 		"\n"
-		"Reads in a filename and sends it to the screen as if it was send by a mud. This\n"
-		"is useful to convert ansi color files to html and viewing log files.\n"
+		"Reads in a file and sends it to the screen as if it was send by a mud. This is\n"
+		"useful to convert ansi color files to html or viewing log files.\n"
+	},
+	{
+		"SEND",
+		"\n"
+		"Command: #send {text}\n"
+		"\n"
+		"Sends the text directly to the MUD, useful if you want to start with an\n"
+		"escape code.\n"
 	},
 	{
 		"SESSION",
 		"\n"
-		"Command: #session {name} {host} {port}\n"
+		"Command: #session {name} {host port}\n"
 		"\n"
 		"Starts a telnet session with the given name, host, and port. The name can\n"
 		"be anything you want, except the name of an already existant session.\n"
@@ -655,6 +746,7 @@ const struct help_type help_table[81] =
 		"\n"
 		"#{name} {command} allows executing a command with the given session without\n"
 		"changing the active session.\n"
+		"The startup session is named 'gts' and can be used for auto relog scripts.\n"
 	},
 
 	{
@@ -693,7 +785,7 @@ const struct help_type help_table[81] =
 	{
 		"SPLIT",
 		"\n"
-		"Command: #split {top line} {bottom line}\n"
+		"Command: #split {top lines} {bottom lines}\n"
 		"\n"
 		"This option only works with a vt102 emulator, this will allow you to set up a\n"
 		"screen.  The keyboard input will be displayed in the bottom window,\n"
@@ -736,7 +828,7 @@ const struct help_type help_table[81] =
 		"#sub {^Zoe%0} {ZOE%0}\n"
 		"any line that starts with the name Zoe will be replaced by a line that\n"
 		"starts with 'ZOE'\n"
-		"#sub {%0massacres%1} {<018>%0<118>MASSACRES<018>%1}\n"
+		"#sub {%0massacres%1} {<<888>018>%0<<888>118>MASSACRES<<888>018>%1}\n"
 		"replace all occurrences of 'massacres' with 'MASSACRES' and change the colors,\n"
 		"see #help colors, for more information.\n"
 	},
@@ -763,7 +855,6 @@ const struct help_type help_table[81] =
 		"\n"
 		"Adds a word to the tab completion list, alphabetically sorted.\n"
 	},
-
 	{
 		"TEXTIN",
 		"\n"
@@ -814,6 +905,15 @@ const struct help_type help_table[81] =
 		"#unalias {*go*}\n"
 		"remove all aliases that contain the fragment 'go'\n"
 	},
+	{
+		"UNCLASS",
+		"\n"
+		"Command: #unclass {class}\n"
+		"\n"
+		"Removes class(es) from the class list which match {class}. All items\n"
+		"associated to the class will be removed as well.\n"
+	},
+
 	{
 		"UNFUNCTION",
 		"\n"
@@ -863,7 +963,7 @@ const struct help_type help_table[81] =
 		"#act {You cannot go that way.} {#unpath}\n"
 	},
 	{
-		"UNRPROMPT",
+		"UNPROMPT",
 		"\n"
 		"format: 	#unprompt\n"
 		"\n"
@@ -940,15 +1040,17 @@ const struct help_type help_table[81] =
 		"\n"
 		"#alias {target} {#var target %0}\n"
 		"#alias {x}      {flame $target}\n"
-		"#alias {xx}     {flash $target}\n"
-		"#alias {xxx}    {harm  $target}\n"
 		"\n"
 		"The name of a variable must exist of only letters and numbers in\n"
 		"order to be substituted. If you do not meet these requirements do\n"
 		"not panic, simply encapsulate the variable in braces:\n"
 		"\n"
 		"#variable {cool website} {http://tintin.sourceforge.net}\n"
-		"#chat I was on a ${cool website} yesterday!.\n"
+		"#chat I was on ${cool website} yesterday!.\n"
+		"\n"
+		"In order to check if a variable exists you can use regexp:\n"
+		"\n"
+		"#if {\"$target\" == \"?target\"} {#showme variable is not defined}\n"
 	},
 	{
 		"WALK",
@@ -963,7 +1065,7 @@ const struct help_type help_table[81] =
 	{
 		"WILDCARD",
 		"\n"
-		"format:		#command {regexp}\n"
+		"format:		#trigger {regexp}\n"
 		"\n"
 		"You may use wildcards with certain commands such as #alias, #action,\n"
 		"#substitute, #unalias, etc.  In commands like #alias, wildcards are\n"
@@ -994,16 +1096,6 @@ const struct help_type help_table[81] =
 		"Writes all current actions, aliases, subs, antisubs, highlights, and\n"
 		"variables to a coms file, specified by filename.\n"
 	},
-	{
-		"WRITESESSION",
-		"\n"
-		"Command: #writesession {filename}\n"
-		"               #writesession filename\n"
-		"\n"
-		"Write all current actions, aliases, subs, antisubs, highlights, and\n"
-		"variables that are specific to your current session to a file.  This\n"
-		"means actions that were not defined when there was no session active.\n"
-	},
 	{
 		"ZAP",
 		"\n"
@@ -1018,6 +1110,7 @@ const struct help_type help_table[81] =
 	}
 };
 
+
 DO_COMMAND(do_help)
 {
 	char buf[BUFFER_SIZE] = { 0 }, add[BUFFER_SIZE];
@@ -1042,9 +1135,9 @@ DO_COMMAND(do_help)
 	}
 	else
 	{
-		for (cnt = 0 ; help_table[cnt].name != 0 ; cnt++)
+		for (cnt = 0 ; *help_table[cnt].name != 0 ; cnt++)
 		{
-			if (is_abbrev(arg, help_table[cnt].name))
+			if (is_abbrev(arg, help_table[cnt].name) || atoi(arg) == cnt + 1)
 			{
 				substitute(ses, help_table[cnt].text, buf, SUB_COL);
 
@@ -1061,7 +1154,7 @@ DO_COMMAND(do_help)
 				break;
 			}
 		}
-		if (help_table[cnt].name == 0)
+		if (*help_table[cnt].name == 0)
 		{
 			tintin_printf2(ses, "No help found for '%s'", arg);
 		}
