@@ -60,27 +60,21 @@ DO_COMMAND(do_highlight)
 	{
 		if (show_node_with_wild(ses, left, LIST_HIGHLIGHT) == FALSE)
 		{
-			if (show_message(ses, LIST_HIGHLIGHT))
-			{
-				tintin_puts2("#THAT HIGHLIGHT IS NOT DEFINED.", ses);
-			}
+			show_message(ses, LIST_HIGHLIGHT, "#HIGHLIGHT: NO MATCH(ES) FOUND FOR {%s}.", left);
 		}
 	}
 	else
 	{
 		if (get_highlight_codes(right, temp) == FALSE)
 		{
-			tintin_printf2(ses, "#HIGHLIGHT: VALID COLORS ARE:\n\r");
-			tintin_printf2(ses, "reset, light, faint, underscore, blink, reverse, dim, black, red, green, yellow, blue, magenta, cyan, white, b black, b red, b green, b yellow, b blue, b magenta, b cyan, b white");
+			tintin_printf2(ses, "#HIGHLIGHT: VALID COLORS ARE:\r\n");
+			tintin_printf2(ses, "reset, bold, faint, underscore, blink, reverse, dim, black, red, green, yellow, blue, magenta, cyan, white, b black, b red, b green, b yellow, b blue, b magenta, b cyan, b white");
 		}
 		else
 		{
 			updatenode_list(ses, left, right, pr, LIST_HIGHLIGHT);
 
-			if (show_message(ses, LIST_HIGHLIGHT))
-			{
-				tintin_printf2(ses, "#OK. {%s} IS NOW HIGHLIGHTED {%s} @ {%s}", left, right, pr);
-			}
+			show_message(ses, LIST_HIGHLIGHT, "#OK. {%s} NOW HIGHLIGHTS {%s} @ {%s}", left, right, pr);
 		}
 	}
 	return ses;
@@ -92,7 +86,7 @@ DO_COMMAND(do_unhighlight)
 	char left[BUFFER_SIZE];
 	struct listroot *root;
 	struct listnode *node;
-	int flag = FALSE;
+	int found = FALSE;
 
 	root = ses->list[LIST_HIGHLIGHT];
 
@@ -100,17 +94,16 @@ DO_COMMAND(do_unhighlight)
 
 	while ((node = search_node_with_wild(root, left)))
 	{
-		if (show_message(ses, LIST_HIGHLIGHT))
-		{
-			tintin_printf2(ses, "#OK. {%s} IS NO LONGER A HIGHLIGHT.", node->left);
-		}
+		show_message(ses, LIST_HIGHLIGHT, "#OK. {%s} IS NO LONGER A HIGHLIGHT.", node->left);
+
 		deletenode_list(ses, node, LIST_HIGHLIGHT);
-		flag = TRUE;
+
+		found = TRUE;
 	}
 
-	if (flag == FALSE && show_message(ses, LIST_HIGHLIGHT))
+	if (found == FALSE)
 	{
-		tintin_puts2("#THAT HIGHLIGHT IS NOT DEFINED.", ses);
+		show_message(ses, LIST_HIGHLIGHT, "#HIGHLIGHT: NO MATCH(ES) FOUND FOR {%s}.", left);
 	}
 	return ses;
 }
@@ -130,6 +123,11 @@ void check_all_highlights(char *original, char *line, struct session *ses)
 
 			pt1 = strstr(original, buf1) ? original : line;
 			pt2 = strstr(pt1, buf1);
+
+			if (pt2 == NULL)
+			{
+				break;
+			}
 			*pt2 = 0;
 			pt3 = pt2 + strlen(buf1);
 
@@ -141,8 +139,6 @@ void check_all_highlights(char *original, char *line, struct session *ses)
 			sprintf(temp, "%s%s%s\033[0m%s%s", pt1, buf2, buf1, buf3, pt3);
 
 			strcpy(original, temp);
-
-			break;
 		}
 	}
 }
