@@ -60,33 +60,6 @@ void kill_list(struct session *ses, int index)
 	}
 }
 
-/*
-	This function will clear all lists associated with a session
-*/
-
-DO_COMMAND(do_killall)
-{
-	int cnt;
-
-	push_call("kill_all(%p,%p)",ses,arg);
-
-	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
-	{
-		kill_list(ses, cnt);
-
-		if (arg == NULL)
-		{
-			free(ses->list[cnt]);
-		}
-	}
-
-	if (arg != NULL)
-	{
-		tintin_puts2("#KILLALL: LISTS CLEARED.", ses);
-	}
-	pop_call();
-	return ses;
-}
 
 /*
 	make a copy of a list - return: ptr to copy
@@ -134,8 +107,6 @@ void insertnode_list(struct session *ses, const char *ltext, const char *rtext,
 	if (ses->class && index != LIST_CLASS)
 	{
 		newnode->class = ses->class;
-
-		ses->class->data += NODE_FLAG_MAX;
 	}
 
 	ses->list[index]->count++;
@@ -145,14 +116,14 @@ void insertnode_list(struct session *ses, const char *ltext, const char *rtext,
 		case PRIORITY:
 			for (node = ses->list[index]->f_node ; node ; node = node->next)
 			{
-				if (atof(prtext) < atof(node->pr))
+				if (atof(prtext) < atof(node->pr) || (atof(prtext) == atof(node->pr) && strcmp(ltext, node->left) < 0))
 				{
-					INSERT_LEFT(newnode, node, ses->list[index]->f_node, next, prev);
+					INSERT_LEFT(newnode, node, ses->list[index]->f_node);
 					pop_call();
 					return;
 				}
 			}
-			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
+			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node);
 			break;
 
 		case ALPHA:
@@ -160,20 +131,20 @@ void insertnode_list(struct session *ses, const char *ltext, const char *rtext,
 			{
 				if (strcmp(ltext, node->left) < 0)
 				{
-					INSERT_LEFT(newnode, node, ses->list[index]->f_node, next, prev);
+					INSERT_LEFT(newnode, node, ses->list[index]->f_node);
 					pop_call();
 					return;
 				}
 			}
-			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
+			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node);
 			break;
 
 		case APPEND:
-			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
+			LINK(newnode, ses->list[index]->f_node, ses->list[index]->l_node);
 			break;
 
 		default:
-			tintin_printf2(NULL, "insertnode_list: bad list_table mode: %d", list_table[index].mode);
+			tintin_printf2(NULL, "#BUG: insertnode_list: bad list_table mode: %d", list_table[index].mode);
 			break;
 	}
 	pop_call();
@@ -197,6 +168,8 @@ void updatenode_list(struct session *ses, const char *ltext, const char *rtext,
 				node->right = strdup(rtext);
 			}
 
+			node->data = 0;
+
 			switch (list_table[index].mode)
 			{
 				case PRIORITY:
@@ -215,8 +188,8 @@ void updatenode_list(struct session *ses, const char *ltext, const char *rtext,
 						free(node->pr);
 						node->pr = strdup(prtext);
 					}
-					UNLINK(node, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
-					LINK(node, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
+					UNLINK(node, ses->list[index]->f_node, ses->list[index]->l_node);
+					LINK(node, ses->list[index]->f_node, ses->list[index]->l_node);
 					break;
 
 				case ALPHA:
@@ -228,7 +201,7 @@ void updatenode_list(struct session *ses, const char *ltext, const char *rtext,
 					break;
 
 				default:
-					tintin_printf2(ses, "updatenode_list: bad mode");
+					tintin_printf2(ses, "#BUG: updatenode_list: bad mode");
 					break;
 			}
 			pop_call();
@@ -247,7 +220,7 @@ void deletenode_list(struct session *ses, struct listnode *node, int index)
 	if ((node->next == NULL && node != ses->list[index]->l_node)
 	||  (node->prev == NULL && node != ses->list[index]->f_node))
 	{
-		tintin_puts2("#ERROR: delete_nodelist: unlink error.", NULL);
+		tintin_printf2(NULL, "#BUG: delete_nodelist: unlink error.");
 		dump_stack();
 	}
 	else
@@ -257,30 +230,18 @@ void deletenode_list(struct session *ses, struct listnode *node, int index)
 			ses->list[index]->update = node->next;
 		}
 
-		if (index == LIST_CLASS)
-		{
-			if (node->data >= NODE_FLAG_MAX)
-			{
-				class_kill(ses, node->left);
-			}
-		}
-
 		if (ses->class == node)
 		{
 			ses->class = NULL;
 		}
-		UNLINK(node, ses->list[index]->f_node, ses->list[index]->l_node, next, prev);
+
+		UNLINK(node, ses->list[index]->f_node, ses->list[index]->l_node);
 
 		free(node->left);
 		free(node->right);
 		free(node->pr);
 		free(node);
 
-		if (node->class)
-		{
-			node->class->data -= NODE_FLAG_MAX;
-		}
-
 		ses->list[index]->count--;
 	}
 	pop_call();
@@ -351,13 +312,13 @@ void shownode_list(struct session *ses, struct listnode *nptr, int index)
 			sprintf(buf, "#%s <118>{<088>%s<118>}", list_table[index].name, nptr->left);
 			break;
 		default:
-			sprintf(buf, "ERROR: list_table[index].args == 0");
+			sprintf(buf, "#BUG: list_table[index].args == 0");
 			break;
 	}
 
 	substitute(ses, buf, out, SUB_COL);
 
-	tintin_puts2(out, ses);
+	tintin_printf2(ses, "%s", out);
 }
 
 /*
@@ -399,7 +360,7 @@ struct listnode *search_node_with_wild(struct listroot *listhead, const char *cp
 
 	for (node = listhead->f_node ; node ; node = node->next)
 	{
-		if (regexp(cptr, node->left))
+		if (regexp(cptr, node->left) || !strcmp(cptr, node->left))
 		{
 			return node;
 		}
@@ -430,11 +391,12 @@ void addnode_list(struct listroot *listhead, const char *ltext, const char *rtex
 	sprintf(newnode->right, "%s", rtext);
 	sprintf(newnode->pr,    "%s", prtext);
 
-	LINK(newnode, listhead->f_node, listhead->l_node, next, prev);
+	LINK(newnode, listhead->f_node, listhead->l_node);
 
 	listhead->count++;
 }
 
+
 int count_list(struct listroot *listhead)
 {
 	struct listnode *node;
@@ -445,3 +407,229 @@ int count_list(struct listroot *listhead)
 	}
 	return cnt;
 }
+
+DO_COMMAND(do_killall)
+{
+	char left[BUFFER_SIZE];
+	int cnt, fnd = FALSE;
+
+	if (arg == NULL || *arg == 0) 
+	{
+		for (cnt = 0 ; cnt < LIST_ALL ; cnt++)
+		{
+			kill_list(ses, cnt);
+
+			if (arg == NULL)
+			{
+				free(ses->list[cnt]);
+			}
+		}
+		if (arg != NULL)
+		{
+			tintin_printf(ses, "#OK: #KILL - ALL LISTS CLEARED.");
+		}
+		return ses;
+	}
+
+	arg = get_arg_in_braces(arg, left,  0);
+
+	for (cnt = fnd = 0 ; cnt < LIST_ALL ; cnt++)
+	{
+		if (!is_abbrev(left, list_table[cnt].name_multi) && strcasecmp(left, "ALL"))
+		{
+			continue;
+		}
+
+		switch (cnt)
+		{
+			case LIST_CLASS:
+				kill_classes(ses);
+				break;
+
+			default:
+				kill_list(ses, cnt);
+				break;
+		}
+		tintin_printf(ses, "#OK: #%s LIST CLEARED.", list_table[cnt].name);
+		fnd = TRUE;
+	}
+	if (fnd == FALSE)
+	{
+		tintin_printf(ses, "#ERROR: #KILL {%s} - NO MATCH FOUND.", left);
+	}
+	return ses;
+}
+
+DO_COMMAND(do_message)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	int cnt, fnd = FALSE;
+
+	arg = get_arg_in_braces(arg, left,  0);
+	arg = get_arg_in_braces(arg, right, 0);
+
+	if (*left == 0)
+	{
+		tintin_header(ses, " MESSAGES ");
+
+		for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
+		{
+			tintin_printf2(ses, "  %-20s %3s", list_table[cnt].name_multi, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE) ? "ON" : "OFF");
+		}
+
+		tintin_header(ses, "");
+	}
+	else
+	{
+		for (cnt = fnd = 0 ; cnt < LIST_MAX ; cnt++)
+		{
+			if (!is_abbrev(left, list_table[cnt].name_multi) && strcasecmp(left, "ALL"))
+			{
+				continue;
+			}
+			if (*right == 0)
+			{
+				TOG_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE);
+			}
+			else if (is_abbrev(right, "ON"))
+			{
+				SET_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE);
+			}
+			else if (is_abbrev(right, "OFF"))
+			{
+				DEL_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE);
+			}
+			else
+			{
+				tintin_printf(ses, "#SYNTAX: #MESSAGE [NAME] [ON|OFF]");
+				break;
+			}
+			tintin_printf(ses, "#OK: #%s MESSAGES HAVE BEEN SET TO: %s.", list_table[cnt].name, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE) ? "ON" : "OFF");
+			fnd = TRUE;
+		}
+
+		if (fnd == FALSE)
+		{
+			tintin_printf(ses, "#ERROR: #MESSAGE {%s} - NO MATCH FOUND.", left);
+		}
+	}
+	return ses;
+}
+
+
+DO_COMMAND(do_ignore)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	int cnt, fnd = FALSE;
+
+	arg = get_arg_in_braces(arg, left,  0);
+	arg = get_arg_in_braces(arg, right, 0);
+
+	if (*left == 0)
+	{
+		tintin_header(ses, " IGNORES ");
+
+		for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
+		{
+			tintin_printf2(ses, "  %-20s %3s", list_table[cnt].name_multi, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE) ? "ON" : "OFF");
+		}
+
+		tintin_header(ses, "");
+	}
+	else
+	{
+		for (cnt = fnd = 0 ; cnt < LIST_MAX ; cnt++)
+		{
+			if (!is_abbrev(left, list_table[cnt].name_multi) && strcasecmp(left, "ALL"))
+			{
+				continue;
+			}
+			if (*right == 0)
+			{
+				TOG_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE);
+			}
+			else if (is_abbrev(right, "ON"))
+			{
+				SET_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE);
+			}
+			else if (is_abbrev(right, "OFF"))
+			{
+				DEL_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE);
+			}
+			else
+			{
+				tintin_printf(ses, "#SYNTAX: #IGNORE [NAME] [ON|OFF]");
+				break;
+			}
+			tintin_printf(ses, "#OK: #%s IGNORE STATUS HAS BEEN SET TO: %s.", list_table[cnt].name, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE) ? "ON" : "OFF");
+			fnd = TRUE;
+		}
+
+		if (fnd == FALSE)
+		{
+			tintin_printf(ses, "#ERROR: #IGNORE {%s} - NO MATCH FOUND.", left);
+		}
+	}
+	return ses;
+}
+
+
+DO_COMMAND(do_debug)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	int cnt, fnd = FALSE;
+
+	arg = get_arg_in_braces(arg, left,  0);
+	arg = get_arg_in_braces(arg, right, 0);
+
+	if (*left == 0)
+	{
+		tintin_header(ses, " DEBUGS ");
+
+		for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
+		{
+			tintin_printf2(ses, "  %-20s %3s", list_table[cnt].name_multi, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG) ? "ON" : "OFF");
+		}
+
+		tintin_header(ses, "");
+	}
+	else
+	{
+		for (cnt = fnd = 0 ; cnt < LIST_MAX ; cnt++)
+		{
+			if (!is_abbrev(left, list_table[cnt].name_multi) && strcasecmp(left, "ALL"))
+			{
+				continue;
+			}
+			if (*right == 0)
+			{
+				TOG_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG);
+			}
+			else if (is_abbrev(right, "ON"))
+			{
+				SET_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG);
+			}
+			else if (is_abbrev(right, "OFF"))
+			{
+				DEL_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG);
+			}
+			else if (is_abbrev(right, "LOG"))
+			{
+				SET_BIT(ses->list[cnt]->flags, LIST_FLAG_LOG);
+			}
+			else
+			{
+				tintin_printf(ses, "#SYNTAX: #DEBUG [NAME] [ON|OFF|LOG]");
+				break;
+			}
+			tintin_printf(ses, "#OK: #%s DEBUG STATUS HAS BEEN SET TO: %s.", list_table[cnt].name, is_abbrev(right, "LOG") ? "LOG" : HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG) ? "ON" : "OFF");
+			fnd = TRUE;
+		}
+
+		if (fnd == FALSE)
+		{
+			tintin_printf2(ses, "#DEBUG {%s} - NO MATCH FOUND.", left);
+		}
+	}
+	return ses;
+}
