@@ -56,11 +56,11 @@ DO_COMMAND(do_log)
 	{
 		fclose(ses->logfile);
 		ses->logfile = NULL;
-		tintin_puts("#OK. LOGGING TURNED OFF.", ses);
+		tintin_printf(ses, "#OK: LOGGING TURNED OFF.");
 	}
 	else if (*left == 0 || *right == 0 || (!is_abbrev(left, "APPEND") && !is_abbrev(left, "OVERWRITE")))
 	{
-		tintin_puts2("#SYNTAX: #LOG [<APPEND|OVERWRITE> <FILENAME>]", ses);
+		tintin_printf(ses, "#SYNTAX: #LOG [<APPEND|OVERWRITE> <FILENAME>]");
 	}
 	else
 	{
@@ -68,23 +68,22 @@ DO_COMMAND(do_log)
 		{
 			if ((ses->logfile = fopen(right, "a")))
 			{
-				tintin_printf2(ses, "#OK. LOGGING OUTPUT TO '%s' FILESIZE: %ld", right, ftell(ses->logfile));
-
 				if (ftell(ses->logfile) == 0 && HAS_BIT(ses->flags, SES_FLAG_LOGHTML))
 				{
 					write_html_header(ses->logfile);
 				}
+				tintin_printf(ses, "#OK: LOGGING OUTPUT TO '%s' FILESIZE: %ld", right, ftell(ses->logfile));
 			}
 			else
 			{
-				tintin_printf2(ses, "#COULDN'T OPEN FILE '%s'", right);
+				tintin_printf(ses, "#ERROR: #LOG {%s} {%s} - COULDN'T OPEN FILE.", left, right);
 			}
 		}
 		else
 		{
 			if ((ses->logfile = fopen(right, "w")))
 			{
-				tintin_printf2(ses, "#OK. LOGGING OUTPUT TO '%s'", right);
+				tintin_printf(ses, "#OK: LOGGING OUTPUT TO '%s'", right);
 
 				if (HAS_BIT(ses->flags, SES_FLAG_LOGHTML))
 				{
@@ -93,7 +92,7 @@ DO_COMMAND(do_log)
 			}
 			else
 			{
-				tintin_printf2(ses, "#COULDN'T OPEN FILE '%s'", right);
+				tintin_printf(ses, "#ERROR: #LOG {%s} {%s} - COULDN'T OPEN FILE.", left, right);
 			}
 		}
 	}
@@ -132,7 +131,7 @@ DO_COMMAND(do_logline)
 	}
 	else
 	{
-		tintin_printf2(ses, "#LOGLINE: COULDN'T OPEN FILE '%s'", left);
+		tintin_printf(ses, "#ERROR: #LOGLINE {%s} - COULDN'T OPEN FILE.", left);
 	}
 	return ses;
 }
@@ -146,19 +145,15 @@ DO_COMMAND(do_writebuffer)
 
 	arg = get_arg_in_braces(arg, left, FALSE);
 
-	if (ses->buffer == 0)
+	if (*left == 0)
 	{
-		tintin_printf2(ses, "#ERROR: THIS SESSION HAS NO BUFFER.");
-	}
-	else if (*left == 0)
-	{
-		tintin_printf2(ses, "#SYNTAX: #WRITEBUFFER <FILENAME>]");
+		tintin_printf(ses, "#SYNTAX: #WRITEBUFFER <FILENAME>]");
 	}
 	else
 	{
 		if ((fp = fopen(left, "w")))
 		{
-			tintin_printf2(ses, "#OK. WRITING BUFFER TO '%s'", left);
+			tintin_printf(ses, "#OK: WRITING BUFFER TO '%s'", left);
 
 			if (HAS_BIT(ses->flags, SES_FLAG_LOGHTML))
 			{
@@ -204,7 +199,7 @@ DO_COMMAND(do_writebuffer)
 		}
 		else
 		{
-			tintin_printf2(ses, "#WRITEBUFFER, FAILED TO OPEN '%s'", left);
+			tintin_printf(ses, "#ERROR: #WRITEBUFFER {%s} - FAILED TO OPEN FILE.", left);
 		}
 	}
 	return ses;
@@ -216,7 +211,7 @@ void write_html_header(FILE *fp)
 		"<html>\n"
 		"<head>\n"
 		"<meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-1\">\n"
-		"<meta name=\"description\" content=\"Generated by TinTin++ 1.93 - http://www.scandum.com/tintin\">\n"
+		"<meta name=\"description\" content=\"Generated by TinTin++ 1.96 - http://tintin.sf.net\">\n"
 		"<style type=\"text/css\">\n"
 		"{\n\tfont-family: Courier;\n\tfont-size: 10pt;\n}\n"
 		".d30{ color: #000; } .l30{ color: #555; } .b40{ background-color: #000; }\n"
