@@ -47,7 +47,7 @@ DO_COMMAND(do_macro)
 	{
 		if (show_node_with_wild(ses, left, LIST_MACRO) == FALSE)
 		{
-			tintin_printf2(ses, "#MACRO, NO MATCH(ES) FOUND FOR {%s}", left);
+			show_message(ses, LIST_MACRO, "#MACRO: NO MATCH(ES) FOUND FOR {%s}.", left);
 		}
 	}
 	else
@@ -58,7 +58,7 @@ DO_COMMAND(do_macro)
 
 		rl_macro_bind(left, temp, gtd->keymap);
 
-		tintin_printf2(ses, "#OK {%s} MACROS {%s}", left, right);
+		show_message(ses, LIST_MACRO, "#OK. {%s} MACROS {%s}.", left, right);
 	}
 	return ses;
 }
@@ -69,7 +69,7 @@ DO_COMMAND(do_unmacro)
 	char left[BUFFER_SIZE];
 	struct listroot *root;
 	struct listnode *node;
-	int flag = FALSE;
+	int found = FALSE;
 
 	root = ses->list[LIST_MACRO];
 
@@ -77,18 +77,15 @@ DO_COMMAND(do_unmacro)
 
 	while ((node = search_node_with_wild(root, left)))
 	{
-		if (show_message(ses, LIST_MACRO))
-		{
-			tintin_printf(ses, "#OK {%s} IS NO LONGER A MACRO.", node->left);
-		}
+		show_message(ses, LIST_MACRO, "#OK {%s} IS NO LONGER A MACRO.", node->left);
 
 		deletenode_list(ses, node, LIST_MACRO);
 
-		flag = TRUE;
+		found = TRUE;
 	}
-	if (!flag && show_message(ses, LIST_MACRO))
+	if (found == FALSE)
 	{
-		tintin_printf2(ses, "#NO MATCH(ES) FOUND FOR {%s}", left);
+		show_message(ses, LIST_MACRO, "UNMACRO: #NO MATCH(ES) FOUND FOR {%s}", left);
 	}
 	return ses;
 }
