@@ -39,6 +39,19 @@
 struct session *gts;
 struct tintin_data *gtd;
 
+RETSIGTYPE pipe_handler(int no_care)
+{
+	tintin_printf(NULL, "broken_pipe: dumping stack");
+
+	dump_stack();
+
+	if (signal(SIGPIPE, pipe_handler) == BADSIG)
+	{
+		syserr("signal SIGPIPE");
+	}
+
+}
+
 /*
 	when the screen size changes, take note of it
 */
@@ -55,9 +68,9 @@ RETSIGTYPE winchhandler(int no_care)
 	{
 		init_screen_size(ses);
 
-		if (HAS_BIT(ses->flags, SES_FLAG_NAWS))
+		if (HAS_BIT(ses->telopts, TELOPT_FLAG_NAWS))
 		{
-			send_sb_naws(ses);
+			send_sb_naws(ses, 0, NULL);
 		}
 	}
 
@@ -71,31 +84,6 @@ RETSIGTYPE winchhandler(int no_care)
 	}
 }
 
-/*
-	CHANGED to get rid of double-echoing bug when tintin++ gets suspended
-*/
-
-RETSIGTYPE tstphandler(int no_care)
-{
-	printf("\e[r\e[%d;%dH", gtd->ses->rows, 1);
-
-	fflush(stdout);
-
-	kill(0, SIGSTOP);
-
-	dirty_screen(gtd->ses);
-
-	tintin_puts("#RETURNING BACK TO TINTIN++.", NULL);
-
-	/*
-		we haveta reinitialize the signals for sysv machines
-	*/
-
-	if (signal(SIGTSTP, tstphandler) == BADSIG)
-	{
-		syserr("signal SIGTSTP");
-	}
-}
 
 void trap_handler(int signal)
 {
@@ -125,7 +113,6 @@ void trap_handler(int signal)
 int main(int argc, char **argv)
 {
 	char filestring[256];
-	int arg_num;
 
 	#ifdef SOCKS
 		SOCKSinit(argv[0]);
@@ -146,6 +133,10 @@ int main(int argc, char **argv)
 		syserr("signal SIGSEGV");
 	}
 
+	if (signal(SIGHUP, trap_handler) == BADSIG)
+	{
+		syserr("signal SIGHUP");
+	}
 
 	if (signal(SIGABRT, myquitsig) == BADSIG)
 	{
@@ -157,57 +148,47 @@ int main(int argc, char **argv)
 		syserr("signal SIGINT");
 	}
 
-	if (signal(SIGTSTP, tstphandler) == BADSIG)
+	if (signal(SIGPIPE, pipe_handler) == BADSIG)
 	{
-		syserr("signal SIGTSTP");
+		syserr("signal SIGPIPE");
 	}
 
-
 	if (signal(SIGWINCH, winchhandler) == BADSIG)
 	{
 		syserr("signal SIGWINCH");
 	}
 
-/*
-	if (signal(SIGALRM, tick_handler) == BADSIG)
-	{
-		syserr("signal SIGALRM");
-	}
-*/
-
 	if (getenv("HOME") != NULL)
 	{
-		if (getenv("TINTIN_HISTORY") == NULL)
-		{
-			sprintf(filestring, "%s/%s", getenv("HOME"), HISTORY_FILE);
-		}
-		else
-		{
-			sprintf(filestring, "%s/%s", getenv("TINTIN_HISTORY"), HISTORY_FILE);
-		}
+		sprintf(filestring, "%s/%s", getenv("HOME"), HISTORY_FILE);
+
 		read_history(filestring);
 
 		add_history("");
 	}
 	srand48(time(NULL));
 
-	arg_num = 1;
-
-	if (argc > 1 && argv[1])
+	if (argc > 1)
 	{
-		if (*argv[1] == '-' && argv[1][1] == 'v')
+		if (!strcmp(argv[1], "-v"))
 		{
-			arg_num = 2;
+			do_configure(gtd->ses, "{VERBOSE} {ON}");
 
-			SET_BIT(gts->flags, SES_FLAG_VERBOSE);
+			do_read(gtd->ses, argv[2]);
+		}
+		else if (!strcmp(argv[1], "-e"))
+		{
+			gtd->ses = parse_input(argv[2], gtd->ses);
+		}
+		else if (!strcmp(argv[1], "-h"))
+		{
+			tintin_printf(NULL, "Usage: %s [file] [-v file] [-e \"command\"]", argv[0]);
+		}
+		else
+		{
+			gtd->ses = do_read(gtd->ses, argv[1]);
 		}
 	}
-
-	if (argc > arg_num && argv[arg_num])
-	{
-		gtd->ses = do_read(gts, argv[arg_num]);
-	}
-
 	commandloop();
 
 	pop_call();
@@ -217,7 +198,7 @@ int main(int argc, char **argv)
 
 void init_tintin(void)
 {
-	int cnt;
+	int ref, cnt;
 
 	gts                 = calloc(1, sizeof(struct session));
 
@@ -229,7 +210,7 @@ void init_tintin(void)
 	gts->name           = strdup("gts");
 	gts->host           = strdup("");
 	gts->port           = strdup("");
-	gts->flags          = SES_FLAG_LOCALECHO;
+	gts->telopts        = TELOPT_FLAG_ECHO;
 	gts->map_size       = 6;
 	gts->scroll_line    = -1;
 
@@ -244,24 +225,34 @@ void init_tintin(void)
 	gtd->mud_output_max = 64;
 	gtd->mud_output_buf = calloc(1, gtd->mud_output_max);
 
+	for (ref = 0 ; ref < 26 ; ref++)
+	{
+		for (cnt = 0 ; *command_table[cnt].name != 0 ; cnt++)
+		{
+			if (*command_table[cnt].name == 'a' + ref)
+			{
+				gtd->command_ref[ref] = cnt;
+				break;
+			}
+		}
+	}
+
 	initrl();
 
 	init_screen_size(gts);
 
-/*	dirty_screen(gts); */
-
 	/*
 		Set application keypad mode and  ESC 0 prefix
 	*/
 
-	printf("\e=\e[?1h");
+	printf("\033=\033[?1h");
 
 	SET_BIT(gts->flags, SES_FLAG_VERBOSE);
 
-	do_configure(gts, "{SPEEDWALK}          {ON}");
+	do_configure(gts, "{SPEEDWALK}         {OFF}");
 	do_configure(gts, "{VERBATIM}          {OFF}");
 	do_configure(gts, "{REPEAT ENTER}      {OFF}");
-	do_configure(gts, "{ECHO COMMAND}      {OFF}");
+	do_configure(gts, "{ECHO COMMAND}       {ON}");
 	do_configure(gts, "{VERBOSE}           {OFF}");
 	do_configure(gts, "{WORDWRAP}           {ON}");
 	do_configure(gts, "{LOG}               {RAW}");
