@@ -61,6 +61,23 @@ DO_COMMAND(do_redit)
 	arg = get_arg_in_braces(arg, left,  FALSE);
 	arg = get_arg_in_braces(arg, right, TRUE);
 
+	if (!strcasecmp(left, ""))
+	{
+		tintin_printf2(ses, "Available redit options");
+		tintin_printf2(ses, "#redit list                          (shows list of created rooms)");
+		tintin_printf2(ses, "#redit goto <room name>              (moves you to given room)");
+		tintin_printf2(ses, "#redit map <radius>                  (shows an ascii map)");
+		tintin_printf2(ses, "#redit create <direction>            (creates a new room)");
+		tintin_printf2(ses, "#redit link <direction> <room name>  (links 2 rooms)");
+		tintin_printf2(ses, "#redit name <room name>              (set the room's name)");
+		tintin_printf2(ses, "#redit color <color code>            (set the room's color for map)");
+		tintin_printf2(ses, "#redit find <room name>              (#map and #walk to use)");
+		tintin_printf2(ses, "#redit load <filename>               (load your map from a file)");
+		tintin_printf2(ses, "#redit save <filename>               (save your map to a file)");
+
+		return ses;
+	}
+
 	if (!strcasecmp(left, "goto"))
 	{
 		room = find_room(ses, right);
@@ -82,6 +99,20 @@ DO_COMMAND(do_redit)
 		return ses;
 	}
 
+	if (!strcasecmp(left, "load"))
+	{
+		do_readmap(ses, right);
+
+		return ses;
+	}
+
+	if (!strcasecmp(left, "save"))
+	{
+		do_writemap(ses, right);
+
+		return ses;
+	}
+
 	if (ses->room_list[ses->in_room] == NULL)
 	{
 		tintin_printf2(ses, "Use #redit goto first, or use #redit firstroom.");
@@ -117,6 +148,8 @@ DO_COMMAND(do_redit)
 		if (exit == NULL)
 		{
 			tintin_printf2(ses, "No exit with that name found");
+
+			return ses;
 		}
 
 		room = exit->vnum;
@@ -129,7 +162,7 @@ DO_COMMAND(do_redit)
 				{
 					if (exit->vnum == room)
 					{
-						UNLINK(exit, ses->room_list[new_room]->f_exit, ses->room_list[new_room]->l_exit, next, prev);
+						UNLINK(exit, ses->room_list[new_room]->f_exit, ses->room_list[new_room]->l_exit);
 						break;
 					}
 				}
@@ -258,18 +291,7 @@ DO_COMMAND(do_redit)
 		tintin_printf2(ses, "Created new room.");
 	}
 
-	if (!strcasecmp(left, ""))
-	{
-		tintin_printf2(ses, "Available redit options");
-		tintin_printf2(ses, "#redit list                          (shows list of created rooms)");
-		tintin_printf2(ses, "#redit goto <room name>              (moves you to given room)");
-		tintin_printf2(ses, "#redit map <radius>                  (shows an ascii map)");
-		tintin_printf2(ses, "#redit create <direction>            (creates a new room)");
-		tintin_printf2(ses, "#redit link <direction> <room name>  (links 2 rooms)");
-		tintin_printf2(ses, "#redit name <room name>              (set the room's name)");
-		tintin_printf2(ses, "#redit color <color code>            (set the room's color for map)");
-		tintin_printf2(ses, "#redit find <room name>              (#map and #walk to use)");
-	}
+	do_redit(ses, "");
 
 	return ses;
 }
@@ -313,7 +335,7 @@ void create_exit(struct session *ses, const char *arg)
 	newexit->name = strdup(name);
 	newexit->cmd  = strdup(cmd);
 
-	LINK(newexit, ses->room_list[ses->in_room]->f_exit, ses->room_list[ses->in_room]->l_exit, next, prev);
+	LINK(newexit, ses->room_list[ses->in_room]->f_exit, ses->room_list[ses->in_room]->l_exit);
 
 	tintin_printf2(ses, "#READMAP E %5s {%s} {%s}", vnum, name, cmd);
 }
@@ -521,12 +543,13 @@ void show_map(struct session *ses, char *argument)
 
 	for (y = temp_map_y ; y >= 0 ; y--)
 	{
-		buf[0] = 0;
+		strcpy(buf, "{");
 
 		for (x = 0 ; x <= temp_map_x ; x++)
 		{
 			strcat(buf, draw_room(ses, temp_map[x][y]));
 		}
+		strcat(buf, "}");
 
 		do_showme(ses, buf);
 	}
@@ -539,6 +562,7 @@ void show_map(struct session *ses, char *argument)
 	{
 		case 0:
 			break;
+
 		default:
 			tintin_printf2(ses, "\033(0%s\033(B", buf);
 			break;
@@ -559,7 +583,7 @@ void show_vtmap(struct session *ses)
 	memset(buf, 0, BUFFER_SIZE);
 
 	temp_map_y = (ses->top_row - 3);
-	temp_map_x = (ses->top_row - 3) * 3;
+	temp_map_x = (ses->cols - 1);
 
 	create_map(ses, ses->in_room, temp_map_x, temp_map_y);
 
@@ -614,7 +638,7 @@ char *draw_room(struct session *ses, struct room_data *room)
 	if (room->vnum == ses->in_room)
 	{
 		pop_call();
-		return "\033[1;31mX\033[0m";
+		return "<118>X<088>";
 	}
 
 	for (exit = room->f_exit ; exit ; exit = exit->next)
@@ -857,3 +881,90 @@ void search_coord(int vnum, short x, short y, short z)
 		}
 	}
 }
+
+DO_COMMAND(do_readmap)
+{
+	FILE *myfile;
+	char filename[BUFFER_SIZE], buffer[BUFFER_SIZE], *cptr;
+
+	get_arg_in_braces(arg, filename, TRUE);
+
+	if ((myfile = fopen(filename, "r")) == NULL)
+	{
+		tintin_printf(ses, "#ERROR: #READMAP {%s} - FILE NOT FOUND.", filename);
+		return(ses);
+	}
+
+	SET_BIT(gts->flags, SES_FLAG_QUIET);
+
+	while (fgets(buffer, BUFFER_SIZE - 1, myfile))
+	{
+		for (cptr = buffer ; *cptr && *cptr != '\n' ; cptr++);
+
+		*cptr = 0;
+
+		switch (buffer[0])
+		{
+			case 'R':
+				ses->in_room = create_room(ses, &buffer[2]);
+				break;
+			case 'E':
+				create_exit(ses, &buffer[2]);
+				break;
+		}
+	}
+
+	DEL_BIT(gts->flags, SES_FLAG_QUIET);
+
+	fclose(myfile);
+
+	show_message(ses, -1, "#OK. FILE READ.");
+
+	return(ses);
+}
+
+DO_COMMAND(do_writemap)
+{
+	FILE *file;
+	char filename[BUFFER_SIZE], buffer[BUFFER_SIZE];
+	struct exit_data *exit;
+	int vnum;
+
+	get_arg_in_braces(arg, filename, TRUE);
+
+	if (*buffer == 0 || (file = fopen(filename, "w")) == NULL)
+	{
+		tintin_printf(ses, "#ERROR: #WRITEMAP {%s} - COULDN'T OPEN FILE TO WRITE.", filename);
+		return ses;
+	}
+
+	for (vnum = 0 ; vnum < MAX_ROOM ; vnum++)
+	{
+		if (ses->room_list[vnum])
+		{
+			sprintf(buffer, "R {%5d} {%d} {%s} {%s}\n",
+				ses->room_list[vnum]->vnum,
+				ses->room_list[vnum]->flags,
+				ses->room_list[vnum]->color,
+				ses->room_list[vnum]->name);
+
+			fputs(buffer, file);
+
+			for (exit = ses->room_list[vnum]->f_exit ; exit ; exit = exit->next)
+			{
+				sprintf(buffer, "E {%5d} {%s} {%s}\n",
+					exit->vnum,
+					exit->name,
+					exit->cmd);
+
+				fputs(buffer, file);
+			}
+		}
+	}
+
+	fclose(file);
+
+	show_message(ses, -1, "#OK. FILE WRITTEN.");
+
+	return ses;
+}
