@@ -50,6 +50,7 @@
 struct listnode *mathnode_s;
 struct listnode *mathnode_e;
 struct session  *mathses;
+int precision;
 
 DO_COMMAND(do_math)
 {
@@ -63,11 +64,11 @@ DO_COMMAND(do_math)
 
 	if (*left == 0 || *right == 0)
 	{
-		tintin_printf2(ses, "#SYNTAX: #MATH {variable} {expression}");
+		tintin_printf(ses, "#SYNTAX: #MATH {variable} {expression}");
 	}
 	else
 	{
-		sprintf(result, "{%s} {%lld}", left, mathexp(ses, right));
+		sprintf(result, "{%s} {%.*f}", left, precision, mathexp(ses, right));
 
 		do_variable(ses, result);
 	}
@@ -85,7 +86,7 @@ DO_COMMAND(do_if)
 
 	if (*left == 0 || *true == 0)
 	{
-		tintin_printf2(ses, "#SYNTAX: #IF {expression} {true} {false}");
+		tintin_printf(ses, "#SYNTAX: #IF {expression} {true} {false}");
 	}
 	else
 	{
@@ -103,11 +104,32 @@ DO_COMMAND(do_if)
 	return ses;
 }
 
+double get_number(struct session *ses, const char *str)
+{
+	char number[BUFFER_SIZE];
+
+	substitute(ses, str, number, SUB_VAR|SUB_FUN);
+	
+	return mathexp(ses, number);
+}
+
+char *get_number_string(struct session *ses, const char *str)
+{
+	char number[BUFFER_SIZE];
+	static char result[BUFFER_SIZE];
+
+	substitute(ses, str, number, SUB_VAR|SUB_FUN);	
+
+	sprintf(result, "%.*f", precision, mathexp(ses, number));
+
+	return result;
+}
+
 /******************************************************************************
 * mathematical expression interpreter by Scandum                              *
 ******************************************************************************/
 
-long long mathexp(struct session *ses, const char *str)
+double mathexp(struct session *ses, const char *str)
 {
 	struct listnode *node;
 
@@ -115,10 +137,8 @@ long long mathexp(struct session *ses, const char *str)
 
 	if (mathexp_tokenize(ses, str) == FALSE)
 	{
-		if (ses->list[LIST_MATH]->f_node == NULL)
-		{
-			tintin_printf2(ses, "#MATH EXP: INVALID INPUT: %s", str);
-		}
+		tintin_printf(ses, "#MATH EXP: INVALID INPUT {%s}", str);
+
 		return 0;
 	}
 
@@ -142,7 +162,7 @@ long long mathexp(struct session *ses, const char *str)
 
 #define MATH_NODE(buffercheck, priority, newstatus)               \
 {                                                                 \
-	if (buffercheck && pta == buf3)                              \
+	if (buffercheck && (pta == buf3 || valid == FALSE))          \
 	{                                                            \
 		return FALSE;                                           \
 	}                                                            \
@@ -152,15 +172,20 @@ long long mathexp(struct session *ses, const char *str)
 	insertnode_list(mathses, buf1, buf2, buf3, LIST_MATH);       \
 	status = newstatus;                                          \
 	pta = buf3;                                                  \
+	valid = FALSE;                                               \
+	point = -1;                                                  \
 }
 
 int mathexp_tokenize(struct session *ses, const char *str)
 {
 	char buf1[BUFFER_SIZE], buf2[BUFFER_SIZE], buf3[BUFFER_SIZE], *pti, *pta;
-	int level, status;
+	int level, status, valid, point;
 
-	level  = 0;
-	status = EXP_VARIABLE;
+	level     = 0;
+	point     = -1;
+	status    = EXP_VARIABLE;
+	valid     = FALSE;
+	precision = 0;
 
 	pta = (char *) buf3;
 	pti = (char *) str;
@@ -174,16 +199,34 @@ int mathexp_tokenize(struct session *ses, const char *str)
 			case EXP_VARIABLE:
 				switch (*pti)
 				{
-					case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
+					case '0':
+					case '1':
+					case '2':
+					case '3':
+					case '4':
+					case '5':
+					case '6':
+					case '7':
+					case '8':
+					case '9':
 						*pta++ = *pti++;
+						valid = TRUE;
+
+						if (point >= 0)
+						{
+							point++;
+
+							if (precision < point)
+							{
+								precision = point;
+							}
+						}
 						break;
 
 					case '!':
 					case '~':
 					case '+':
 					case '-':
-					case 'T':
-					case 'F':
 						if (pta != buf3)
 						{
 							MATH_NODE(FALSE, EXP_PR_VAR, EXP_OPERATOR);
@@ -211,10 +254,19 @@ int mathexp_tokenize(struct session *ses, const char *str)
 							return FALSE;
 						}
 						*pta++ = *pti++;
-						MATH_NODE(TRUE, EXP_PR_LVL, EXP_VARIABLE);
+						MATH_NODE(FALSE, EXP_PR_LVL, EXP_VARIABLE);
 						level++;
 						break;
 
+					case ',':
+						*pti++;
+						break;
+
+					case '.':
+						*pta++ = *pti++;
+						point++;
+						break;
+
 					case ' ':
 						pti++;
 						break;
@@ -230,6 +282,7 @@ int mathexp_tokenize(struct session *ses, const char *str)
 				{
 					case '"':
 						*pta++ = *pti++;
+						valid = TRUE;
 						MATH_NODE(FALSE, EXP_PR_VAR, EXP_OPERATOR);
 						break;
 
@@ -249,7 +302,7 @@ int mathexp_tokenize(struct session *ses, const char *str)
 					case ')':
 						*pta++ = *pti++;
 						level--;
-						MATH_NODE(TRUE, EXP_PR_LVL, EXP_OPERATOR);
+						MATH_NODE(FALSE, EXP_PR_LVL, EXP_OPERATOR);
 						break;
 
 					case 'd':
@@ -341,6 +394,7 @@ int mathexp_tokenize(struct session *ses, const char *str)
 							default:
 								MATH_NODE(FALSE, EXP_PR_BITXOR, EXP_VARIABLE);
 								break;
+
 						}
 						break;
 
@@ -351,12 +405,11 @@ int mathexp_tokenize(struct session *ses, const char *str)
 						{
 							case '|':
 								*pta++ = *pti++;
-								MATH_NODE(FALSE, 7, EXP_VARIABLE);
+								MATH_NODE(FALSE, EXP_PR_LOGOR, EXP_VARIABLE);
 								break;
 
 							default:
-								*pta++ = *pti++;
-								MATH_NODE(FALSE, 5, EXP_VARIABLE);
+								MATH_NODE(FALSE, EXP_PR_BITOR, EXP_VARIABLE);
 								break;
 						}
 						break;
@@ -372,7 +425,7 @@ int mathexp_tokenize(struct session *ses, const char *str)
 								break;
 
 							default:
-								tintin_printf2(ses, "#MATH EXP: UNKNOWN OPERATOR: %c%c", pti[0], pti[1]);
+								tintin_printf2(ses, "#MATH EXP: UNKNOWN OPERATOR: %c%c", pti[-1], pti[0]);
 								return FALSE;
 						}
 						break;
@@ -395,6 +448,7 @@ int mathexp_tokenize(struct session *ses, const char *str)
 	{
 		MATH_NODE(TRUE, EXP_PR_VAR, EXP_OPERATOR);
 	}
+
 	return TRUE;
 }
 
@@ -454,7 +508,7 @@ void mathexp_level(struct listnode *node)
 void mathexp_compute(struct listnode *node)
 {
 	char temp[BUFFER_SIZE];
-	long long value;
+	double value;
 
 	switch (node->pr[0])
 	{
@@ -476,22 +530,29 @@ void mathexp_compute(struct listnode *node)
 			if (tintoi(node->next->pr) == 0)
 			{
 				tintin_printf(NULL, "#MATH ERROR: DIVISION ZERO.");
-				value = tintoi(node->prev->pr) / 1;
+				value = tintoi(node->prev->pr);
 			}
 			else
 			{
-				value = tintoi(node->prev->pr) / tintoi(node->next->pr);
+				if (precision)
+				{
+					value = tintoi(node->prev->pr) / tintoi(node->next->pr);
+				}
+				else
+				{
+					value = (long long) tintoi(node->prev->pr) / (long long) tintoi(node->next->pr);
+				}
 			}
 			break;
 		case '%':
 			if (tintoi(node->next->pr) == 0)
 			{
 				tintin_printf(NULL, "#MATH ERROR: MODULO ZERO.");
-				value = tintoi(node->prev->pr) / 1;
+				value = tintoi(node->prev->pr);
 			}
 			else
 			{
-				value = tintoi(node->prev->pr) % tintoi(node->next->pr);
+				value = (long long) tintoi(node->prev->pr) % (long long) tintoi(node->next->pr);
 			}
 			break;
 		case '+':
@@ -507,7 +568,7 @@ void mathexp_compute(struct listnode *node)
 					value = tincmp(node->prev->pr, node->next->pr) <= 0;
 					break;
 				case '<':
-					value = tintoi(node->prev->pr) << tintoi(node->next->pr);
+					value = (long long) tintoi(node->prev->pr) << (long long) tintoi(node->next->pr);
 					break;
 				default:
 					value = tincmp(node->prev->pr, node->next->pr) < 0;
@@ -521,7 +582,7 @@ void mathexp_compute(struct listnode *node)
 					value = tincmp(node->prev->pr, node->next->pr) >= 0;
 					break;
 				case '>':
-					value = tintoi(node->prev->pr) >> tintoi(node->next->pr);
+					value = (long long) tintoi(node->prev->pr) >> (long long) tintoi(node->next->pr);
 					break;
 				default:
 					value = tincmp(node->prev->pr, node->next->pr) > 0;
@@ -536,7 +597,7 @@ void mathexp_compute(struct listnode *node)
 					value = tintoi(node->prev->pr) && tintoi(node->next->pr);
 					break;
 				default:
-					value = tintoi(node->prev->pr) & tintoi(node->next->pr);
+					value = (long long) tintoi(node->prev->pr) & (long long) tintoi(node->next->pr);
 					break;
 			}
 			break;
@@ -548,7 +609,7 @@ void mathexp_compute(struct listnode *node)
 					break;
 
 				default:
-					value = tintoi(node->prev->pr) ^ tintoi(node->next->pr);
+					value = (long long) tintoi(node->prev->pr) ^ (long long) tintoi(node->next->pr);
 					break;
 			}
 			break;
@@ -559,15 +620,15 @@ void mathexp_compute(struct listnode *node)
 					value = tintoi(node->prev->pr) || tintoi(node->next->pr);
 					break;
 				default:
-					value = tintoi(node->prev->pr) | tintoi(node->next->pr);
+					value = (long long) tintoi(node->prev->pr) | (long long) tintoi(node->next->pr);
 					break;
 			}
 			break;
 		case '=':
-			value = (regexp(node->next->pr, node->prev->pr) != 0);
+			value = (tineval(node->next->pr, node->prev->pr) != 0);
 			break;
 		case '!':
-			value = (regexp(node->next->pr, node->prev->pr) == 0);
+			value = (tineval(node->next->pr, node->prev->pr) == 0);
 			break;
 		default:
 			tintin_printf2(NULL, "#COMPUTE EXP: UNKNOWN OPERATOR: %c%c", node->pr[0], node->pr[1]);
@@ -592,40 +653,33 @@ void mathexp_compute(struct listnode *node)
 	free(node->right);
 	node->right = strdup(temp);
 
-	sprintf(temp, "%lld", value);
+	sprintf(temp, "%f", value);
 	free(node->pr);
 	node->pr = strdup(temp);
 }
 
-long long tintoi(const char *str)
+double tintoi(const char *str)
 {
-
 	switch (str[0])
 	{
 		case '!':
-			return !atoll(&str[1]);
+			return !atof(&str[1]);
 
 		case '~':
-			return ~atoll(&str[1]);
+			return (double) ~atoll(&str[1]);
 
 		case '+':
-			return +atoll(&str[1]);
+			return +atof(&str[1]);
 
 		case '-':
-			return -atoll(&str[1]);
-
-		case 'F':
-			return 0;
-
-		case 'T':
-			return 1;
+			return -atof(&str[1]);
 
 		default:
-			return atoll(str);
+			return atof(str);
 	}
 }
 
-long long tincmp(const char *left, const char *right)
+double tincmp(const char *left, const char *right)
 {
 	switch (left[0])
 	{
@@ -637,16 +691,28 @@ long long tincmp(const char *left, const char *right)
 	}
 }
 
-long long tindice(const char *left, const char *right)
+double tineval(const char *left, const char *right)
+{
+	switch (left[0])
+	{
+		case '"':
+			return regexp(left, right);
+
+		default:
+			return tintoi(left) == tintoi(right);
+	}
+}
+			
+double tindice(const char *left, const char *right)
 {
 	long long cnt, numdice, sizedice, sum;
 
-	numdice  = tintoi(left);
-	sizedice = tintoi(right);
+	numdice  = (long long) tintoi(left);
+	sizedice = (long long) tintoi(right);
 
 	for (cnt = sum = 0 ; cnt < numdice ; cnt++)
 	{
 		sum += lrand48() % sizedice + 1;
 	}
-	return sum;
+	return (double) sum;
 }
