@@ -26,11 +26,40 @@
 /*                     coded by peter unold 1992                     */
 /*********************************************************************/
 
-/* note: a bunch of changes were made for readline support -- daw */
-
 #include "tintin.h"
 
 
+DO_COMMAND(do_all)
+{
+	char left[BUFFER_SIZE];
+	struct session *sesptr, *next_ses;
+
+	if (gts->next)
+	{
+		get_arg_in_braces(arg, left, TRUE);
+
+		for (sesptr = gts->next ; sesptr ; sesptr = next_ses)
+		{
+			next_ses = sesptr->next;
+			parse_input(left, sesptr);
+		}
+	}
+	else
+	{
+		tintin_puts2("#BUT THERE ISN'T ANY SESSION AT ALL!", ses);
+	}
+	return ses;
+}
+
+
+DO_COMMAND(do_bell)
+{
+	printf("\007");
+
+	return ses;
+}
+
+
 DO_COMMAND(do_commands)
 {
 	char buf[BUFFER_SIZE] = { 0 }, add[BUFFER_SIZE];
@@ -62,101 +91,14 @@ DO_COMMAND(do_commands)
 }
 
 
-
 DO_COMMAND(do_cr)
 {
-	write_line_mud(" ", ses);
+	write_line_mud("\r\n", ses);
 
 	return ses;
 }
 
 
-DO_COMMAND(do_nop)
-{
-	return ses;
-}
-
-
-DO_COMMAND(do_suspend)
-{
-	tstphandler(0);
-
-	return ses;
-}
-
-
-/********************/
-/* the #all command */
-/********************/
-
-DO_COMMAND(do_all)
-{
-	char left[BUFFER_SIZE];
-	struct session *sesptr, *next_ses;
-
-	if (gts->next)
-	{
-		get_arg_in_braces(arg, left, TRUE);
-
-		for (sesptr = gts->next ; sesptr ; sesptr = next_ses)
-		{
-			next_ses = sesptr->next;
-			parse_input(left, sesptr);
-		}
-	}
-	else
-	{
-		tintin_puts2("#BUT THERE ISN'T ANY SESSION AT ALL!", ses);
-	}
-	return ses;
-}
-
-/*********************/
-/* the #bell command */
-/*********************/
-
-DO_COMMAND(do_bell)
-{
-	printf("\007");
-
-	return ses;
-}
-
-/*********************/
-/* the #boss command */
-/*********************/
-
-DO_COMMAND(do_boss)
-{
-	int i;
-
-	for (i = 0 ; i < 50 ; i++)
-	{
-		tintin_printf2(ses, "in-order traverse of tree starting from node %2d resulted in %2d red nodes\n", i, 50-i);
-	}
-	/*
-		stop screen from scrolling stuff
-	*/
-	getchar();
-
-	return ses;
-}
-
-
-DO_COMMAND(do_end)
-{
-	if (*arg)
-	{
-		tintin_puts("#YOU HAVE TO WRITE #END - NO LESS, TO END!", ses);
-	}
-	else
-	{
-		quitmsg(NULL);
-	}
-	return NULL;
-}
-
-
 DO_COMMAND(do_echo)
 {
 	char temp[BUFFER_SIZE];
@@ -174,29 +116,83 @@ DO_COMMAND(do_echo)
 	return ses;
 }
 
-DO_COMMAND(do_showme)
+
+DO_COMMAND(do_end)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], result[BUFFER_SIZE];
-
-	arg = get_arg_in_braces(arg, left, TRUE);
-	arg = get_arg_in_braces(arg, right, FALSE);
-
-	substitute(ses, left, result, SUB_VAR|SUB_FUN|SUB_COL|SUB_ESC);
-
-	if (*right)
+	if (*arg)
 	{
-		do_one_prompt(ses, result, atoi(right));
+		tintin_puts("#ERROR: USE #END WITHOUT AN ARGUMENT TO END.", ses);
 	}
 	else
 	{
-		tintin_puts(result, ses);
+		quitmsg(NULL);
+	}
+	return NULL;
+}
+
+
+DO_COMMAND(do_forall)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
+
+	arg = get_arg_in_braces(arg, temp,  TRUE);
+	arg = get_arg_in_braces(arg, right, TRUE);
+
+	substitute(ses, temp, left, SUB_VAR|SUB_FUN);
+
+	if (*left == 0 || *right == 0)
+	{
+		tintin_printf2(ses, "#ERROR: #FORALL - PROVIDE 2 ARGUMENTS");
+	}
+	else
+	{
+		arg = left;
+
+		while (*arg)
+		{
+			arg = get_arg_in_braces(arg, temp, FALSE);
+			sprintf(gtd->cmds[0], "%s", temp);
+
+			sprintf(temp, "forall %s", gtd->cmds[0]);
+			do_internal_variable(ses, temp);
+
+			substitute(ses, right, temp, SUB_CMD);
+
+			ses = parse_input(temp, ses);
+		}
 	}
 	return ses;
 }
 
-/*********************/
-/* the #loop command */
-/*********************/
+
+DO_COMMAND(do_gagline)
+{
+	SET_BIT(ses->flags, SES_FLAG_GAG);
+
+	return ses;
+}
+
+
+DO_COMMAND(do_info)
+{
+	int cnt;
+
+	tintin_header(ses, " INFORMATION ");
+
+	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
+	{
+		tintin_printf2(ses, "%-20s %3d  IGNORE %3s  MESSAGE %3s  DEBUG  %3s",
+			list_table[cnt].name_multi,
+			ses->list[cnt]->count,
+			HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE) ? "ON" : "OFF",
+			HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE) ? "ON" : "OFF",
+			HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG)   ? "ON" : "OFF");
+	}
+	tintin_header(ses, "");
+
+	return ses;
+}
+
 
 DO_COMMAND(do_loop)
 {
@@ -211,7 +207,7 @@ DO_COMMAND(do_loop)
 
 	if (sscanf(left, "%d%c%d", &bound1, &temp[0], &bound2) != 3)
 	{
-		tintin_printf2(ses, "#LOOP: NEED 2 ARGUMENTS, FOUND: %s", left);
+		tintin_printf(ses, "#ERROR: #LOOP - NEED 2 ARGUMENTS, FOUND: %s", left);
 	}
 	else
 	{
@@ -241,33 +237,36 @@ DO_COMMAND(do_loop)
 }
 
 
-DO_COMMAND(do_forall)
+DO_COMMAND(do_nop)
+{
+	return ses;
+}
+
+
+DO_COMMAND(do_parse)
 {
 	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE];
+	int cnt;
 
-	arg = get_arg_in_braces(arg, temp,  TRUE);
-	arg = get_arg_in_braces(arg, right, FALSE);
-
+	arg = get_arg_in_braces(arg, temp, 0);
 	substitute(ses, temp, left, SUB_VAR|SUB_FUN);
 
+	arg = get_arg_in_braces(arg, right, 1);
+
 	if (*left == 0 || *right == 0)
 	{
-		tintin_printf2(ses, "#FORALL, PROVIDE 2 ARGUMENTS");
+		tintin_printf2(ses, "#ERROR: #PARSE - PROVIDE 2 ARGUMENTS");
 	}
 	else
 	{
-		arg = left;
-
-		while (*arg)
+		for (cnt = 0 ; left[cnt] != 0 ; cnt++)
 		{
-			arg = get_arg_in_braces(arg, temp, FALSE);
-			sprintf(gtd->cmds[0], "%s", temp);
+			sprintf(gtd->cmds[0], "%c", left[cnt]);
 
-			sprintf(temp, "forall %s", gtd->cmds[0]);
+			sprintf(temp, "parse {%c}", left[cnt]);
 			do_internal_variable(ses, temp);
 
 			substitute(ses, right, temp, SUB_CMD);
-
 			ses = parse_input(temp, ses);
 		}
 	}
@@ -275,175 +274,6 @@ DO_COMMAND(do_forall)
 }
 
 
-DO_COMMAND(do_message)
-{
-	char left[BUFFER_SIZE], right[BUFFER_SIZE];
-	int cnt, fnd = FALSE;
-
-	arg = get_arg_in_braces(arg, left,  0);
-	arg = get_arg_in_braces(arg, right, 0);
-
-	if (*left == 0)
-	{
-		tintin_header(ses, " MESSAGES ");
-
-		for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
-		{
-			tintin_printf2(ses, "  %-20s %3s", list_table[cnt].name_multi, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE) ? "ON" : "OFF");
-		}
-
-		tintin_header(ses, "");
-	}
-	else
-	{
-		for (cnt = fnd = 0 ; cnt < LIST_MAX ; cnt++)
-		{
-			if (!is_abbrev(left, list_table[cnt].name_multi))
-			{
-				continue;
-			}
-			if (*right == 0)
-			{
-				TOG_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE);
-			}
-			else if (is_abbrev(right, "ON"))
-			{
-				SET_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE);
-			}
-			else if (is_abbrev(right, "OFF"))
-			{
-				DEL_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE);
-			}
-			else
-			{
-				tintin_puts2("SYNTAX: #MESSAGE [NAME] [ON|OFF]", ses);
-				break;
-			}
-			tintin_printf2(ses, "#%s MESSAGES HAVE BEEN SET TO: %s.", list_table[cnt].name, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE) ? "ON" : "OFF");
-			fnd = TRUE;
-		}
-
-		if (fnd == FALSE)
-		{
-			tintin_printf2(ses, "#NO MATCH(es) found for {%s}", left);
-		}
-	}
-	return ses;
-}
-
-
-DO_COMMAND(do_ignore)
-{
-	char left[BUFFER_SIZE], right[BUFFER_SIZE];
-	int cnt, fnd = FALSE;
-
-	arg = get_arg_in_braces(arg, left,  0);
-	arg = get_arg_in_braces(arg, right, 0);
-
-	if (*left == 0)
-	{
-		tintin_header(ses, " IGNORES ");
-
-		for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
-		{
-			tintin_printf2(ses, "  %-20s %3s", list_table[cnt].name_multi, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE) ? "ON" : "OFF");
-		}
-
-		tintin_header(ses, "");
-	}
-	else
-	{
-		for (cnt = fnd = 0 ; cnt < LIST_MAX ; cnt++)
-		{
-			if (!is_abbrev(left, list_table[cnt].name_multi))
-			{
-				continue;
-			}
-			if (*right == 0)
-			{
-				TOG_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE);
-			}
-			else if (is_abbrev(right, "ON"))
-			{
-				SET_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE);
-			}
-			else if (is_abbrev(right, "OFF"))
-			{
-				DEL_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE);
-			}
-			else
-			{
-				tintin_puts2("SYNTAX: #IGNORE [NAME] [ON|OFF]", ses);
-				break;
-			}
-			tintin_printf2(ses, "#%s IGNORE STATUS HAS BEEN SET TO: %s.", list_table[cnt].name, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE) ? "ON" : "OFF");
-			fnd = TRUE;
-		}
-
-		if (fnd == FALSE)
-		{
-			tintin_printf2(ses, "#NO MATCH(es) found for {%s}", left);
-		}
-	}
-	return ses;
-}
-
-DO_COMMAND(do_debug)
-{
-	char left[BUFFER_SIZE], right[BUFFER_SIZE];
-	int cnt, fnd = FALSE;
-
-	arg = get_arg_in_braces(arg, left,  0);
-	arg = get_arg_in_braces(arg, right, 0);
-
-	if (*left == 0)
-	{
-		tintin_header(ses, " DEBUGS ");
-
-		for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
-		{
-			tintin_printf2(ses, "  %-20s %3s", list_table[cnt].name_multi, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG) ? "ON" : "OFF");
-		}
-
-		tintin_header(ses, "");
-	}
-	else
-	{
-		for (cnt = fnd = 0 ; cnt < LIST_MAX ; cnt++)
-		{
-			if (!is_abbrev(left, list_table[cnt].name_multi))
-			{
-				continue;
-			}
-			if (*right == 0)
-			{
-				TOG_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG);
-			}
-			else if (is_abbrev(right, "ON"))
-			{
-				SET_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG);
-			}
-			else if (is_abbrev(right, "OFF"))
-			{
-				DEL_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG);
-			}
-			else
-			{
-				tintin_puts2("SYNTAX: #DEBUG [NAME] [ON|OFF]", ses);
-				break;
-			}
-			tintin_printf2(ses, "#%s DEBUG STATUS HAS BEEN SET TO: %s.", list_table[cnt].name, HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG) ? "ON" : "OFF");
-			fnd = TRUE;
-		}
-
-		if (fnd == FALSE)
-		{
-			tintin_printf2(ses, "#NO MATCH(es) found for {%s}", left);
-		}
-	}
-	return ses;
-}
-
 DO_COMMAND(do_return)
 {
 	char result[BUFFER_SIZE];
@@ -459,6 +289,77 @@ DO_COMMAND(do_return)
 	return ses;
 }
 
+
+DO_COMMAND(do_send)
+{
+	char left[BUFFER_SIZE], out[BUFFER_SIZE];
+
+	printf("%s\n", arg);
+
+	get_arg_in_braces(arg, left, TRUE);
+
+	substitute(ses, left, out, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
+
+	write_line_mud(out, ses);
+
+	return ses;
+}
+
+DO_COMMAND(do_showme)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[BUFFER_SIZE], output[BUFFER_SIZE];
+
+	arg = get_arg_in_braces(arg, temp, TRUE);
+	arg = get_arg_in_braces(arg, right, FALSE);
+
+	substitute(ses, temp, left, SUB_COL|SUB_ESC);
+
+	if (*right)
+	{
+		do_one_prompt(ses, left, atoi(right));
+
+		return ses;
+	}
+	do_one_line(left, ses);
+
+	substitute(ses, left, temp, SUB_COL|SUB_ESC);
+
+	if (HAS_BIT(ses->flags, SES_FLAG_GAG))
+	{
+		DEL_BIT(ses->flags, SES_FLAG_GAG);
+
+		return ses;
+	}
+
+	if (ses != gtd->ses)
+	{
+		return ses;
+	}
+
+	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_VERBOSE) && HAS_BIT(gts->flags, SES_FLAG_QUIET))
+	{
+		return ses;
+	}
+
+	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
+	{
+		save_pos(ses);
+		goto_rowcol(ses, ses->bot_row, 1);
+	}
+
+	sprintf(output, "\033[0m%s\033[0m", temp);
+
+	add_line_buffer(ses, output, FALSE);
+
+	printline(ses, output, FALSE);
+
+	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
+	{
+		restore_pos(ses);
+	}
+	return ses;
+}
+
 DO_COMMAND(do_snoop)
 {
 	struct session *sesptr = ses;
@@ -478,6 +379,8 @@ DO_COMMAND(do_snoop)
 		if (sesptr == NULL)
 		{
 			tintin_puts2("#NO SESSION WITH THAT NAME!", ses);
+
+			return ses;
 		}
 	}
 	else
@@ -498,6 +401,20 @@ DO_COMMAND(do_snoop)
 	return ses;
 }
 
+DO_COMMAND(do_suspend)
+{
+	printf("\033[r\033[%d;%dH", gtd->ses->rows, 1);
+
+	fflush(stdout);
+
+	kill(0, SIGSTOP);
+
+	dirty_screen(gtd->ses);
+
+	tintin_puts("#RETURNING BACK TO TINTIN++.", NULL);
+
+	return ses;
+}
 
 DO_COMMAND(do_system)
 {
@@ -507,14 +424,11 @@ DO_COMMAND(do_system)
 
 	if (!*left)
 	{
-		tintin_puts2("#EXECUTE WHAT COMMAND?", ses);
+		tintin_printf(ses, "#SYNTAX: #SYSTEM {COMMAND}");
 		return ses;
 	}
 
-	if (show_message(ses, -1))
-	{
-		tintin_printf2(ses, "#OK, EXECUTING '%s'", left);
-	}
+	show_message(ses, -1, "#OK: EXECUTING '%s'", left);
 
 	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
 	{
@@ -547,29 +461,3 @@ DO_COMMAND(do_zap)
 	return gtd->ses;
 }
 
-DO_COMMAND(do_info)
-{
-	int cnt;
-
-	tintin_header(ses, " INFORMATION ");
-
-	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
-	{
-		tintin_printf2(ses, "%-20s %3d  IGNORE %3s  MESSAGE %3s  DEBUG  %3s",
-			list_table[cnt].name_multi,
-			ses->list[cnt]->count,
-			HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_IGNORE) ? "ON" : "OFF",
-			HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_MESSAGE) ? "ON" : "OFF",
-			HAS_BIT(ses->list[cnt]->flags, LIST_FLAG_DEBUG)   ? "ON" : "OFF");
-	}
-	tintin_header(ses, "");
-
-	return ses;
-}
-
-DO_COMMAND(do_gagline)
-{
-	SET_BIT(ses->flags, SES_FLAG_GAG);
-
-	return ses;
-}
