@@ -30,12 +30,12 @@
 
 #include <errno.h>
 #include <sys/types.h>
-#include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
 #include <signal.h>
 #include <arpa/inet.h>
-
+#include <sys/socket.h>
+#include <fcntl.h>
 
 /**************************************************/
 /* try connect to the mud specified by the args   */
@@ -45,10 +45,10 @@
 
 int connect_mud(const char *host, const char *port, struct session *ses)
 {
-	int sock;
+	int sock, d;
 	struct sockaddr_in sockaddr;
 
-	if (isdigit(*host))                            /* interprete host part */
+	if (sscanf(host, "%d.%d.%d.%d", &d, &d, &d, &d) == 4)
 	{
 		sockaddr.sin_addr.s_addr = inet_addr(host);
 	}
@@ -90,6 +90,11 @@ int connect_mud(const char *host, const char *port, struct session *ses)
 		return 0;
 	}
 
+	if (fcntl(sock, F_SETFL, O_NDELAY|O_NONBLOCK) == -1)
+	{
+		perror("connect_mud: fcntl O_NDELAY|O_NONBLOCK");
+	}
+
 	return sock;
 }
 
@@ -99,8 +104,6 @@ int connect_mud(const char *host, const char *port, struct session *ses)
 
 void write_line_mud(const char *line, struct session *ses)
 {
-	char outtext[BUFFER_SIZE];
-
 	if (ses == gts)
 	{
 		tintin_printf2(ses, "#NO SESSION ACTIVE. USE: %csession {name} {host port} TO START ONE.", gtd->tintin_char);
@@ -113,9 +116,7 @@ void write_line_mud(const char *line, struct session *ses)
 		return;
 	}
 
-	sprintf(outtext, "%s\r\n", line);
-
-	if (write(ses->socket, outtext, strlen(outtext)) == -1)
+	if (write(ses->socket, line, strlen(line)) == -1)
 	{
 		syserr("write in write_line_mud");
 	}
@@ -134,15 +135,20 @@ void read_buffer_mud(struct session *ses)
 
 	gtd->mud_output_len = 0;
 
-	size = read(ses->socket, buf, BUFFER_SIZE -1);
+	if ((size = read(ses->socket, buf, BUFFER_SIZE - 1)) == -1)
+	{
+		tintin_printf2(ses, "");
+		perror("read_buffer_mud:");
+	}
 
 	if (size <= 0)
 	{
 		gtd->mud_output_len = -1;
-
 		return;
 	}
 
+	buf[size] = 0;
+
 	translate_telopts(ses, buf, size);
 
 	gtd->mud_output_buf[gtd->mud_output_len] = 0;
