@@ -34,91 +34,86 @@
 /* parse input, check for TINTIN commands and aliases and send to session */
 /**************************************************************************/
 
-struct session *parse_input(char *input, struct session *ses)
+struct session *parse_input(const char *input, struct session *ses)
 {
-	char temp[BUFFER_SIZE], command[BUFFER_SIZE], arg[BUFFER_SIZE];
-	char *pti, *pta;
-	struct listnode *ln;
-	int i;
+	char command[BUFFER_SIZE], arg[BUFFER_SIZE];
 
-	push_call("[%s] parse_input(%s,%p)",ses->name,input,ses);
+	if (push_call("[%s] parse_input(%s,%p)",ses->name,input,ses))
+	{
+		pop_call();
+		return ses;
+	}
 
 	DEL_BIT(ses->flags, SES_FLAG_BREAK);
 
 	if (*input == 0)
 	{
-		write_line_mud("", ses);
+		write_line_mud("\r\n", ses);
 
 		pop_call();
 		return(ses);
 	}
 
-	if (HAS_BIT(ses->flags, SES_FLAG_VERBATIM) && ses != gts)
+	if (HAS_BIT(ses->flags, SES_FLAG_VERBATIM) && *input != gtd->tintin_char)
 	{
-		if (*input != gtd->tintin_char)
-		{
-			write_line_mud(input, ses);
-			pop_call();
-			return(ses);
-		}
+		sprintf(command, "%s\r\n", input);
+
+		write_line_mud(command, ses);
+
+		pop_call();
+		return(ses);
 	}
 
 	if (*input == gtd->verbatim_char)
 	{
-		TOG_BIT(ses->flags, SES_FLAG_VERBATIM);
-		write_line_mud(input + 1, ses);
-		TOG_BIT(ses->flags, SES_FLAG_VERBATIM);
+		sprintf(command, "%s\r\n", &input[1]);
+
+		write_line_mud(command, ses);
 
 		pop_call();
-		return(ses);
+		return ses;
 	}
 
-	pti = input;
-
-	while (*pti)
+	while (*input)
 	{
-		if (*pti == ';')
+		if (*input == ';')
 		{
-			if (*++pti == 0)
+			if (*++input == 0)
 			{
 				break;
 			}
 		}
-		pti = (char *)get_arg_stop_spaces(pti, temp);
-		pti = (char *)get_arg_all(pti, arg);
+		input = get_arg_stop_spaces(input, arg);
 
-		substitute(ses, temp, command, SUB_VAR|SUB_FUN);
+		substitute(ses, arg, command, SUB_VAR|SUB_FUN);
+
+		input = get_arg_all(input, arg);
 
 		if (*command == gtd->tintin_char)
 		{
 			ses = parse_tintin_command(command + 1, arg, ses);
 		}
-		else if ((ln = searchnode_list_begin(ses->list[LIST_ALIAS], command, ALPHA)))
-		{
-			strcpy(gtd->vars[0], arg);
-
-			pta = arg;
-
-			for (i = 1 ; i < 10 ; i++)
-			{
-				pta = (char *) get_arg_in_braces(pta, gtd->vars[i], FALSE);
-			}
-			substitute(ses, ln->right, command, SUB_ARG);
-
-			if (!strcmp(ln->right, command) && *arg)
-			{
-				sprintf(command, "%s %s", ln->right, arg);
-			}
-			ses = parse_input(command, ses);
-		}
-		else if (HAS_BIT(ses->flags, SES_FLAG_SPEEDWALK) && !*arg && is_speedwalk_dirs(command))
-		{
-			process_speedwalk(command, ses);
-		}
 		else
 		{
-			get_arg_with_spaces(arg, arg);
-			write_com_arg_mud(command, arg, ses);
+			if (*arg)
+			{
+				cat_sprintf(command, " %s", arg);
+			}
+
+			if (check_all_aliases(command, arg, ses))
+			{
+				DEL_BIT(gts->flags, SES_FLAG_USERCOMMAND);
+
+				ses = parse_input(command, ses);
+			}
+			else if (HAS_BIT(ses->flags, SES_FLAG_SPEEDWALK) && !*arg && is_speedwalk_dirs(command))
+			{
+				process_speedwalk(command, ses);
+			}
+			else
+			{
+				write_mud(ses, command);
+			}
 		}
 		if (HAS_BIT(ses->flags, SES_FLAG_BREAK))
 		{
@@ -191,7 +186,7 @@ void process_speedwalk(const char *cp, struct session *ses)
 
 			for (i = 0 ; i < loopcnt ; i++)
 			{
-				write_com_arg_mud(sc, "", ses);
+				write_mud(ses, sc);
 			}
 			while (*cp != sc[0])
 			{
@@ -202,7 +197,7 @@ void process_speedwalk(const char *cp, struct session *ses)
 		{
 			sc[0] = *cp;
 
-			write_com_arg_mud(sc, "", ses);
+			write_mud(ses, sc);
 		}
 	}
 	pop_call();
@@ -226,7 +221,7 @@ struct session *parse_tintin_command(const char *command, char *arg, struct sess
 		{
 			if (*arg)
 			{
-				get_arg_with_spaces(arg, arg);
+				get_arg_in_braces(arg, arg, TRUE);
 				substitute(ses, arg, newcommand, SUB_VAR|SUB_FUN);
 				parse_input(newcommand, sesptr);  /* was: #sessioname commands */
 				return ses;
@@ -238,51 +233,48 @@ struct session *parse_tintin_command(const char *command, char *arg, struct sess
 		}
 	}
 
-	if (isdigit(*command))
+	if (atoi(command) > 0)
 	{
 		int i = atoi(command);
 
-		if (i > 0)
-		{
-			get_arg_in_braces(arg, arg, 1);
+		get_arg_in_braces(arg, arg, TRUE);
 
-			while (i-- > 0)
-			{
-				ses = parse_input(arg, ses);
-			}
-		}
-		else
+		while (i-- > 0)
 		{
-			tintin_puts("#YEAH RIGHT! GO REPEAT THAT YOURSELF DUDE.", ses);
+			ses = parse_input(arg, ses);
 		}
 		return ses;
 	}
 
-	for (cmd = 0 ; *command_table[cmd].name != 0 ; cmd++)
+	if (isalpha(*command))
 	{
-		if (is_abbrev(command, command_table[cmd].name))
-		{
-			if (HAS_BIT(command_table[cmd].flags, CMD_FLAG_SUB))
-			{
-				substitute(ses, arg, argument, SUB_VAR|SUB_FUN);
+		cmd = gtd->command_ref[tolower(*command) - 'a'];
 
-				return (ses = (*command_table[cmd].command) (ses, argument));
-			}
-			else
+		for ( ; *command_table[cmd].name != 0 ; cmd++)
+		{
+			if (is_abbrev(command, command_table[cmd].name))
 			{
-				return (ses = (*command_table[cmd].command) (ses, arg));
+				if (HAS_BIT(command_table[cmd].flags, CMD_FLAG_SUB))
+				{
+					substitute(ses, arg, argument, SUB_VAR|SUB_FUN);
+
+					return (ses = (*command_table[cmd].command) (ses, argument));
+				}
+				else
+				{
+					return (ses = (*command_table[cmd].command) (ses, arg));
+				}
 			}
 		}
 	}
-	tintin_printf2(ses, "#UNKNOWN TINTIN-COMMAND '%s'.", command);
+	tintin_printf(ses, "#ERROR: #UNKNOWN TINTIN-COMMAND '%s'.", command);
 
 	return ses;
 }
 
 
-
 /*
-	get all arguments - don't remove "s and \s
+	get all arguments - only check for unescaped ';'
 */
 
 const char *get_arg_all(const char *s, char *arg)
@@ -293,71 +285,23 @@ const char *get_arg_all(const char *s, char *arg)
 
 	while (*s)
 	{
-		if (*s == '\\')
-		{
-			*arg++ = *s++;
-			if (*s)
-			{
-				*arg++ = *s++;
-			}
-		}
 #ifdef BIG5
-		else if (*s & 0x80)
+		if (*s & 0x80)
 		{
 			*arg++ = *s++;
 			if (*s)
 			{
 				*arg++ = *s++;
 			}
+			continue;
 		}
 #endif
-		else if (*s == ';' && nest < 1)
-		{
-			break;
-		}
-		else if (*s == DEFAULT_OPEN)
-		{
-			nest++;
-			*arg++ = *s++;
-		}
-		else if (*s == DEFAULT_CLOSE)
-		{
-			nest--;
-			*arg++ = *s++;
-		}
-		else
+
+		if (*s == '\\' && s[1] == ';')
 		{
 			*arg++ = *s++;
 		}
-	}
-	*arg = '\0';
-	return(s);
-}
-
-/**************************************/
-/* get all arguments - remove "s etc. */
-/* Example:                           */
-/* In: "this is it" way way hmmm;     */
-/* Out: this is it way way hmmm       */ 
-/**************************************/
-
-const char *get_arg_with_spaces(const char *s, char *arg)
-{
-	int nest = 0;
-
-	s = space_out(s);
-
-	while (*s)
-	{
-		if (*s == '\\')
-		{
-			*arg++ = *s++;
-			if (*s == 0)
-			{
-				break;
-			}
-		}
-		else if (*s == ';' && !nest)
+		else if (*s == ';' && nest == 0)
 		{
 			break;
 		}
@@ -369,29 +313,19 @@ const char *get_arg_with_spaces(const char *s, char *arg)
 		{
 			nest--;
 		}
-#ifdef BIG5
-		else if (*s & 0x80)
-		{
-			*arg++ = *s++;
-			if (*s == 0)
-			{
-				break;
-			}
-		}
-#endif
 		*arg++ = *s++;
 	}
-	*arg = '\0'; 
-	return(s);
+	*arg = '\0';
+	return s;
 }
 
 /*
-	my own routine
+	Braces are stripped in braced arguments leaving all else as is.
 */
 
 const char *get_arg_in_braces(const char *s, char *arg, int flag)
 {
-	int nest = 0;
+	int nest = 1;
 	const char *ptr;
 
 	s = space_out(s);
@@ -407,12 +341,24 @@ const char *get_arg_in_braces(const char *s, char *arg, int flag)
 		{
 			s = get_arg_with_spaces(ptr, arg);
 		}
-		return(s);
+		return s;
 	}
 	s++;
 
-	while (*s && !(*s == DEFAULT_CLOSE && !nest))
+	while (*s)
 	{
+#ifdef BIG5
+		if (*s & 0x80)
+		{
+			*arg++ = *s++;
+			if (*s == 0)
+			{
+				break;
+			}
+			continue;
+		}
+#endif
+
 		if (*s == DEFAULT_OPEN)
 		{
 			nest++;
@@ -420,23 +366,18 @@ const char *get_arg_in_braces(const char *s, char *arg, int flag)
 		else if (*s == DEFAULT_CLOSE)
 		{
 			nest--;
-		}
-#ifdef BIG5
-		else if (*s & 0x80)
-		{
-			*arg++ = *s++;
-			if (*s == 0)
+
+			if (nest == 0)
 			{
 				break;
 			}
 		}
-#endif
 		*arg++ = *s++;
 	}
 
-	if (!*s)
+	if (*s == 0)
 	{
-		tintin_puts2("#Unmatched braces error!", NULL);
+		tintin_printf2(NULL, "#Unmatched braces error!");
 	}
 	else
 	{
@@ -444,59 +385,112 @@ const char *get_arg_in_braces(const char *s, char *arg, int flag)
 	}
 	*arg = '\0';
 
-	return(s);
+	return s;
 }
 
-/*
-	get one arg, stop at spaces, remove quotes
-*/
+/**************************************/
+/* get all arguments                  */
+/**************************************/
 
-const char *get_arg_stop_spaces(const char *s, char *arg)
+const char *get_arg_with_spaces(const char *s, char *arg)
 {
-	int inside = FALSE;
-
-	s = space_out(s);
+	int nest = 0;
 
 	while (*s)
 	{
-		if (*s == '\\')
-		{
-			*arg++ = *s++;
-			if (*s)
-			{
-				*arg++ = *s++;
-			}
-		}
 #ifdef BIG5
-		else if(*s & 0x80)
+		if (*s & 0x80)
 		{
 			*arg++ = *s++;
-			if (*s)
-			{
-				*arg++ = *s++;
-			}
-		}
-#endif
-		else if (*s == '"')
-		{
-			s++;
-			inside = !inside;
-		}
-		else if (*s == ';')
-		{
-			if (inside)
-			{
-				*arg++ = *s++;
-			}
-			else
+			if (*s == 0)
 			{
 				break;
 			}
 		}
-		else if (!inside && *s == ' ')
+		continue;
+#endif
+
+		if (*s == '\\')
+		{
+			*arg++ = *s++;
+
+			if (*s)
+			{
+				*arg++ = *s++;
+			}
+		}
+		else if (*s == ';' && nest == 0)
 		{
 			break;
 		}
+		else if (*s == DEFAULT_OPEN)
+		{
+			*arg++ = *s++;
+			nest++;
+		}
+		else if (*s == DEFAULT_CLOSE)
+		{
+			*arg++ = *s++;
+			nest--;
+		}
+		else
+		{
+			*arg++ = *s++;
+		}
+	}
+	*arg = '\0'; 
+	return s;
+}
+
+/*
+	get one arg, stop at spaces
+*/
+
+const char *get_arg_stop_spaces(const char *s, char *arg)
+{
+	int nest = 0;
+
+	while (*s)
+	{
+#ifdef BIG5
+		if(*s & 0x80)
+		{
+			*arg++ = *s++;
+			if (*s)
+			{
+				*arg++ = *s++;
+			}
+			continue;
+		}
+#endif
+
+		if (*s == '\\')
+		{
+			*arg++ = *s++;
+
+			if (*s)
+			{
+				*arg++ = *s++;
+			}
+		}
+		else if (*s == ';' && nest == 0)
+		{
+			break;
+		}
+		else if (*s == ' ' && nest == 0)
+		{
+			break;
+		}
+		else if (*s == DEFAULT_OPEN)
+		{
+			*arg++ = *s++;
+			nest++;
+		}
+		else if (*s == DEFAULT_CLOSE)
+		{
+			*arg++ = *s++;
+			nest--;
+		}
 		else
 		{
 			*arg++ = *s++;
@@ -504,7 +498,7 @@ const char *get_arg_stop_spaces(const char *s, char *arg)
 	}
 	*arg = '\0';
 
-	return(s);
+	return s;
 }
 
 /*
@@ -524,11 +518,11 @@ const char *space_out(const char *s)
 	send command+argument to the mud
 */
 
-void write_com_arg_mud(const char *command, const char *argument, struct session *ses)
+void write_mud(struct session *ses, const char *command)
 {
-	char temp[BUFFER_SIZE], outtext[BUFFER_SIZE];
+	char output[BUFFER_SIZE];
 
-	if (ses->in_room && argument[0] == 0)
+	if (ses->in_room)
 	{
 		follow_map(ses, command);
 	}
@@ -538,17 +532,9 @@ void write_com_arg_mud(const char *command, const char *argument, struct session
 		check_insert_path(command, ses);
 	}
 
-	if (*argument == 0)
-	{
-		strcpy(temp, command);
-	}
-	else
-	{
-		sprintf(temp, "%s %s", command, argument);
-	}
-	substitute(ses, temp, outtext, SUB_VAR|SUB_FUN|SUB_ESC);
+	substitute(ses, command, output, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
 
-	write_line_mud(outtext, ses);
+	write_line_mud(output, ses);
 }
 
 
@@ -587,6 +573,7 @@ void do_one_line(char *line, struct session *ses)
 	{
 		check_all_highlights(line, strip, ses);
 	}
+
 	pop_call();
 	return;
 }
