@@ -67,14 +67,16 @@ DO_COMMAND(do_map)
 
 	root = ses->list[LIST_PATH];
 
-	sprintf(buf, "%-10s", "#PATH:");
+	tintin_printf(ses, "#NOTE: This command will be used for the automapper in a future update. You can use #PATH instead.");
+
+	sprintf(buf, "%-8s", "#PATH:");
 
 	for (node = root->f_node ; node ; node = node->next)
 	{
 		if (strlen(buf) + strlen(node->left) > ses->cols)
 		{
 			tintin_puts2(buf, ses);
-			sprintf(buf, "%-8s", "#PATH:");
+			sprintf(buf, "%-8s", "");
 		}
 		strcat(buf, node->left);
 		strcat(buf, " ");
@@ -174,10 +176,7 @@ DO_COMMAND(do_loadpath)
 
 	if ((node = searchnode_list(ses->list[LIST_ALIAS], left)) == NULL)
 	{
-		if (show_message(ses, LIST_PATH))
-		{
-			tintin_printf2(ses, "#ALIAS {%s} NOT FOUND.", left);
-		}
+		show_message(ses, LIST_PATH, "#ALIAS {%s} NOT FOUND.", left);
 	}
 	else
 	{
@@ -192,26 +191,32 @@ DO_COMMAND(do_loadpath)
 				arg++;
 			}
 
-			arg = get_arg_with_spaces(arg, left);
+			arg = get_arg_in_braces(arg, left, TRUE);
 
-			addnode_list(root, left, "", "0");
+			if ((node = searchnode_list(ses->list[LIST_PATHDIR], left)))
+			{
+				addnode_list(ses->list[LIST_PATH], node->left, node->right, "0");
+			}
+			else
+			{
+				addnode_list(root, left, left, "0");
+			}
 
 			if (HAS_BIT(root->flags, LIST_FLAG_DEBUG))
 			{
-				tintin_printf2(ses, "[LOADPATH: %3d %s]", root->count, left);
+				tintin_printf2(ses, "#DEBUG #LOADPATH: %3d %s", root->count, left);
 			}
 		}
 
-		if (show_message(ses, LIST_PATH))
-		{
-			tintin_printf2(ses, "#PATH WITH %d NODES LOADED.", root->count);
-		}
+		show_message(ses, LIST_PATH, "#OK. PATH WITH %d NODES LOADED.", root->count);
 	}
 	return ses;
 }
 
 DO_COMMAND(do_path)
 {
+	struct listroot *root;
+	struct listnode *node;
 	char left[BUFFER_SIZE], right[BUFFER_SIZE];
 
 	arg = get_arg_in_braces(arg, left, FALSE);
@@ -219,24 +224,35 @@ DO_COMMAND(do_path)
 
 	if (*left == 0)
 	{
-		tintin_printf2(ses, "#SYNTAX: #PATH <command> <reversed command>");
+		root = ses->list[LIST_PATH];
+
+		sprintf(left, "%-8s", "#PATH:");
+
+		for (node = root->f_node ; node ; node = node->next)
+		{
+			if (strlen(left) + strlen(node->left) > ses->cols)
+			{
+				tintin_puts2(left, ses);
+				sprintf(left, "%-8s", "");
+			}
+			strcat(left, node->left);
+			strcat(left, " ");
+		}
+
+		if (strlen(left) > 8)
+		{
+			tintin_puts2(left, ses);
+		}
 	}
-	else if (*right == 0)
+	else if (*right == 0 && searchnode_list(ses->list[LIST_PATHDIR], left))
 	{
-		if (searchnode_list(ses->list[LIST_PATHDIR], left))
-		{
-			check_insert_path(left, ses);
-		}
-		else
-		{
-			tintin_printf2(ses, "#PATH: {%s} IS NOT A PATHDIR.", left);
-		}
+		check_insert_path(left, ses);
 	}
 	else
 	{
 		addnode_list(ses->list[LIST_PATH], left, right, "0");
 
-		tintin_printf2(ses, "#OK {%s} = {%s} ADDED TO PATH.", left, right);
+		show_message(ses, LIST_PATH, "#OK. #PATH - {%s} = {%s} ADDED.", left, right);
 	}
 	return ses;
 }
@@ -252,12 +268,12 @@ DO_COMMAND(do_unpath)
 
 	if (root->l_node)
 	{
-		tintin_printf2(ses, "#OK.  DELETED MOVE {%s}", root->l_node->left);
+		show_message(ses, LIST_PATH, "#OK. #UNPATH - DELETED MOVE {%s}", root->l_node->left);
 		deletenode_list(ses, root->l_node, LIST_PATH);
 	}
 	else
 	{
-		tintin_puts("#NO MOVES LEFT.", ses);
+		tintin_puts("#ERROR: #UNPATH - NO MOVES LEFT.", ses);
 	}
 	return ses;
 }
@@ -299,7 +315,7 @@ DO_COMMAND(do_walk)
 				break;
 
 			default:
-				tintin_printf2(ses, "#SYNTAX: #WALK {FORWARD|BACKWARD}");
+				tintin_printf(ses, "#SYNTAX: #WALK {FORWARD|BACKWARD}");
 				break;
 		}
 		ses->flags = flags;
@@ -326,20 +342,14 @@ DO_COMMAND(do_pathdir)
 	{
 		if (show_node_with_wild(ses, left, LIST_PATHDIR) == FALSE)
 		{
-			if (show_message(ses, LIST_PATHDIR))
-			{
-				tintin_printf2(ses, "#NO MATCH(ES) FOUND FOR {%s}", left);
-			}
+			show_message(ses, LIST_PATHDIR, "#NO MATCH(ES) FOUND FOR {%s}", left);
 		}
 	}
 	else
 	{
 		updatenode_list(ses, left, right, "0", LIST_PATHDIR);
 
-		if (show_message(ses, LIST_PATHDIR))
-		{
-			tintin_printf2(ses, "#OK. DIRECTION {%s} WILL BE REVERSED AS {%s}", left, right);
-		}
+		show_message(ses, LIST_PATHDIR, "#OK: DIRECTION {%s} WILL BE REVERSED AS {%s}", left, right);
 	}
 	return ses;
 }
