@@ -28,7 +28,6 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <errno.h>
-#include <stdarg.h>
 
 /*
 	some readline stuff
@@ -50,6 +49,7 @@ void initrl(void)
 	stifle_history(gtd->history_size);
 
 	rl_variable_bind("horizontal-scroll-mode", "on");
+	rl_variable_bind("enable-keypad", "on");
 #ifdef USE_ISO_LATIN_ONE
 	rl_variable_bind("meta-flag", "on");
 	rl_variable_bind("convert-meta", "off");
@@ -58,12 +58,13 @@ void initrl(void)
 
 	gtd->keymap = rl_get_keymap();
 
-	rl_macro_bind("\e[5~", "[[#buffer u]]", gtd->keymap);
-	rl_macro_bind("\e[6~", "[[#buffer d]]", gtd->keymap);
-	rl_macro_bind("\e[7~", "[[#buffer h]]", gtd->keymap);
-	rl_macro_bind("\e[H",  "[[#buffer h]]", gtd->keymap);
-	rl_macro_bind("\e[8~", "[[#buffer e]]", gtd->keymap);
-	rl_macro_bind("\e[F",  "[[#buffer e]]", gtd->keymap);
+	rl_macro_bind("\033[5~", "[[#buffer u]]", gtd->keymap);
+	rl_macro_bind("\033[6~", "[[#buffer d]]", gtd->keymap);
+	rl_macro_bind("\033[7~", "[[#buffer h]]", gtd->keymap);
+	rl_macro_bind("\033[H",  "[[#buffer h]]", gtd->keymap);
+	rl_macro_bind("\033[8~", "[[#buffer e]]", gtd->keymap);
+	rl_macro_bind("\033[F",  "[[#buffer e]]", gtd->keymap);
+	rl_macro_bind("\\C-t",   "[[#buffer l]]", gtd->keymap);
 }
 
 
@@ -76,52 +77,72 @@ void printline(struct session *ses, const char *str, int prompt)
 		return;
 	}
 
-	if (HAS_BIT(ses->flags, SES_FLAG_SCAN))
+	if (HAS_BIT(ses->flags, SES_FLAG_SCAN) && !HAS_BIT(ses->flags, SES_FLAG_VERBOSE))
 	{
 		return;
 	}
 
 	word_wrap(ses, str, wrapped_str, TRUE);
 
+/*	strcpy(wrapped_str, str); */
+
 	if (prompt)
 	{
 		printf("%s", wrapped_str);
 	}
 	else
 	{
-		printf("%s\n\r", wrapped_str);
+		printf("%s\r\n", wrapped_str);
 	}
+
 }
 
 void bait(void)
 {
-	fd_set readfds, excfds;
+	fd_set readfds, writefds, excfds;
 	struct session *ses;
-	struct timeval to;
+	struct chat_data *buddy;
+	static struct timeval to;
 	int rv;
 
 	to.tv_sec  = 0;
+	to.tv_usec = 0;
 
 	{
 		FD_ZERO(&readfds);
+		FD_ZERO(&writefds);
+		FD_ZERO(&excfds);
 
 		if (!FD_ISSET(0, &readfds))
 		{
-			FD_ZERO(&readfds);
-			FD_ZERO(&excfds);
-			FD_SET(0, &readfds);	/* stdin */
+			FD_SET(0, &readfds);  /* stdin */
+
+			if (gtd->chat)
+			{
+				FD_SET(gtd->chat->fd, &readfds);
+			}
 
 			for (ses = gts->next ; ses ; ses = ses->next)
 			{
 				if (HAS_BIT(ses->flags, SES_FLAG_CONNECTED))
 				{
 					FD_SET(ses->socket, &readfds);
+					FD_SET(ses->socket, &writefds);
+					FD_SET(ses->socket, &excfds);
 				}
 			}
 
-			to.tv_usec = 0;
+			if (gtd->chat)
+			{
+				for (buddy = gtd->chat->next ; buddy ; buddy = buddy->next)
+				{
+					FD_SET(buddy->fd, &readfds);
+					FD_SET(buddy->fd, &writefds);
+					FD_SET(buddy->fd, &excfds);
+				}
+			}
 
-			rv = select(FD_SETSIZE, &readfds, NULL, &excfds, &to);
+			rv = select(FD_SETSIZE, &readfds, &writefds, &excfds, &to);
 
 			if (rv <= 0)
 			{
@@ -130,7 +151,24 @@ void bait(void)
 					return;
 				}
 				syserr("select");
-				
+			}
+
+			if (gtd->chat)
+			{
+				process_chat_connections(&readfds, &writefds, &excfds);
+			}
+
+			for (ses = gts->next ; ses ; ses = gtd->update)
+			{
+				gtd->update = ses->next;
+
+				if (FD_ISSET(ses->socket, &excfds))
+				{
+					FD_CLR(ses->socket, &readfds);
+					FD_CLR(ses->socket, &writefds);
+
+					cleanup_session(ses);
+				}
 			}
 
 			for (ses = gts->next ; ses ; ses = gtd->update)
@@ -147,7 +185,7 @@ void bait(void)
 			}
 		}
 	}
-	readline_echoing_p = HAS_BIT(gtd->ses->flags, SES_FLAG_LOCALECHO);
+	readline_echoing_p = HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO);
 }
 
 
@@ -163,65 +201,6 @@ void mainloop(void)
 }
 
 
-void commandloop(void)
-{
-	char *line, buffer[BUFFER_SIZE];
-
-	while (TRUE)
-	{
-		if (HAS_BIT(gtd->ses->flags, SES_FLAG_CONVERTMETA))
-		{
-			line = readkeyboard();
-		}
-		else
-		{
-			line = readline(NULL);
-		}
-
-		if (line == NULL)
-		{
-			continue;
-		}
-
-		if (HAS_BIT(gtd->ses->flags, SES_FLAG_LOCALECHO))
-		{
-			if ((line = rlhist_expand(line)) == NULL)
-			{
-				continue;
-			}
-		}
-
-
-		sprintf(buffer, "%s", line);
-
-		if (!HAS_BIT(gtd->ses->flags, SES_FLAG_LOCALECHO))
-		{
-			memset(buffer, '*', strlen(line));
-		}
-
-		echo_command(gtd->ses, buffer, 0);
-
-		SET_BIT(gts->flags, SES_FLAG_USERCOMMAND);
-
-		if (gtd->ses->scroll_line != -1)
-		{
-			buffer_e();
-		}
-
-		parse_input(line, gtd->ses);
-
-		DEL_BIT(gts->flags, SES_FLAG_USERCOMMAND);
-
-		if (IS_SPLIT(gtd->ses))
-		{
-			erase_toeol();
-		}
-		free(line);
-
-		fflush(stdout);
-	}
-}
-
 /*
 	Read data, handle telopts, handle triggers, display data
 */
@@ -252,7 +231,6 @@ void readmud(struct session *ses)
 	}
 	gtd->mud_output_len = 0;
 
-
 	/* separate into lines and print away */
 
 	if (HAS_BIT(gtd->ses->flags, SES_FLAG_SPLIT))
@@ -277,16 +255,21 @@ void readmud(struct session *ses)
 			break;
 		}
 
+
 		if (next_line == NULL && strlen(ses->more_output) < BUFFER_SIZE / 2)
 		{
-			if (gts->check_output)
+			if (!HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_PROMPT))
 			{
-				strcat(ses->more_output, line);
-				ses->check_output = utime() + gts->check_output;
-				break;
+				if (gts->check_output)
+				{
+					strcat(ses->more_output, line);
+					ses->check_output = utime() + gts->check_output;
+					break;
+				}
 			}
 		}
 
+
 		if (*line && ses->more_output[0])
 		{
 			if (ses->check_output || HAS_BIT(ses->flags, SES_FLAG_SPLIT))
@@ -307,6 +290,7 @@ void readmud(struct session *ses)
 
 		process_mud_output(ses, linebuf, next_line == NULL);
 	}
+	DEL_BIT(gtd->ses->telopts, TELOPT_FLAG_PROMPT);
 
 	DEL_BIT(gtd->ses->flags, SES_FLAG_READMUD);
 
@@ -340,18 +324,14 @@ void process_mud_output(struct session *ses, char *linebuf, int prompt)
 
 		ses->more_output[0] = 0;
 
-		DEL_BIT(ses->flags, SES_FLAG_GAGPROMPT);
 		DEL_BIT(ses->flags, SES_FLAG_GAG);
 
 		return;
 	}
+
 	add_line_buffer(ses, linebuf, prompt);
 
-	if (HAS_BIT(ses->flags, SES_FLAG_GAGPROMPT))
-	{
-		DEL_BIT(ses->flags, SES_FLAG_GAGPROMPT);
-	}
-	else if (ses == gtd->ses)
+	if (ses == gtd->ses)
 	{
 		printline(ses, linebuf, prompt);
 	}
@@ -363,138 +343,60 @@ void process_mud_output(struct session *ses, char *linebuf, int prompt)
 	}
 }
 
-
-char * readkeyboard(void)
-{
-	char ch, buf[10], *line;
-
-	while (TRUE)
-	{
-		ch = rl_read_key();
-
-		switch (ch)
-		{
-			case '\e':
-				rl_insert_text("\\e");
-				break;			 
-
-			case '\n':
-			case '\r':
-				printf("\n");
-				line = strdup(rl_line_buffer);
-				rl_point = 0;
-				rl_delete_text(0, rl_end);
-				rl_redisplay();
-				return line;
-
-			case 127:
-				if (rl_point)
-				{
-					rl_point--;
-					rl_delete_text(rl_point, rl_point+1);
-				}
-				break;
-
-			default:
-				if (ch < 27)
-				{
-					sprintf(buf, "\\C-%c", 'a' + ch - 1);
-				}								 
-				else
-				{
-					sprintf(buf, "%c", ch);
-				}
-				rl_insert_text(&buf[0]);
-				break;
-		}
-		rl_redisplay();
-	}
-	return NULL;
-}
-
-void echo_command(struct session *ses, char *line, int newline)
-{
-	char buffer[BUFFER_SIZE];
-
-	sprintf(buffer, "\033[0m%s", line);
-
-	if (newline)
-	{
-		if (gtd->ses->more_output[0] == 0)
-		{
-			return;
-		}
-		if (HAS_BIT(gtd->ses->flags, SES_FLAG_SPLIT) && gtd->ses->list[LIST_PROMPT]->f_node)
-		{
-			return;
-		}
-		gtd->ses->check_output = 0;
-
-		SET_BIT(gtd->ses->flags, SES_FLAG_SCROLLSTOP);
-
-		if (HAS_BIT(gtd->ses->flags, SES_FLAG_SPLIT))
-		{
-			tintin_printf2(gtd->ses, "%s%s", gtd->ses->more_output, buffer);
-		}
-		else
-		{
-			tintin_printf2(gtd->ses, "%s", buffer);
-		}
-
-		DEL_BIT(gtd->ses->flags, SES_FLAG_SCROLLSTOP);
-
-		add_line_buffer(gtd->ses, buffer, FALSE);
-
-		return;
-	}
-
-	if (HAS_BIT(gtd->ses->flags, SES_FLAG_SPLIT))
-	{
-		if (HAS_BIT(gtd->ses->flags, SES_FLAG_ECHOCOMMAND) || gtd->ses->list[LIST_PROMPT]->f_node == NULL)
-		{
-			SET_BIT(gtd->ses->flags, SES_FLAG_SCROLLSTOP);
-
-			if (gtd->ses->more_output[0] && gtd->ses->list[LIST_PROMPT]->f_node == NULL)
-			{
-				gtd->ses->check_output = 0;
-
-				tintin_printf2(gtd->ses, "%s%s", gtd->ses->more_output, buffer);
-			}
-			else
-			{
-				tintin_printf2(gtd->ses, "%s", buffer);
-			}
-			DEL_BIT(gtd->ses->flags, SES_FLAG_SCROLLSTOP);
-		}
-	}
-	add_line_buffer(gtd->ses, buffer, -1);
-}
-
-int show_message(struct session *ses, int index)
+void show_message(struct session *ses, int index, const char *format, ...)
 {
 	struct listroot *root;
+	char buf[BUFFER_SIZE];
+	va_list args;
 
+	push_call("show_message(%p,%p,%p)",ses,index,format);
+
+	va_start(args, format);
+
+	vsprintf(buf, format, args);
+
+	va_end(args);
+	
 	if (index == -1)
 	{
-		return HAS_BIT(gts->flags, SES_FLAG_USERCOMMAND);
+		if (HAS_BIT(gts->flags, SES_FLAG_USERCOMMAND))
+		{
+			tintin_puts2(buf, ses);
+		}
+		pop_call();
+		return;
 	}
 
 	root = ses->list[index];
 
 	if (HAS_BIT(root->flags, LIST_FLAG_DEBUG))
 	{
-		return TRUE;
+		tintin_puts2(buf, ses);
+
+		pop_call();
+		return;
 	}
 
 	if (HAS_BIT(root->flags, LIST_FLAG_MESSAGE))
 	{
 		if (HAS_BIT(gts->flags, SES_FLAG_USERCOMMAND))
 		{
-			return TRUE;
+			tintin_puts2(buf, ses);
+
+			pop_call();
+			return;
 		}
 	}
 
-	return FALSE;
+	if (HAS_BIT(root->flags, LIST_FLAG_LOG))
+	{
+		if (ses->logfile)
+		{
+			logit(ses, buf, ses->logfile);
+		}
+	}
+	pop_call();
+	return;
 }
 
 void tintin_header(struct session *ses, const char *format, ...)
@@ -530,7 +432,10 @@ void socket_printf(struct session *ses, size_t length, const char *format, ...)
 	vsprintf(buf, format, args);
 	va_end(args);
 
-	write(ses->socket, buf, length);
+	if (HAS_BIT(ses->flags, SES_FLAG_CONNECTED))
+	{
+		write(ses->socket, buf, length);
+	}
 }
 
 void tintin_printf2(struct session *ses, const char *format, ...)
@@ -571,12 +476,16 @@ void tintin_puts2(const char *cptr, struct session *ses)
 		ses = gtd->ses;
 	}
 
-	if (ses != gtd->ses)
+	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_VERBOSE) && HAS_BIT(gts->flags, SES_FLAG_QUIET))
 	{
 		return;
 	}
 
-	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_VERBOSE) && HAS_BIT(gts->flags, SES_FLAG_QUIET))
+	sprintf(output, "\033[0m\033[0K%s\033[0m", cptr);
+
+	add_line_buffer(ses, output, FALSE);
+
+	if (ses != gtd->ses)
 	{
 		return;
 	}
@@ -587,10 +496,6 @@ void tintin_puts2(const char *cptr, struct session *ses)
 		goto_rowcol(ses, ses->bot_row, 1);
 	}
 
-	sprintf(output, "\033[0m\033[0K%s\033[0m", cptr);
-
-	add_line_buffer(ses, output, FALSE);
-
 	printline(ses, output, FALSE);
 
 	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
@@ -606,30 +511,24 @@ void tintin_puts2(const char *cptr, struct session *ses)
 
 void tintin_puts(const char *cptr, struct session *ses)
 {
-	/*
-		bug! doesn't do_one_line() sometimes send output to stdout?
-	*/
+	char buf[BUFFER_SIZE];
 
-	if (ses)
+	if (ses == NULL)
 	{
-		char buf[BUFFER_SIZE];
+		ses = gtd->ses;
+	}
 
-		sprintf(buf, "%s", cptr);
-		do_one_line(buf, ses);
+	sprintf(buf, "%s", cptr);
 
-		if (!HAS_BIT(ses->flags, SES_FLAG_GAG) && !HAS_BIT(ses->flags, SES_FLAG_GAGPROMPT))
-		{
-			tintin_puts2(buf, ses);
-		}
-		else
-		{
-			DEL_BIT(ses->flags, SES_FLAG_GAGPROMPT);
-			DEL_BIT(ses->flags, SES_FLAG_GAG);
-		}
+	do_one_line(buf, ses);
+
+	if (!HAS_BIT(ses->flags, SES_FLAG_GAG))
+	{
+		tintin_puts2(buf, ses);
 	}
 	else
 	{
-		tintin_puts2(cptr, ses);
+		DEL_BIT(ses->flags, SES_FLAG_GAG);
 	}
 }
 
@@ -643,21 +542,26 @@ void quitmsg(const char *m)
 	struct session *ses;
 	char filestring[256];
 
+	if (HAS_BIT(gts->flags, SES_FLAG_PREPPED))
+	{
+		rl_initialize();
+		rl_deprep_terminal();
+	}
+
 	while ((ses = gts->next) != NULL) 
 	{
 		cleanup_session(ses);
 	}
 
+	if (gtd->chat)
+	{
+		close(gtd->chat->fd);
+	}
+
 	if (history_max_entries != 0)
 	{
-		if (getenv("TINTIN_HISTORY") == NULL)
-		{
-			sprintf(filestring, "%s/%s", getenv("HOME"), HISTORY_FILE);
-		}
-		else
-		{
-			sprintf(filestring, "%s/%s", getenv("TINTIN_HISTORY"), HISTORY_FILE);
-		}
+		sprintf(filestring, "%s/%s", getenv("HOME"), HISTORY_FILE);
+
 		write_history(filestring);
 	}
 
@@ -675,9 +579,20 @@ void quitmsg(const char *m)
 	exit(0);
 }
 
-/* quit tintin++ fast!  for use with signal() */
+/* quit tintin++ fast, for use with signal() */
 
 RETSIGTYPE myquitsig(int no_care)
 {
-	quitmsg(NULL);
+	if (gtd->ses->connect_retry > utime())
+	{
+		gtd->ses->connect_retry = 0;
+	}
+	else if (HAS_BIT(gts->flags, SES_FLAG_PREPPED))
+	{
+		socket_printf(gtd->ses, 1, "%c", 3);
+	}
+	else
+	{
+		quitmsg(NULL);
+	}
 }
