@@ -136,6 +136,8 @@ void show_session(struct session *ses, struct session *ptr)
 
 struct session *newactive_session(void)
 {
+	push_call("newactive_session(void)");
+
 	if (gts->next)
 	{
 		activate_session(gts->next);
@@ -144,9 +146,10 @@ struct session *newactive_session(void)
 	{
 		gtd->ses = gts;
 		dirty_screen(gtd->ses);
-		tintin_puts("#THERE'S NO ACTIVE SESSION NOW.", NULL);
-	}
 
+		tintin_printf(NULL, "#THERE'S NO ACTIVE SESSION NOW.");
+	}
+	pop_call();
 	return gtd->ses;
 }
 
@@ -171,25 +174,29 @@ struct session *new_session(const char *name, const char *address, struct sessio
 	char host[BUFFER_SIZE], port[BUFFER_SIZE];
 	struct session *newsession;
 
-	address = get_arg_in_braces(address, host, FALSE);
-	address = get_arg_in_braces(address, port, FALSE);
-
-	if (*host == 0)
+	if (name)
 	{
-		tintin_puts("#HEY! SPECIFY AN ADDRESS WILL YOU?", ses);
-		return(ses);
-	}
+		address = get_arg_in_braces(address, host, FALSE);
+		address = get_arg_in_braces(address, port, FALSE);
 
-	if (*port == 0)
-	{
-		tintin_puts("#HEY! SPECIFY A PORT NUMBER WILL YOU?", ses);
-		return(ses);
+		if (*host == 0)
+		{
+			tintin_puts("#HEY! SPECIFY AN ADDRESS WILL YOU?", ses);
+			return(ses);
+		}
+
+		if (*port == 0)
+		{
+			tintin_puts("#HEY! SPECIFY A PORT NUMBER WILL YOU?", ses);
+			return(ses);
+		}
 	}
 
 	newsession                = calloc(1, sizeof(struct session));
 
-	newsession->name          = strdup(name);
+	newsession->name          = name ? strdup(name) : strdup("run");
 	newsession->flags         = gts->flags;
+	newsession->telopts       = gts->telopts;
 	newsession->host          = strdup(host);
 	newsession->port          = strdup(port);
 	newsession->map_size      = gts->map_size;
@@ -197,11 +204,11 @@ struct session *new_session(const char *name, const char *address, struct sessio
 
 	gtd->ses                  = newsession;
 
-	LINK(newsession, gts->next, gts->prev, next, prev);
+	LINK(newsession, gts->next, gts->prev);
 
 	for (cnt = 0 ; cnt < LIST_ALL ; cnt++)
 	{
-		copy_list(newsession, gts->list[cnt], cnt);
+		newsession->list[cnt] = copy_list(newsession, gts->list[cnt], cnt);
 	}
 
 	init_screen_size(newsession);
@@ -214,8 +221,18 @@ struct session *new_session(const char *name, const char *address, struct sessio
 
 	dirty_screen(newsession);
 
-	connect_session(newsession);
+	if (name)
+	{
+		connect_session(newsession);
+	}
+	else
+	{
+		newsession->socket = atoi(address);
 
+		SET_BIT(newsession->flags, SES_FLAG_CONNECTED);
+
+		gtd->ses = ses;
+	}
 	return gtd->ses;
 }
 
@@ -231,7 +248,6 @@ void connect_session(struct session *ses)
 
 	reconnect:
 
-
 	sock = connect_mud(ses->host, ses->port, ses);
 
 	if (sock == -1)
@@ -246,6 +262,8 @@ void connect_session(struct session *ses)
 		gtd->ses    = ses;
 		ses->socket = sock;
 
+		ses->connect_retry = 0;
+
 		SET_BIT(ses->flags, SES_FLAG_CONNECTED);
 
 		tintin_printf2(ses, "#SESSION '%s' CONNECTED TO '%s' PORT '%s'\n\r", ses->name, ses->host, ses->port);
@@ -297,7 +315,7 @@ void cleanup_session(struct session *ses)
 		gtd->update = ses->next;
 	}
 
-	UNLINK(ses, gts->next, gts->prev, next, prev);
+	UNLINK(ses, gts->next, gts->prev);
 
 	if (ses->socket)
 	{
@@ -307,7 +325,9 @@ void cleanup_session(struct session *ses)
 		}
 	}
 
-	tintin_printf(ses, "\n\r#SESSION '%s' DIED.", ses->name);
+	tintin_printf(ses, "");
+
+	tintin_printf(ses, "#SESSION '%s' DIED.", ses->name);
 
 	if (ses == gtd->ses)
 	{
@@ -326,12 +346,14 @@ void cleanup_session(struct session *ses)
 		fclose(ses->logline);
 	}
 
+	pop_call();
+	push_call("cleanup_session(%p) (free)", ses);
+
+	init_buffer(ses, 0);
 
 	free(ses->name);
 	free(ses->host);
 	free(ses->port);
-	init_buffer(ses, 2);
-	free(ses->buffer);
 
 	free(ses);
 
