@@ -69,7 +69,7 @@ char *str_hash(char *str, int lines)
 	hash_ptr->hash  = hash;
 	strcpy((char *) hash_ptr + gtd->str_hash_size, str);
 
-	LINK(hash_ptr, str_hash_index[hash].f_node, str_hash_index[hash].l_node, next, prev);
+	LINK(hash_ptr, str_hash_index[hash].f_node, str_hash_index[hash].l_node);
 
 	return (char *) hash_ptr + gtd->str_hash_size;
 }
@@ -81,9 +81,9 @@ char *str_unhash(char *str)
 
 	hash_ptr = (struct str_hash_data *) (str - gtd->str_hash_size);
 
-	if (--hash_ptr->count == 0)
+	if (--(hash_ptr->count) == 0)
 	{
-		UNLINK(hash_ptr, str_hash_index[hash_ptr->hash].f_node, str_hash_index[hash_ptr->hash].l_node, next, prev);
+		UNLINK(hash_ptr, str_hash_index[hash_ptr->hash].f_node, str_hash_index[hash_ptr->hash].l_node);
 		free(hash_ptr);
 	}
 	return NULL;
@@ -150,11 +150,11 @@ DO_COMMAND(do_hash)
 			max_hash = cnt_hash;
 		}
 	}
-	tintin_printf2(ses, "Strings Allocated: %8d", cnt_str);
-	tintin_printf2(ses, "Total Links:       %8d", cnt_count);
-	tintin_printf2(ses, "Max Hash:          %8d", max_hash);
-	tintin_printf2(ses, "Total Memory:      %8d", cnt_size);
-	tintin_printf2(ses, "Saved Memory:      %8d", cnt_unhashed_size - cnt_size);
+	tintin_printf2(NULL, "Strings Allocated: %8d", cnt_str);
+	tintin_printf2(NULL, "Total Links:       %8d", cnt_count);
+	tintin_printf2(NULL, "Max Hash:          %8d", max_hash);
+	tintin_printf2(NULL, "Total Memory:      %8d", cnt_size);
+	tintin_printf2(NULL, "Saved Memory:      %8d", cnt_unhashed_size - cnt_size);
 
 	return ses;
 }
