@@ -29,15 +29,14 @@
 #include "tintin.h"
 
 
-const struct command_type command_table[MAX_COMMAND] =
+const struct command_type command_table[] =
 {
 	{	"action",            do_action,            CMD_FLAG_NONE    },
 	{	"alias",             do_alias,             CMD_FLAG_NONE    },
 	{	"all",               do_all,               CMD_FLAG_SUB     },
 	{	"bell",              do_bell,              CMD_FLAG_NONE    },
-	{	"boss",              do_boss,              CMD_FLAG_NONE    },
-
 	{	"buffer",            do_buffer,            CMD_FLAG_SUB     },
+	{	"chat",              do_chat,              CMD_FLAG_SUB     },
 	{	"class",             do_class,             CMD_FLAG_SUB     },
 	{	"commands",          do_commands,          CMD_FLAG_NONE    },
 	{	"config",            do_configure,         CMD_FLAG_SUB     },
@@ -51,8 +50,6 @@ const struct command_type command_table[MAX_COMMAND] =
 	{	"function",          do_function,          CMD_FLAG_NONE    },
 	{	"gag",               do_gag,               CMD_FLAG_NONE    },
 	{	"gagline",           do_gagline,           CMD_FLAG_NONE    },
-	{	"getitemnr",         do_getitemnr,         CMD_FLAG_SUB     },
-	{	"getlistlength",     do_getlistlength,     CMD_FLAG_SUB     },
 	{	"grep",              do_grep,              CMD_FLAG_SUB     },
 	{	"help",              do_help,              CMD_FLAG_NONE    },
 	{	"highlight",         do_highlight,         CMD_FLAG_NONE    },
@@ -60,6 +57,7 @@ const struct command_type command_table[MAX_COMMAND] =
 	{	"ignore",            do_ignore,            CMD_FLAG_SUB     },
 	{	"info",              do_info,              CMD_FLAG_NONE    },
 	{	"killall",           do_killall,           CMD_FLAG_NONE    },
+	{	"list",              do_list,              CMD_FLAG_SUB     },
 	{	"loadpath",          do_loadpath,          CMD_FLAG_SUB     },
 	{	"log",               do_log,               CMD_FLAG_SUB     },
 	{	"logline",           do_logline,           CMD_FLAG_SUB     },
@@ -70,19 +68,19 @@ const struct command_type command_table[MAX_COMMAND] =
 	{	"mark",              do_mark,              CMD_FLAG_NONE    },
 	{	"message",           do_message,           CMD_FLAG_SUB     },
 	{	"nop",               do_nop,               CMD_FLAG_NONE    },
+	{    "parse",             do_parse,             CMD_FLAG_NONE    },
 	{	"path",              do_path,              CMD_FLAG_NONE    },
 	{	"pathdir",           do_pathdir,           CMD_FLAG_NONE    },
 	{	"prompt",            do_prompt,            CMD_FLAG_NONE    },
 	{	"read",              do_read,              CMD_FLAG_SUB     },
-	{	"readmap",           do_readmap,           CMD_FLAG_SUB     },
 	{	"redit",             do_redit,             CMD_FLAG_SUB     },
-	{	"removestring",      do_removestring,      CMD_FLAG_NONE    },
 	{	"replacestring",     do_replacestring,     CMD_FLAG_SUB     },
 	{	"return",            do_return,            CMD_FLAG_NONE    },
 	{	"savepath",          do_savepath,          CMD_FLAG_SUB     },
 	{	"scan",              do_scan,              CMD_FLAG_SUB     },
+	{	"send",              do_send,              CMD_FLAG_NONE    },
 	{	"session",           do_session,           CMD_FLAG_SUB     },
-	{	"showme",            do_showme,            CMD_FLAG_NONE    },
+	{	"showme",            do_showme,            CMD_FLAG_SUB     },
 	{	"snoop",             do_snoop,             CMD_FLAG_SUB     },
 	{	"split",             do_split,             CMD_FLAG_SUB     },
 	{	"substitute",        do_substitute,        CMD_FLAG_NONE    },
@@ -107,8 +105,6 @@ const struct command_type command_table[MAX_COMMAND] =
 	{	"variable",          do_variable,          CMD_FLAG_NONE    },
 	{	"walk",              do_walk,              CMD_FLAG_NONE    },
 	{	"write",             do_write,             CMD_FLAG_SUB     },
-	{	"writemap",          do_writemap,          CMD_FLAG_SUB     },
-	{	"writesession",      do_writesession,      CMD_FLAG_SUB     },
 	{	"zap",               do_zap,               CMD_FLAG_NONE    },
 	{	"",                  NULL,                 CMD_FLAG_NONE    }
 };
@@ -126,8 +122,8 @@ const struct list_type list_table[LIST_ALL] =
 	{
 		"ALIAS",
 		"ALIASES",
-		ALPHA,
-		2
+		PRIORITY,
+		3
 
 	},
 	{
@@ -217,7 +213,7 @@ const struct list_type list_table[LIST_ALL] =
 		"MATH",
 		"MATHEMATICS",
 		APPEND,
-		0
+		3
 	},
 
 	{
@@ -225,11 +221,18 @@ const struct list_type list_table[LIST_ALL] =
 		"DELAYS",
 		PRIORITY,
 		3
+	},
+
+	{
+		"HISTORY",
+		"HISTORIES",
+		APPEND,
+		1
 	}
 };
 
 
-const struct config_type config_table[CONFIG_MAX] =
+const struct config_type config_table[] =
 {
 	{
 		"SPEEDWALK",
@@ -347,14 +350,29 @@ const struct config_type config_table[CONFIG_MAX] =
 		"Tintin converts meta prefixed characters.",
 		"Tintin doesn't convert meta prefixed characters.",
 		config_convertmeta
+	},
+
+	{
+		"LOG LEVEL",
+		"Tintin only logs low level mud data.",
+		"Tintin only logs high level mud data.",
+		config_loglevel
+	},
+
+	{
+		"",
+		"",
+		0,
+		0
 	}
 };
 
-const struct color_type color_table[COLOR_MAX] =
+const struct color_type color_table[] =
 {
 	{	"reset",          "0", "<088>" },
 	{	"light",          "1", "<188>" },
-	{	"faint",          "2", "<288>" },
+	{	"bold",           "1", "<188>" },
+	{    "faint",          "2", "<288>" },
 	{	"underscore",     "4", "<488>" },
 	{	"blink",          "5", "<588>" },
 	{	"reverse",        "7", "<788>" },
@@ -405,3 +423,50 @@ const struct class_type class_table[CLASS_MAX] =
 		class_kill
 	}
 };
+
+const struct chat_type chat_table[] =
+{
+	{     "ACCEPT",                    chat_accept            },
+	{     "CALL",                      chat_call              },
+	{     "CANCELFILE",                chat_cancelfile        },
+	{     "COLOR",                     chat_color             },
+	{     "DECLINE",                   chat_decline           },
+	{     "DND",                       chat_dnd               },
+	{     "DOWNLOADDIR",               chat_downloaddir       },
+	{     "EMOTE",                     chat_emote             },
+	{     "FORWARD",                   chat_forward           },
+	{     "FORWARDALL",                chat_forwardall        },
+	{     "FILESTAT",                  chat_filestat          },
+	{     "GROUP",                     chat_group             },
+	{     "IGNORE",                    chat_ignore            },
+	{     "INITIALIZE",                chat_initialize        },
+	{     "INFO",                      chat_info              },
+	{     "IP",                        chat_ip                },
+	{     "MESSAGE",                   chat_message           },
+	{     "NAME",                      chat_name              },
+	{     "PASTE",                     chat_paste             },
+	{     "PEEK",                      chat_peek              },
+	{     "PING",                      chat_ping              },
+	{     "PRIVATE",                   chat_private           },
+	{     "PUBLIC",                    chat_public            },
+	{     "REPLY",                     chat_reply             },
+	{     "REQUEST",                   chat_request           },
+	{     "SEND",                      chat_send              },
+	{     "SENDFILE",                  chat_sendfile          },
+	{     "SERVE",                     chat_serve             },
+	{     "UNINITIALIZE",              chat_uninitialize      },
+	{     "WHO",                       chat_who               },
+	{     "ZAP",                       chat_zap               },
+	{     "",                          NULL                   }
+};
+
+
+const struct array_type array_table[ARRAY_MAX] =
+{
+	{     "DEL",                       array_del              },
+	{     "FND",                       array_fnd              },
+	{     "GET",                       array_get              },
+	{     "INS",                       array_ins              },
+	{     "LEN",                       array_len              },
+	{     "SET",                       array_set              },
+};
