@@ -44,6 +44,17 @@ int regexp(const char *exp, const char *str)
 			str += cnt;
 		}
 
+#ifdef BIG5
+		if (*exp & 0x80 && *str & 0x80)
+		{
+			if (*++exp != *++str)
+			{
+				return FALSE;
+			}
+			continue;
+		}
+#endif
+
 		switch (*exp)
 		{
 			case '?':
@@ -71,10 +82,12 @@ int regexp(const char *exp, const char *str)
 				break;
 
 			default:
+
 				if (*exp != *str)
 				{
 					return FALSE;
 				}
+
 				break;
 		}
 	}
@@ -89,18 +102,38 @@ int regexp(const char *exp, const char *str)
 void substitute(struct session *ses, const char *string, char *result, int flags)
 {
 	struct listnode *node;
-	char temp[BUFFER_SIZE], funcargs[BUFFER_SIZE], *pti, *pto, *ptv, *ptf;
+	char temp[BUFFER_SIZE], buf[BUFFER_SIZE], *pti, *pto, *ptt;
 	const char *pte;
-	short i, cnt;
+	int i, cnt, flags_neol = flags;
 
 	pti = (char *) string;
 	pto = (char *) result;
 
+	DEL_BIT(flags_neol, SUB_EOL);
+
 	while (TRUE)
 	{
+
+#ifdef BIG5
+		if (*pti & 0x80)
+		{
+			*pto++ = *pti++;
+			if (*pti)
+			{
+				*pto++ = *pti++;
+			}
+			continue;
+		}
+#endif
+
 		switch (*pti)
 		{
 			case '\0':
+				if (HAS_BIT(flags, SUB_EOL))
+				{
+					*pto++ = '\r';
+					*pto++ = '\n';
+				}
 				*pto = 0;
 				return;
 
@@ -109,14 +142,15 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 				{
 					if (pti[1] == DEFAULT_OPEN)
 					{
-						for (ptv = temp, i = 2 ; pti[i] && pti[i] != DEFAULT_CLOSE ; i++)
+						for (ptt = buf, i = 2 ; pti[i] && pti[i] != DEFAULT_CLOSE ; i++)
 						{
-							*ptv++ = pti[i];
+							*ptt++ = pti[i];
 						}
 
 						if (pti[i] == DEFAULT_CLOSE)
 						{
-							*ptv = 0;
+							*ptt = 0;
+							substitute(ses, buf, temp, flags_neol);
 						}
 						else
 						{
@@ -124,23 +158,23 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 						}
 					}
 
-					if (isalnum(pti[1]))
+					if (isalnum(pti[1]) || pti[1] == '_')
 					{
-						for (ptv = temp, i = 1 ; isalnum(pti[i]) ; i++)
+						for (ptt = temp, i = 1 ; isalnum(pti[i]) || pti[i] == '_' ; i++)
 						{
-							*ptv++ = pti[i];
+							*ptt++ = pti[i];
 						}
-						*ptv = 0;
+						*ptt = 0;
 					}
 
 					if ((node = searchnode_list(ses->list[LIST_VARIABLE], temp)) != NULL)
 					{
-						substitute(ses, node->right, pto, flags - SUB_VAR);
+						substitute(ses, node->right, pto, flags_neol - SUB_VAR);
 
 						if (pti[1] == DEFAULT_OPEN)
 						{
 							pto += strlen(pto);
-							pti += 3 + strlen(temp);
+							pti += 3 + strlen(buf);
 						}
 						else
 						{
@@ -214,11 +248,11 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 			case '@':
 				if (HAS_BIT(flags, SUB_FUN))
 				{
-					for (ptf = temp, i = 1 ; isalnum(pti[i]) ; i++)
+					for (ptt = temp, i = 1 ; isalnum(pti[i]) ; i++)
 					{
-						*ptf++ = pti[i];
+						*ptt++ = pti[i];
 					}
-					*ptf = 0;
+					*ptt = 0;
 
 					node = searchnode_list(ses->list[LIST_FUNCTION], temp);
 
@@ -229,22 +263,32 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 						continue;
 					}
 
-					pti = (char *) get_arg_in_braces(&pti[i], funcargs, FALSE);
-					pte = funcargs;
+					pti = (char *) get_arg_in_braces(&pti[i], temp, FALSE);
 
+					substitute(ses, temp, buf, flags_neol);
+
+					pte = buf;
+
+					if (HAS_BIT(ses->list[LIST_FUNCTION]->flags, LIST_FLAG_DEBUG))
+					{
+						tintin_printf2(ses, "#FUNCTION DEBUG: (%s) (%s)", node->left, temp);
+					}
 					for (i = 0 ; i < 10 ; i++)
 					{
-						pte = get_arg_in_braces(pte, temp, FALSE);
-
-						strcpy(gtd->cmds[i], temp);
+						pte = get_arg_in_braces(pte, gtd->cmds[i], FALSE);
 					}
 					substitute(ses, node->right, temp, SUB_CMD);
 
 					parse_input(temp, ses);
 
-					substitute(ses, "$result", temp, SUB_VAR);
-					substitute(ses, temp, pto, flags - SUB_FUN);
+					DEL_BIT(ses->flags, SES_FLAG_BREAK);
 
+					substitute(ses, "$result", pto, SUB_VAR);
+
+					if (HAS_BIT(ses->list[LIST_FUNCTION]->flags, LIST_FLAG_DEBUG))
+					{
+						tintin_printf2(ses, "#FUNCTION DEBUG: (%s) (%s)", node->left, pto);
+					}
 					pto += strlen(pto);
 				}
 				else
@@ -256,11 +300,24 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 			case '%':
 				if (HAS_BIT(flags, SUB_ARG) && isdigit(pti[1]))
 				{
-					for (cnt = 0 ; gtd->vars[pti[1] - '0'][cnt] ; cnt++)
+					ptt = gtd->vars[pti[1] - '0'];
+
+					while (*ptt)
 					{
+#ifdef BIG5
+						if (*ptt & 0x80)
+						{
+							*pto++ = *ptt++;
+							if (*ptt)
+							{
+								*pto++ = *ptt++;
+							}
+							continue;
+						}
+#endif
 						if (HAS_BIT(flags, SUB_SEC))
 						{
-							switch (gtd->vars[pti[1] - '0'][cnt])
+							switch (*ptt)
 							{
 								case ';':
 									*pto++ = '\\';
@@ -273,17 +330,23 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 									break;
 
 								default:
-									*pto++ = gtd->vars[pti[1] - '0'][cnt];
+									*pto++ = *ptt;
 									break;
 							}
+							ptt++;
 						}
 						else
 						{
-							*pto++ = gtd->vars[pti[1] - '0'][cnt];
+							*pto++ = *ptt++;
 						}
 					}
 					pti += 2;
 				}
+				else if (HAS_BIT(flags, SUB_FMT))
+				{
+					*pto++ = '%';
+					*pto++ = *pti++;
+				}
 				else
 				{
 					*pto++ = *pti++;
@@ -338,8 +401,11 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 						case 'a':
 							*pto++ = '\a';
 							break;
+						case 'b':
+							*pto++ = '\b';
+							break;
 						case 'e':
-							*pto++ = '\e';
+							*pto++ = '\033';
 							break;
 						case 'n':
 							*pto++ = '\n';
@@ -350,12 +416,24 @@ void substitute(struct session *ses, const char *string, char *result, int flags
 						case 't':
 							*pto++ = '\t';
 							break;
+						case 'x':
+							pti++;
+
+							if (pti[0] && pti[1])
+							{
+								*pto++ = hex_number(pti);
+								pti++;
+							}
+							break;
 						case '[':
 							*pto++ = '{';
 							break;
 						case ']':
 							*pto++ = '}';
 							break;
+						case '\0':
+							*pto = 0;
+							return;
 						default:
 							*pto++ = *pti;
 							break;
@@ -404,17 +482,17 @@ int check_one_action(const char *line, const char *original, const char *action,
 	if (*ptr == '^')
 	{
 		ptr++;
-		return action_regexp(ptr, ptl);
+		return action_regexp(ptr, ptl, 255);
 	}
 
 	if (ptr[0] == '%' && isdigit(ptr[1]))
 	{
-		return action_regexp(ptr, ptl);
+		return action_regexp(ptr, ptl, 255);
 	}
 
 	while (*ptl)
 	{
-		if (*ptr == *ptl && action_regexp(ptr, ptl))
+		if ((*ptr == '[' || *ptr == *ptl) && action_regexp(ptr, ptl, 255))
 		{
 			return TRUE;
 		}
@@ -424,20 +502,51 @@ int check_one_action(const char *line, const char *original, const char *action,
 }
 
 
-int action_regexp(const char *exp, const char *str)
+int action_regexp(const char *exp, const char *str, unsigned char arg)
 {
-	short cnt;
+	short cnt, cnt2;
+
+	for (cnt = 0 ; cnt < 10 ; cnt++)
+	{
+		gtd->vars[cnt][0] = 0;
+	}
+
+	thestart:
 
 	while (*exp)
 	{
+
+
+#ifdef BIG5
+		if (*exp & 0x80 && *str & 0x80)
+		{
+			exp++;
+			str++;
+			if (*exp++ != *str++)
+			{
+				return FALSE;
+			}
+			continue;
+		}
+#endif
+
 		switch (exp[0])
 		{
 			case '%':
 				if (exp[1] >= '0' && exp[1] <= '9')
 				{
-					for (cnt = strlen(str) ; cnt >= 0 ; cnt--)
+					arg = exp[1] - '0';
+
+					if (exp[2] == 0)
 					{
-						if (action_regexp(exp + 2, &str[cnt]))
+						strcpy(gtd->vars[exp[1] - '0'], str);
+
+						return TRUE;
+					}
+
+					for (cnt = 0 ; str[cnt] ; cnt++)
+					{
+						if (action_regexp(exp + 2, &str[cnt], arg))
 						{
 							memcpy(gtd->vars[exp[1] - '0'], str, cnt);
 							gtd->vars[exp[1] - '0'][cnt] = 0;
@@ -455,6 +564,68 @@ int action_regexp(const char *exp, const char *str)
 					return (exp[1] == str[0]);
 				}
 				break;
+
+			case '[':
+				if (*str != '[' && regexp("[*|*]*", exp))
+				{
+					cnt  = 1;
+					cnt2 = 0;
+
+					while (TRUE)
+					{
+						if (exp[cnt] == '|' || exp[cnt] == ']')
+						{
+							arg = (arg == 9) ? arg : arg + 1;
+							cnt--;
+							cnt2 = 0;
+
+							while (exp[cnt] != '|' && exp[cnt] != '[')
+							{
+								cnt--;
+								gtd->vars[arg][cnt2++] = *str++;
+							}
+							gtd->vars[arg][cnt2] = 0;
+
+							while (*exp != ']')
+							{
+								exp++;
+							}
+							exp++;
+
+							goto thestart;
+						}
+
+						if (str[cnt2] == 0)
+						{
+							break;
+						}
+
+						if (exp[cnt] == str[cnt2])
+						{
+							cnt++;
+							cnt2++;
+							continue;
+						}
+						else
+						{
+							cnt2 = 0;
+
+							while (exp[cnt])
+							{
+								if (exp[cnt] == ']')
+								{
+									return FALSE;
+								}
+								if (exp[cnt++] == '|')
+								{
+									break;
+								}
+
+							}
+						}
+					}
+				}
+				break;
 		}
 
 		if (*exp != *str)
