@@ -33,6 +33,7 @@
 #include <readline/readline.h>
 #include <readline/history.h>
 #include <ctype.h>
+#include <stdarg.h>
 
 /******************************************************************************
 *   Autoconf patching by David Hedbor, neotron@lysator.liu.se                 *
@@ -81,26 +82,29 @@
 	Typedefs
 */
 
+typedef struct listnode LNODE;
+
 typedef struct session *COMMAND (struct session *ses, const char *arg);
 typedef struct session *CONFIG  (struct session *ses, char *arg, int index);
-typedef struct session *CLASS   (struct session *ses, char *arg);
-
+typedef struct session *CLASS   (struct session *ses, char *left, char *right);
+typedef void CHAT (const char *arg);
+typedef struct session *ARRAY   (struct session *ses, LNODE *list, const char *arg);
 /*
 	A bunch of constants
 */
 
-#define	TRUE                    1
-#define	FALSE                   0
+#define TRUE                    1
+#define FALSE                   0
 
-#define	SCREEN_WIDTH            80
-#define	SCREEN_HEIGHT           24
+#define SCREEN_WIDTH            80
+#define SCREEN_HEIGHT           24
 
-#define	PRIORITY                0
-#define	ALPHA                   1
+#define PRIORITY                0
+#define ALPHA                   1
 #define APPEND                  2
 
-#define	DEFAULT_OPEN            '{'
-#define	DEFAULT_CLOSE           '}'
+#define DEFAULT_OPEN            '{'
+#define DEFAULT_CLOSE           '}'
 
 #define HISTORY_FILE              ".tt_history"
 
@@ -112,8 +116,6 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 #define DEFAULT_BZIP_EXT          ".bz2"
 #define DEFAULT_BUNZIP_STR        "bunzip2 -c "
 
-
-#define FILE_SIZE                150000
 #define BUFFER_SIZE               10000
 #define NUMBER_SIZE                 100
 
@@ -129,17 +131,7 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 
 #define ESCAPE                         27
 
-
-/*
-	Size of the telopt table
-*/
-
-#define TELOPT_MAX                    18
-
-#define TELNET_PORT                   23
-
-#define TELOPT_MCCP2                  86
-
+#define TELNET_PORT                    23
 
 /*
 	Index for lists used by tintin
@@ -164,18 +156,63 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 
 #define LIST_MATH                     LIST_MAX + 0
 #define LIST_DELAY                    LIST_MAX + 1
-#define LIST_ALL                      LIST_MAX + 2
-
-#define CONFIG_MAX                    17
-
-#define COLOR_MAX                     24
+#define LIST_HISTORY                  LIST_MAX + 2
+#define LIST_ALL                      LIST_MAX + 3
 
 #define CLASS_MAX                      5
 
+#define ARRAY_MAX                      6
+
 /*
 	Various flags
 */
 
+#define CHAT_NAME_CHANGE                 1
+#define CHAT_REQUEST_CONNECTIONS         2
+#define CHAT_CONNECTION_LIST             3
+#define CHAT_TEXT_EVERYBODY              4
+#define CHAT_TEXT_PERSONAL               5
+#define CHAT_TEXT_GROUP                  6
+#define CHAT_MESSAGE                     7
+#define CHAT_DO_NOT_DISTURB              8
+
+#define CHAT_SEND_ACTION                 9
+#define CHAT_SEND_ALIAS                 10
+#define CHAT_SEND_MACRO                 11
+#define CHAT_SEND_VARIABLE              12
+#define CHAT_SEND_EVENT                 13
+#define CHAT_SEND_GAG                   14
+#define CHAT_SEND_HIGHLIGHT             15
+#define CHAT_SEND_LIST                  16
+#define CHAT_SEND_ARRAY                 17
+#define CHAT_SEND_BARITEM               18
+
+#define CHAT_VERSION                    19
+#define CHAT_FILE_START                 20
+#define CHAT_FILE_DENY                  21
+#define CHAT_FILE_BLOCK_REQUEST         22
+#define CHAT_FILE_BLOCK                 23     
+#define CHAT_FILE_END                   24
+#define CHAT_FILE_CANCEL                25
+#define CHAT_PING_REQUEST               26 
+#define CHAT_PING_RESPONSE              27 
+#define CHAT_PEEK_CONNECTIONS           28
+#define CHAT_PEEK_LIST                  29
+#define CHAT_SNOOP_START                30
+#define CHAT_SNOOP_DATA                 31
+
+#define CHAT_END_OF_COMMAND            255
+
+#define CHAT_FLAG_PRIVATE             (1 <<  1)
+#define CHAT_FLAG_REQUEST             (1 <<  2)
+#define CHAT_FLAG_SERVE               (1 <<  3)
+#define CHAT_FLAG_IGNORE              (1 <<  4)
+#define CHAT_FLAG_FORWARD             (1 <<  5)
+#define CHAT_FLAG_FORWARDBY           (1 <<  6)
+#define CHAT_FLAG_FORWARDALL          (1 <<  7)
+#define CHAT_FLAG_DND                 (1 <<  8)
+#define CHAT_FLAG_LINKLOST            (1 <<  9)
+
 #define SUB_NONE                      0
 #define SUB_ARG                       (1 <<  0)
 #define SUB_VAR                       (1 <<  1)
@@ -185,11 +222,13 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 #define SUB_CMD                       (1 <<  5)
 #define SUB_ANC                       (1 <<  6)
 #define SUB_SEC                       (1 <<  7)
+#define SUB_EOL                       (1 <<  8)
+#define SUB_FMT                       (1 <<  9)
 
 #define SES_FLAG_NAWS                 (1 <<  0)
 #define SES_FLAG_ECHOCOMMAND          (1 <<  1)
 #define SES_FLAG_SNOOP                (1 <<  2)
-#define SES_FLAG_LOCALECHO            (1 <<  3)
+
 #define SES_FLAG_MAPPING              (1 <<  4)
 #define SES_FLAG_SPLIT                (1 <<  5)
 #define SES_FLAG_SPEEDWALK            (1 <<  6)
@@ -201,22 +240,37 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 #define SES_FLAG_REPEATENTER          (1 << 12)
 #define SES_FLAG_VERBOSE              (1 << 13)
 #define SES_FLAG_QUIET                (1 << 14)
-#define SES_FLAG_ANTISUB              (1 << 15)
+#define SES_FLAG_LOGLEVEL             (1 << 15)
 #define SES_FLAG_LOGPLAIN             (1 << 16)
 #define SES_FLAG_LOGHTML              (1 << 17)
 #define SES_FLAG_GAG                  (1 << 18)
-#define SES_FLAG_DEBUGTELNET          (1 << 19)
+
 #define SES_FLAG_USERCOMMAND          (1 << 20)
 #define SES_FLAG_SCROLLLOCK           (1 << 21)
 #define SES_FLAG_SCAN                 (1 << 22)
 #define SES_FLAG_SCROLLSTOP           (1 << 23)
 #define SES_FLAG_CONVERTMETA          (1 << 24)
-#define SES_FLAG_GAGPROMPT            (1 << 25)
-#define SES_FLAG_BREAK                (1 << 26)
+#define SES_FLAG_PREPPED              (1 << 25)
+
+
+#define SES_FLAG_BREAK                (1 << 28)
+
+#define TELOPT_FLAG_INIT_SGA          (1 <<  0)
+#define TELOPT_FLAG_SGA               (1 <<  1)
+#define TELOPT_FLAG_EOR               (1 <<  2)
+#define TELOPT_FLAG_ECHO              (1 <<  3)
+#define TELOPT_FLAG_NAWS              (1 <<  4)
+#define TELOPT_FLAG_PROMPT            (1 <<  5)
+#define TELOPT_FLAG_IAC               (1 <<  6)
+#define TELOPT_FLAG_DEBUG             (1 <<  7)
+#define TELOPT_FLAG_INIT_TSPEED       (1 <<  8)
+#define TELOPT_FLAG_INIT_TTYPE        (1 <<  9)
+#define TELOPT_FLAG_INIT_NAWS         (1 << 10)
 
 #define LIST_FLAG_IGNORE              (1 <<  0)
 #define LIST_FLAG_MESSAGE             (1 <<  1)
 #define LIST_FLAG_DEBUG               (1 <<  2)
+#define LIST_FLAG_LOG                 (1 <<  3)
 #define LIST_FLAG_DEFAULT             LIST_FLAG_MESSAGE
 
 #define NODE_FLAG_CLASS               (1 <<  0)
@@ -227,9 +281,6 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 #define CMD_FLAG_NONE                 (0 <<  0)
 #define CMD_FLAG_SUB                  (1 <<  0)
 
-
-#define MAX_COMMAND                   82
-
 #define MAX_STR_HASH                  5000
 
 #define MAX_ROOM                      5000
@@ -238,7 +289,7 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 	Some macros to deal with double linked lists
 */
 
-#define LINK(link, head, tail, next, prev)                       \
+#define LINK(link, head, tail)                                   \
 {                                                                \
      if ((head) == NULL)                                         \
      {                                                           \
@@ -254,7 +305,7 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 }
 
 
-#define INSERT_LEFT(link, rght, head, next, prev)                \
+#define INSERT_LEFT(link, rght, head)                            \
 {                                                                \
      (link)->prev                  = (rght)->prev;               \
      (rght)->prev                  = (link);                     \
@@ -271,7 +322,7 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 }
 
 
-#define INSERT_RIGHT(link, left, tail, next, prev)               \
+#define INSERT_RIGHT(link, left, tail)                           \
 {                                                                \
      (link)->next                  = (left)->next;               \
      (left)->next                  = (link);                     \
@@ -287,8 +338,14 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
      }                                                           \
 }
 
-#define UNLINK(link, head, tail, next, prev)                     \
+#define UNLINK(link, head, tail)                                 \
 {                                                                \
+     if (((link)->prev == NULL && (link) != head)                \
+     ||  ((link)->next == NULL && (link) != tail))               \
+     {                                                           \
+          tintin_printf2(NULL, "#UNLINK ERROR in file %s on line %d", __FILE__, __LINE__); \
+          dump_stack();                                          \
+     }                                                           \
      if ((link)->prev == NULL)                                   \
      {                                                           \
           (head)                  = (link)->next;                \
@@ -319,7 +376,12 @@ typedef struct session *CLASS   (struct session *ses, char *arg);
 
 #define DO_COMMAND(command) struct session  *command (struct session *ses, const char *arg)
 #define DO_CONFIG(config) struct session *config (struct session *ses, char *arg, int index)
-#define DO_CLASS(class) struct session *class (struct session *ses, char *arg)
+#define DO_CLASS(class) struct session *class (struct session *ses, char *left, char *right)
+#define DO_ARRAY(array) struct session *array (struct session *ses, LNODE *list, const char *arg)
+
+#define DO_CHAT(chat) void chat (const char *arg)
+
+
 
 /*
 	Bit operations
@@ -357,6 +419,7 @@ struct listnode
 	char              * right;
 	char              * pr;
 	long long           data;
+	short               flags;
 };
 
 struct session
@@ -371,6 +434,7 @@ struct session
 	FILE                  * logfile;
 	FILE                  * logline;
 	struct listroot       * list[LIST_ALL];
+	struct listroot       * history;
 	struct listnode       * class;
 	int                     rows;
 	int                     cols;
@@ -378,6 +442,8 @@ struct session
 	int                     bot_row;
 	int                     cur_row;
 	int                     sav_row;
+	int                     cur_col;
+	int                     sav_col;
 	int                     map_size;
 	int                     scroll_max;
 	int                     scroll_row;
@@ -387,6 +453,7 @@ struct session
 	int                     bgc;
 	int                     vtc;
 	int                     socket;
+	int                     telopts;
 	long long               flags;
 	char                  * host;
 	char                  * port;
@@ -400,6 +467,7 @@ struct tintin_data
 {
 	struct session        * ses;
 	struct session        * update;
+	struct chat_data      * chat;
 	char                  * mud_output_buf;
 	int                     mud_output_max;
 	int                     mud_output_len;
@@ -409,6 +477,7 @@ struct tintin_data
 	long long               time;
 	int                     str_hash_size;
 	int                     history_size;
+	int                     command_ref[26];
 	char                    tintin_char;
 	char                    verbatim_char;
 	char                    repeat_char;
@@ -416,6 +485,32 @@ struct tintin_data
 	char                    cmds[10][BUFFER_SIZE];
 };
 
+struct chat_data
+{
+	struct chat_data      * next;
+	struct chat_data      * prev;
+	char                  * name;
+	char                  * ip;
+	char                  * version;
+	char                  * download;
+	char                  * reply;
+	char                  * paste_buf;
+	char                  * color;
+	char                  * group;
+	int                     port;
+	int                     fd;
+	time_t                  timeout;
+	int                     flags;
+	long long               paste_time;
+	FILE                  * file_pt;
+	char                  * file_name;
+	long long               file_size;
+	int                     file_block_cnt;
+	int                     file_block_tot;
+	int                     file_block_patch;
+	long long               file_start_time;
+};
+
 struct command_type
 {
 	char                  * name;
@@ -453,13 +548,24 @@ struct class_type
 	CLASS                 * class;
 };
 
+struct chat_type
+{
+	char                  * name;
+	CHAT                  * chat;
+};
+
+struct array_type
+{
+	char                  * name;
+	ARRAY                 * array;
+};
 
 struct str_hash_data
 {
 	struct str_hash_data    * next;
 	struct str_hash_data    * prev;
 	unsigned short            count;
-        unsigned short            flags;
+	unsigned short            flags;
 	unsigned short            lines;
 	unsigned short            hash;
 };
@@ -504,11 +610,115 @@ extern int readline_echoing_p;
 	Function declarations
 */
 
+#ifndef __INPUT_H__
+#define __INPUT_H__
+
+extern void commandloop(void);
+extern char *readkeyboard(void);
+extern void echo_command(struct session *ses, char *line, int newline);
+
+#endif
+
+#ifndef __ARRAY_H__
+#define __ARRAY_H__
+
+extern DO_COMMAND(do_list);
+
+extern DO_ARRAY(array_ins);
+extern DO_ARRAY(array_del);
+extern DO_ARRAY(array_fnd);
+extern DO_ARRAY(array_get);
+extern DO_ARRAY(array_len);
+extern DO_ARRAY(array_set);
+
+#endif
+
+#ifndef __CHAT_H__
+#define __CHAT_H__
+
+extern DO_COMMAND(do_chat);
+extern DO_CHAT(chat_accept);
+extern DO_CHAT(chat_call);
+extern DO_CHAT(chat_cancelfile);
+extern DO_CHAT(chat_color);
+extern DO_CHAT(chat_decline);
+extern DO_CHAT(chat_dnd);
+extern DO_CHAT(chat_downloaddir);
+extern DO_CHAT(chat_emote);
+extern DO_CHAT(chat_filestat);
+extern DO_CHAT(chat_group);
+extern DO_CHAT(chat_forward);
+extern DO_CHAT(chat_forwardall);
+extern DO_CHAT(chat_ignore);
+extern DO_CHAT(chat_initialize);
+extern DO_CHAT(chat_info);
+extern DO_CHAT(chat_ip);
+extern DO_CHAT(chat_message);
+extern DO_CHAT(chat_name);
+extern DO_CHAT(chat_paste);
+extern DO_CHAT(chat_peek);
+extern DO_CHAT(chat_ping);
+extern DO_CHAT(chat_private);
+extern DO_CHAT(chat_public);
+extern DO_CHAT(chat_reply);
+extern DO_CHAT(chat_request);
+extern DO_CHAT(chat_send);
+extern DO_CHAT(chat_sendfile);
+extern DO_CHAT(chat_transfer);
+extern DO_CHAT(chat_serve);
+extern DO_CHAT(chat_uninitialize);
+extern DO_CHAT(chat_who);
+extern DO_CHAT(chat_zap);
+
+extern  int chat_new(int s);
+extern void chat_call(const char *ip);
+extern void close_chat(struct chat_data *buddy, int unlink);
+extern void process_chat_connections(fd_set *read_set, fd_set *write_set, fd_set *exc_set);
+extern void chat_forward_session(struct session *ses, char *linelog);
+extern void chat_socket_printf(struct chat_data *buddy, const char *format, ...);
+extern void chat_printf(const char *format, ...);
+extern  int process_chat_input(struct chat_data *buddy);
+extern void get_chat_commands(struct chat_data *buddy, char *buf, int len);
+extern void chat_name_change(struct chat_data *buddy, char *txt);
+extern void chat_receive_text_everybody(struct chat_data *buddy, char *txt);
+extern void chat_receive_text_personal(struct chat_data *buddy, char *txt);
+extern void chat_receive_text_group(struct chat_data *buddy, char *txt);
+extern void chat_receive_message(struct chat_data *buddy, char *txt);
+extern void chat_receive_snoop_data(struct chat_data *buddy, char *txt);
+
+extern void ping_response(struct chat_data *ch, char *time);
+
+extern void request_response(struct chat_data *requester);
+extern void parse_requested_connections(struct chat_data *buddy, char *txt);
+
+extern void peek_response(struct chat_data *peeker);
+extern void parse_peeked_connections(struct chat_data *buddy, char *txt);
+
+
+extern void chat_receive_file(char *arg, struct chat_data *ch);
+extern void send_block(struct chat_data *ch);
+extern void receive_block(unsigned char *s, struct chat_data *ch, int size);
+extern void deny_file(struct chat_data *ch, char *arg);
+extern void file_denied(struct chat_data *ch, char *txt);
+extern void file_cleanup(struct chat_data *buddy);
+
+extern int get_file_size(char *fpath);
+
+extern void chat_puts(char *arg);
+extern struct chat_data *find_buddy(const char *arg);
+extern struct chat_data *find_group(const char *arg);
+extern char *fix_file_name(char *name);
+
+
+
+#endif
+ 
 #ifndef __ECHO_H__
 #define __ECHO_H__
 
 extern void echo_on(struct session *ses);
 extern void echo_off(struct session *ses);
+extern void check_character_mode(struct session *ses);
 
 #endif
 
@@ -516,7 +726,8 @@ extern void echo_off(struct session *ses);
 #define __CLASS_H__
 
 extern DO_COMMAND(do_class);
-extern DO_COMMAND(do_unclass);
+extern int count_class(struct session *ses, struct listnode *class);
+extern void kill_classes(struct session *ses);
 extern DO_CLASS(class_open);
 extern DO_CLASS(class_close);
 extern DO_CLASS(class_read);
@@ -552,13 +763,16 @@ extern void search_coord(int vnum, short x, short y, short z);
 
 extern DO_COMMAND(do_math);
 extern DO_COMMAND(do_if);
-extern long long mathexp(struct session *ses, const char *str);
+extern double get_number(struct session *ses, const char *str);
+extern char *get_number_string(struct session *ses, const char *str);
+extern double mathexp(struct session *ses, const char *str);
 extern int mathexp_tokenize(struct session *ses, const char *str);
 extern void mathexp_level(struct listnode *node);
 extern void mathexp_compute(struct listnode *node);
-extern long long tintoi(const char *str);
-extern long long tincmp(const char *left, const char *right);
-extern long long tindice(const char *left, const char *right);
+extern double tintoi(const char *str);
+extern double tincmp(const char *left, const char *right);
+extern double tineval(const char *left, const char *right);
+extern double tindice(const char *left, const char *right);
 
 #endif
 
@@ -584,7 +798,7 @@ extern void do_one_prompt(struct session *ses, char *prompt, int row);
 extern int regexp(const char *exp, const char *str);
 extern void substitute(struct session *ses, const char *string, char *result, int flags);
 extern int check_one_action(const char *line, const char *original, const char *action, struct session *ses);
-extern int action_regexp(const char *exp, const char *str);
+extern int action_regexp(const char *exp, const char *str, unsigned char arg);
 
 
 #endif
@@ -611,6 +825,7 @@ extern DO_CONFIG(config_verbatimchar);
 extern DO_CONFIG(config_repeatchar);
 extern DO_CONFIG(config_debugtelnet);
 extern DO_CONFIG(config_convertmeta);
+extern DO_CONFIG(config_loglevel);
 
 #endif
 
@@ -648,12 +863,15 @@ extern void buffer_u(void);
 extern void buffer_d(void);
 extern void buffer_h(void);
 extern void buffer_e(void);
+extern void buffer_l(void);
+extern void buffer_f(const char *arg);
+
 #endif
 
 #ifndef __DEBUG_H__
 #define __DEBUG_H__
 
-extern void push_call(char *f, ...);
+extern int push_call(char *f, ...);
 extern void pop_call(void);
 extern void dump_stack(void);
 
@@ -674,6 +892,7 @@ extern void check_all_actions(const char *original, char *line, struct session *
 
 extern struct session *do_alias(struct session *ses, const char *arg);
 extern struct session *do_unalias(struct session *ses, const char *arg);
+extern int check_all_aliases(char *original, char *line, struct session *ses);
 
 #endif
 
@@ -691,9 +910,7 @@ extern int check_all_antisubstitutions(const char *original, char *line, struct
 #define __FILES_H__
 
 extern DO_COMMAND(do_read);
-extern struct session *readfile(struct session *ses, const char *arg, struct listnode *class);
 extern DO_COMMAND(do_readmap);
-extern DO_COMMAND(do_writesession);
 extern DO_COMMAND(do_write);
 extern DO_COMMAND(do_writemap);
 extern DO_COMMAND(do_session);
@@ -751,6 +968,9 @@ extern int show_node_with_wild(struct session *ses, const char *cptr, int index)
 extern struct listnode *search_node_with_wild(struct listroot *listhead, const char *cptr);
 extern void addnode_list(struct listroot *listhead, const char *ltext, const char *rtext, const char *prtext);
 extern int count_list(struct listroot *listhead);
+extern DO_COMMAND(do_message);
+extern DO_COMMAND(do_ignore);
+extern DO_COMMAND(do_debug);
 #endif
 
 
@@ -774,7 +994,6 @@ extern struct tintin_data *gtd;
 
 extern void trap_handler(int signal);
 extern void winchhandler(int no_care);
-extern void tstphandler(int no_care);
 extern int main(int argc, char **argv);
 extern void init_tintin(void);
 
@@ -783,26 +1002,26 @@ extern void init_tintin(void);
 #ifndef __MISC_H__
 #define __MISC_H__
 
+extern DO_COMMAND(do_all);
+extern DO_COMMAND(do_bell);
 extern DO_COMMAND(do_commands);
 extern DO_COMMAND(do_cr);
-extern DO_COMMAND(do_nop);
-extern DO_COMMAND(do_suspend);
-extern struct session *do_all(struct session *ses, const char *arg);
-extern DO_COMMAND(do_bell);
-extern DO_COMMAND(do_boss);
 extern DO_COMMAND(do_echo);
 extern DO_COMMAND(do_end);
-extern DO_COMMAND(do_showme);
-extern DO_COMMAND(do_loop);
 extern DO_COMMAND(do_forall);
-extern DO_COMMAND(do_message);
-extern DO_COMMAND(do_ignore);
-extern DO_COMMAND(do_debug);
+extern DO_COMMAND(do_info);
+extern DO_COMMAND(do_loop);
+extern DO_COMMAND(do_nop);
+extern DO_COMMAND(do_parse);
 extern DO_COMMAND(do_return);
+extern DO_COMMAND(do_run);
+extern DO_COMMAND(do_send);
+extern DO_COMMAND(do_showme);
 extern DO_COMMAND(do_snoop);
+extern DO_COMMAND(do_suspend);
 extern DO_COMMAND(do_system);
 extern DO_COMMAND(do_zap);
-extern DO_COMMAND(do_info);
+
 extern DO_COMMAND(do_gagline);
 #endif
 
@@ -819,35 +1038,45 @@ extern void read_buffer_mud(struct session *ses);
 #define __TELOPT_H__
 
 extern void translate_telopts(struct session *ses, unsigned char *src, int cplen);
-extern void send_will_sga(struct session *ses);
-extern void send_will_ttype(struct session *ses);
-extern void send_will_tspeed(struct session *ses);
-extern void send_will_naws(struct session *ses);
-extern void send_wont_xdisploc(struct session *ses);
-extern void send_wont_new_environ(struct session *ses);
-extern void send_sb_naws(struct session *ses);
-extern void send_wont_lflow(struct session *ses);
-extern void send_sb_tspeed(struct session *ses);
-extern void send_sb_ttype(struct session *ses);
-extern void send_wont_status(struct session *ses);
-extern void send_dont_status(struct session *ses);
-extern void send_do_sga(struct session *ses);
-extern void send_wont_oldenviron(struct session *ses);
-extern void send_echo_on(struct session *ses);
-extern void send_echo_off(struct session *ses);
-extern void send_echo_will(struct session *ses);
-extern void send_do_mccp2(struct session *ses);
-extern void send_dont_mccp2(struct session *ses);
-extern void init_mccp(struct session *ses);
+
+extern int send_will_sga(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_do_eor(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int mark_prompt(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_will_ttype(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_will_tspeed(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_will_naws(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_wont_xdisploc(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_wont_new_environ(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_sb_naws(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_wont_lflow(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_sb_tspeed(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_sb_ttype(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_wont_status(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_dont_status(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_dont_sga(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_do_sga(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_wont_oldenviron(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_echo_on(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_echo_off(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_echo_will(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_ip(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_wont_telopt(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_dont_telopt(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int exec_zmp(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_do_mccp2(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int send_dont_mccp2(struct session *ses, int cplen, unsigned char *cpsrc);
+extern int init_mccp(struct session *ses, int cplen, unsigned char *cpsrc);
 extern void *zlib_alloc(void *opaque, unsigned int items, unsigned int size);
 extern void zlib_free(void *opaque, void *address);
 
+extern void init_telnet_session(struct session *ses);
+
 #endif
 
 #ifndef __PARSE_H__
 #define __PARSE_H__
 
-extern struct session *parse_input(char *input, struct session *ses);
+extern struct session *parse_input(const char *input, struct session *ses);
 extern int is_speedwalk_dirs(const char *cp);
 extern void process_speedwalk(const char *cp, struct session *ses);
 extern struct session *parse_tintin_command(const char *command, char *arg, struct session *ses);
@@ -856,7 +1085,7 @@ extern const char *get_arg_with_spaces(const char *s, char *arg);
 extern const char *get_arg_in_braces(const char *s, char *arg, int flag);
 extern const char *get_arg_stop_spaces(const char *s, char *arg);
 extern const char *space_out(const char *s);
-extern void write_com_arg_mud(const char *command, const char *argument, struct session *ses);
+extern void write_mud(struct session *ses, const char *command);
 extern void do_one_line(char *line, struct session *ses);
 
 #endif
@@ -882,14 +1111,11 @@ extern void check_insert_path(const char *command, struct session *ses);
 
 extern void printline(struct session *ses, const char *str, int isaprompt);
 extern void initrl(void);
-extern void commandloop(void);
 extern void mainloop(void);
 extern void readmud(struct session *ses);
 extern void bait(void);
 extern void process_mud_output(struct session *ses, char *linebuf, int prompt);
-extern char *readkeyboard(void);
-extern void echo_command(struct session *ses, char *line, int force);
-extern int  show_message(struct session *ses, int index);
+extern void show_message(struct session *ses, int index, const char *format, ...);
 extern void tintin_header(struct session *ses, const char *format, ...);
 extern void socket_printf(struct session *ses, size_t length, const char *format, ...);
 extern void tintin_printf2(struct session *ses, const char *format, ...);
@@ -937,8 +1163,7 @@ extern void strip_vt102_codes(const char *str, char *buf);
 extern void strip_vt102_codes_non_graph(const char *str, char *buf);
 extern void strip_non_vt102_codes(const char *str, char *buf);
 extern int strip_vt102_strlen(const char *str);
-extern void interpret_vt102_codes(struct session *ses, const char *str);
-
+extern int interpret_vt102_codes(struct session *ses, const char *str, int real);
 
 #endif
 
@@ -953,7 +1178,7 @@ extern int get_scroll_size(struct session *ses);
 #ifndef __SESSION_H__
 #define __SESSION_H__
 
-extern void init_telnet_session(struct session *ses);
+
 extern struct session *session_command(const char *arg, struct session *ses);
 extern void show_session(struct session *ses, struct session *ptr);
 extern struct session *newactive_session(void);
@@ -980,11 +1205,13 @@ extern void check_all_substitutions(char *original, char *line, struct session *
 #define __TABLES_H__
 
 
-extern const struct command_type command_table[MAX_COMMAND];
+extern const struct command_type command_table[];
 extern const struct list_type list_table[LIST_ALL];
-extern const struct config_type config_table[CONFIG_MAX];
-extern const struct color_type color_table[COLOR_MAX];
+extern const struct config_type config_table[];
+extern const struct color_type color_table[];
 extern const struct class_type class_table[CLASS_MAX];
+extern const struct chat_type chat_table[];
+extern const struct array_type array_table[ARRAY_MAX];
 
 #endif
 
@@ -1010,9 +1237,12 @@ extern void tick_update(void);
 #define __UTILS_H__
 
 extern int is_abbrev(const char *s1, const char *s2);
+extern int is_suffix(const char *s1, const char *s2);
 extern int is_number(const char *str);
+extern int hex_number(const char *str);
 extern long long utime(void);
 extern char *capitalize(const char *str);
+extern void cat_sprintf(char *dest, const char *fmt, ...);
 extern void syserr(const char *msg);
 
 #endif
@@ -1024,14 +1254,11 @@ extern DO_COMMAND(do_variable);
 extern DO_COMMAND(do_internal_variable);
 extern DO_COMMAND(do_unvariable);
 
-extern DO_COMMAND(do_getlistlength);
-extern DO_COMMAND(do_getitemnr);
 extern DO_COMMAND(do_format);
 extern DO_COMMAND(do_tolower);
 extern DO_COMMAND(do_toupper);
 extern DO_COMMAND(do_postpad);
 extern DO_COMMAND(do_prepad);
-extern DO_COMMAND(do_removestring);
 extern DO_COMMAND(do_replacestring);
 
 #endif
