@@ -47,6 +47,15 @@ int is_abbrev(const char *s1, const char *s2)
 	return(!strncasecmp(s2, s1, strlen(s1)));
 }
 
+int is_suffix(const char *s1, const char *s2)
+{
+	int sl1, sl2;
+
+	sl1 = strlen(s1);
+	sl2 = strlen(s2);
+
+	return (sl1 <= sl2 && !strcasecmp(s1, s2 + sl2 - sl1)) ? FALSE : TRUE;
+}
 /*
 	return TRUE if the string is a number
 */
@@ -65,6 +74,38 @@ int is_number(const char *str)
 	return TRUE;
 }
 
+int hex_number(const char *str)
+{
+	int value = '?';
+
+	if (str)
+	{
+		if (isdigit(*str))
+		{
+			value = (*str - '0') << 4;
+		}
+		else
+		{
+			value = (*str - 'A' + 10) << 4;
+		}
+		str++;
+	}
+
+	if (str)
+	{
+		if (isdigit(*str))
+		{
+			value += *str - '0';
+		}
+		else
+		{
+			value += *str - 'A' + 10;
+		}
+		str++;
+	}
+
+	return value;
+}
 
 long long utime()
 {
@@ -98,6 +139,18 @@ char *capitalize(const char *str)
 	return outbuf;
 }
 
+void cat_sprintf(char *dest, const char *fmt, ...)
+{
+	char buf[BUFFER_SIZE];
+
+	va_list args;
+
+	va_start(args, fmt);
+	vsprintf(buf, fmt, args);
+	va_end(args);
+
+	strcat(dest, buf);
+}
 
 /*************************************************/
 /* print system call error message and terminate */
