@@ -51,20 +51,14 @@ DO_COMMAND(do_variable)
 	{
 		if (show_node_with_wild(ses, left, LIST_VARIABLE) == FALSE)
 		{
-			if (show_message(ses, LIST_VARIABLE))
-			{
-				tintin_puts2("#THAT VARIABLE IS NOT DEFINED.", ses);
-			}
+			show_message(ses, LIST_VARIABLE, "#VARIABLE: NO MATCH(ES) FOUND FOR {%s}.", left);
 		}
 	}
 	else
 	{
 		updatenode_list(ses, left, right, "0", LIST_VARIABLE);
 
-		if (show_message(ses, LIST_VARIABLE))
-		{
-			tintin_printf2(ses, "#Ok. $%s is now set to {%s}.", left, right);
-		}
+		show_message(ses, LIST_VARIABLE, "#OK. $%s IS NOW SET TO {%s}.", left, right);
 	}
 	return ses;
 }
@@ -94,9 +88,9 @@ DO_COMMAND(do_internal_variable)
 DO_COMMAND(do_unvariable)
 {
 	char left[BUFFER_SIZE];
-	int flag = FALSE;
 	struct listroot *root;
 	struct listnode *node;
+	int found = FALSE;
 
 	root = ses->list[LIST_VARIABLE];
 
@@ -104,132 +98,19 @@ DO_COMMAND(do_unvariable)
 
 	while ((node = search_node_with_wild(root, left)) != NULL)
 	{
-		if (show_message(ses, LIST_VARIABLE))
-		{
-			tintin_printf2(ses, "#Ok. $%s is no longer a variable.", node->left);
-		}
+		show_message(ses, LIST_VARIABLE, "#OK. $%s IS NO LONGER A VARIABLE.", node->left);
+
 		deletenode_list(ses, node, LIST_VARIABLE);
-		flag = TRUE;
+
+		found = TRUE;
 	}
-	if (!flag && show_message(ses, LIST_VARIABLE))
+	if (found == FALSE)
 	{
-		tintin_puts2("#THAT VARIABLE IS NOT DEFINED.", ses);
+		show_message(ses, LIST_VARIABLE, "#UNVARIABLE: NO MATCH(ES) FOUND FOR {%s}.", ses);
 	}
 	return ses;
 }
 
-/********************************************************/
-/* the #getlistlength command * By Sverre Normann       */
-/********************************************************/
-/* Syntax: #getlistlength {destination variable} {list} */
-/*****************************************************************/
-/* Note: This will return the number of items in the list.       */
-/*       An item is either a word, or grouped words in brackets. */
-/*       Ex:  #getl {listlength} {smile {say Hi!} flip bounce}   */
-/*            -> listlength = 4                                  */
-/*****************************************************************/
-
-DO_COMMAND(do_getlistlength)
-{
-	char left[BUFFER_SIZE], list[BUFFER_SIZE], result[BUFFER_SIZE];
-	int i = 0;
-
-	arg = get_arg_in_braces(arg, left, FALSE);
-	arg = get_arg_in_braces(arg, list, TRUE);
-
-	if (*left == 0)
-	{
-		tintin_puts2("#Error - Syntax: #getlistlength {dest var} {list}", ses);
-	}
-	else
-	{
-		for (arg = list ; *arg ; arg = get_arg_in_braces(arg, result, FALSE))
-		{
-			i++;
-		}
-		sprintf(result, "{%s} {%d}", left, i);
-
-		do_internal_variable(ses, result);
-	}
-	return ses;
-}
-
-
-/******************************************************************/
-/* the #getitemnr command * By Sverre Normann                     */  
-/******************************************************************/
-/* Syntax: #getitemnr {destination variable} {item number} {list} */
-/******************************************************************/
-/* Note: This will return a specified item from a list.           */
-/*       An item is either a word, or grouped words in brackets.  */
-/*       Ex:  #geti {dothis} {2} {smile {say Hi!} flip bounce}    */
-/*            -> dothis = say Hi!                                 */
-/******************************************************************/
-
-DO_COMMAND(do_getitemnr)
-{
-	char destvar[BUFFER_SIZE], itemval[BUFFER_SIZE], list[BUFFER_SIZE], temp[BUFFER_SIZE];
-	int i;
-
-	arg = get_arg_in_braces(arg, destvar, FALSE);
-	arg = get_arg_in_braces(arg, itemval, FALSE);
-	arg = get_arg_in_braces(arg, list,    TRUE);
-
-	if (*destvar == 0 || *itemval == 0)
-	{
-		tintin_puts2("#SYNTAX: #GETITEMNR: {destination variable} {item number} {list}", ses);
-	}
-	else
-	{
-		if (!is_number(itemval))
-		{
-			tintin_printf2(ses, "#GETITEMNR: ITEM NUMBER {%s} IS NOT VALID.", itemval);
-		}
-		else
-		{
-			arg = list;
-
-			for (i = temp[0] = 0 ; i < atoi(itemval) ; i++)
-			{
-				arg = get_arg_in_braces(arg, temp, FALSE);
-
-				if (*temp == 0)
-				{
-					break;
-				}
-			}
-
-			if (*temp == 0)
-			{
-				tintin_printf2(ses, "#GETITEMNR: ITEM {%d} DOES NOT EXIST.", atoi(itemval));
-			}
-			else
-			{
-				sprintf(list, "{%s} {%s}", destvar, temp);
-
-				do_internal_variable(ses, list);
-			}
-		}
-	}
-	return ses;
-}
-
-DO_COMMAND(do_removestring)
-{
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], result[BUFFER_SIZE];
-
-	arg = get_arg_in_braces(arg, left,  FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
-
-	sprintf(result, "{%s} {%s}", left, right);
-
-	tintin_printf2(ses, "#OLD COMMAND, REPLACING WITH: #REPLACESTRING %s", result);
-
-	do_replacestring(ses, result);
-
-	return ses;
-}
-
 
 DO_COMMAND(do_replacestring)
 {
@@ -245,24 +126,15 @@ DO_COMMAND(do_replacestring)
 
 	if (*var == 0 || *old == 0)
 	{
-		if (show_message(ses, LIST_VARIABLE))
-		{
-			tintin_puts2("#Syntax: #replacestring <var> <oldtext> <newtext>", ses);
-		}
+		show_message(ses, LIST_VARIABLE, "#Syntax: #replacestring <var> <oldtext> <newtext>", ses);
 	}
 	else	if ((node = searchnode_list(root, var)) == NULL)
 	{
-		if (show_message(ses, LIST_VARIABLE))
-		{
-			tintin_puts2("#THAT VARIABLE IS NOT DEFINED.", ses);
-		}
+		show_message(ses, LIST_VARIABLE, "#REPLACESTRING: VARIABLE {%s} NOT FOUND.", var);
 	}
 	else if ((ptr = strstr(node->right, old)) == NULL)
 	{
-		if (show_message(ses, LIST_VARIABLE))
-		{
-			tintin_printf2(ses, "#REPLACESTRING {%s} NOT FOUND IN {%s}", old, node->right);
-		}
+		show_message(ses, LIST_VARIABLE, "#REPLACESTRING: {%s} NOT FOUND IN {%s}", old, node->right);
 	}
 	else
 	{
@@ -291,10 +163,8 @@ DO_COMMAND(do_replacestring)
 
 		sprintf(result, "{%s} {%s}", node->left, var);
 
-		if (show_message(ses, LIST_VARIABLE))
-		{
-			tintin_printf2(ses, "#REPLACESTRING: $%s is now set to {%s}", node->left, var);
-		}
+		show_message(ses, LIST_VARIABLE, "#REPLACESTRING: $%s IS NOW SET TO {%s}", node->left, var);
+
 		do_internal_variable(ses, result);
 	}
 	return ses;
@@ -380,11 +250,7 @@ void reversestring(char *str)
 
 void mathstring(struct session *ses, char *str)
 {
-	char result[BUFFER_SIZE];
-
-	sprintf(result, "%lld", mathexp(ses, str));
-
-	strcpy(str, result);
+	strcpy(str, get_number_string(ses, str));
 }
 
 void thousandgroupingstring(char *str)
@@ -415,6 +281,30 @@ void thousandgroupingstring(char *str)
 	strcpy(str, result);
 }
 
+
+void stripspaces(char *str)
+{
+	int cnt;
+
+	for (cnt = strlen(str) - 1 ; cnt >= 0 ; cnt--)
+	{
+		if (!isspace(str[cnt]))
+		{
+			break;
+		}
+		str[cnt] = 0;
+	}
+
+	for (cnt = 0 ; str[cnt] != 0 ; cnt++)
+	{
+		if (!isspace(str[cnt]))
+		{
+			break;
+		}
+	}
+	strcpy(str, &str[cnt]);
+}
+
 /******************************************************************************
 * #format command - Scandum                                                   *
 *******************************************************************************
@@ -483,6 +373,10 @@ DO_COMMAND(do_format)
 
 				switch (*ptf)
 				{
+					case 'a':
+						sprintf(arglist[i], "%c", atoi(arglist[i]));
+						break;
+
 					case 'c':
 						colorstring(arglist[i]);
 						break;
@@ -509,6 +403,10 @@ DO_COMMAND(do_format)
 						arglist[i][0] = toupper(arglist[i][0]);
 						break;
 
+					case 'p':
+						stripspaces(arglist[i]);
+						break;
+
 					case 'r':
 						reversestring(arglist[i]);
 						break;
@@ -531,6 +429,12 @@ DO_COMMAND(do_format)
 						sprintf(arglist[i], "%d", ses->cols);
 						break;
 
+					case 'D':
+						timeval_t  = (time_t) atoi(arglist[i]);
+						timeval_tm = *localtime(&timeval_t);
+						strftime(arglist[i], BUFFER_SIZE, "%d", &timeval_tm);
+						break;
+
 					case 'G':
 						thousandgroupingstring(arglist[i]);
 						break;
@@ -539,6 +443,12 @@ DO_COMMAND(do_format)
 						sprintf(arglist[i], "%d", (int) strlen(arglist[i]));
 						break;
 
+					case 'M':
+						timeval_t  = (time_t) atoi(arglist[i]);
+						timeval_tm = *localtime(&timeval_t);
+						strftime(arglist[i], BUFFER_SIZE, "%m", &timeval_tm);
+						break;
+
 					case 'R':
 						sprintf(arglist[i], "%d", ses->rows);
 						break;
@@ -551,11 +461,14 @@ DO_COMMAND(do_format)
 						sprintf(arglist[i], "%lld", utime());
 						break;
 
+					case 'Y':
+						timeval_t  = (time_t) atoi(arglist[i]);
+						timeval_tm = *localtime(&timeval_t);
+						strftime(arglist[i], BUFFER_SIZE, "%Y", &timeval_tm);
+						break;
+
 					default:
-						if (show_message(ses, LIST_VARIABLE))
-						{
-							tintin_printf2(ses, "#FORMAT: UNKNOWN ARGUMENT {%%%c}", *ptf);
-						}
+						show_message(ses, LIST_VARIABLE, "#FORMAT: UNKNOWN ARGUMENT {%%%c}", *ptf);
 						break;
 				}
 				*ptf = 's';
@@ -568,10 +481,7 @@ DO_COMMAND(do_format)
 
 	sprintf(format, "{%s} {%s}", destvar, argument);
 
-	if (show_message(ses, LIST_VARIABLE))
-	{
-		tintin_printf2(ses, "#FORMAT: $%s is now set to {%s}", destvar, argument);
-	}
+	show_message(ses, LIST_VARIABLE, "#FORMAT: $%s IS NOW SET TO {%s}", destvar, argument);
 
 	do_internal_variable(ses, format);
 
